/** 
* @file   studio_setup_structs.h
* @author Bernd Ottmann
* @date   15.02.2002
* @brief  Auflistung der Standard- definitionen und Strukturen
*/
#pragma once
#ifndef STUDIO_SETUP_STRUCTS_H
#define STUDIO_SETUP_STRUCTS_H

#ifdef _MSC_VER
#pragma warning(disable : 4996)
#endif

#pragma pack(push, 1)

/////////////////////////////////////////////////////////////////////
// Angaben ueber verwendete Speichergroessen / Startadressen

#define EEP_SIZE             0x1000   ///< Gesamtgroesse vom EEPROM des Mikrokontroller (4kByte) 
#define EEP_SIZE_SU          0x0200   ///< Groesse des EEPROM die in das Geraet uebertragen wird (512Byte) 
#define EEP_SIZE_PT          0x2C     ///< Reservierter EEPROM - Bereich fuer Pointereintraege (ersten 44Byte)
#define FLASH_SIZE_SU        0x4000   ///< Groesse des FLASH - Bereichs fuer Setupdaten (16kByte) 
#define FLASH_SIZE_SU2       0x8000   ///< Groesse des FLASH - Erweiterung des Bereichs fuer Setupdaten (32kByte) 
#define EXTENDED_SIZE_SU     0x4000   ///< Groesse des Erweiterten FLASH - Bereichs fuer Setupdaten (16kByte) 
#define FLASH_SIZE           (FLASH_SIZE_SU + FLASH_SIZE_SU2 + EXTENDED_SIZE_SU)
#define CONFIG_SIZE_SU       0x64     ///< Groesse des Speicherbereichs fuer Grunddaten des Geraets (100Byte)
#define FLASH_KONFIG_START   0x0FC000 ///< Startadresse, Absoluter Addressoffset fuer alle Zeigerwerte.
                                      ///< Der absolute Speicheroffset, wurde ursprünglich Aufaddiert um sich dieses in der Firmware zu ersparen.
#define FLASH_LIST_START     0x0E0000 ///< Startadresse des Listenspeichers, es sind somit 100kByte Gesamtspeicher verfuegbar.
#define FLASH_LIST_START_NEW 0x180000 ///< Startadresse des Listenspeichers, ab 04.00.23 Listengroesse auf 244kByte erhoeht. 
#define FLASH_SIZE_LIST      0x018000 ///< Groesse des Listenspeichers (98kByte) 
#define FLASH_SIZE_LIST_NEW  0x03D000 ///< Groesse des Listenspeichers ab 04.00.23 (ComVersion 5.2) auf 244kByte erhoeht.
#define MAX_FLASH_MEM        0x100000 ///< Groesse des eingesetzten Flashspeichers

/////////////////////////////////////////////////////////////////////
// Automatische Tastenumschaltung im PZE Modus 2

// Aufbau des Datensatztarray
// ++++++++++++++++++++++++++
// 4 Byte Verwaltung (Version etc. Derzeit immer 0)
// 15 * 4 Byte Datensaetze wobei,
//      erstes  Byte: Funktion
//                    0: Nicht gewaehlt
//                  250: Hauptmenue
//                  251: Letzte Taste
//                  252: Letzte Tasten F1, F2
//                 1..6: Funktionstaste F1 .. F6
//      zweites Byte: Stunde
//      drittes Byte: Minute
//      viertes Byte: Wochentag
//                    Bit 0 .. Bit 6: Montag .. Sonntag

enum class EnumAutoKeyFunction : unsigned char
{
    None            = 0,
    Key_F1          = 1,
    Key_F2          = 2,
    Key_F3          = 3,
    Key_F4          = 4,
    Key_F5          = 5,
    Key_F6          = 6,
    Key_F7          = 7,
    Key_F8          = 8,
    Key_F9          = 9,
    Key_F10        = 10,
    Key_F11        = 11,
    Key_F12        = 12,
    Key_F13        = 13,
    Key_F14        = 14,
    Key_F15        = 15,
    MainMenu      = 250,
    LastKey       = 251,
    LastKey_F1_F2 = 252
};

enum class EnumAutoKeyWeekday : unsigned char
{
    Monday      = 0x01,
    Tuesday     = 0x02,
    Wednesday   = 0x04,
    Thursday    = 0x08,
    Friday      = 0x10,
    Saturday    = 0x20,
    Sunday      = 0x40
};

struct StructAutoKey
{
    EnumAutoKeyFunction function;
    unsigned char hour;
    unsigned char minute;
    EnumAutoKeyWeekday weekdays;
};

struct StructAutoKeys
{
    unsigned int version;
    StructAutoKey key[15];
};

/////////////////////////////////////////////////////////////////////
// Formate fuer Zeigerablage

/////////////////////////////////////////////////////////////////////
// Versionsnummern, 
// Zeiger die im EEPROM - Bereich liegen und auf das Flash verweisen.

struct StructFlashHeader
{
    unsigned short unused_0;           ///< Wird durch DatafoxStudioIV-Versionsständen < 04.03.11 auf 0 gesetzt.
    unsigned short version;            ///< Kommunikationsversionsnummer des Setup. (16bit, seit Studio 04.02.02.08 fix 0x0204)
    unsigned int KONFIG;               ///< Verwendeter Absoluter Addressoffset. Bis Studio 04.03.11 war Big-Endien
    unsigned char HEADLINE[18];        ///< Kopfzeile im Hauptmenue (17 Zeichen + Nullterminie)
    unsigned char unused_1[10];
    unsigned int LIST_STRUCT;          ///< Startadresse Listenbeschreibung 
    unsigned int LIST[20];             ///< Adresse der Daten der Liste Nr. 0 im Flash. Die weiteren Adressen sind nachfolgend gespeichert. z.B. Adresse von Liste Nr. 4  P_LIST_0+4 * sizeof(unsigned int) 
    unsigned char unused_2[8];
    unsigned int DAT_STRUCT;           ///< Kopie von P_DAT_STRUCT Fehler von ATmega zu umgehen 
    unsigned int FLASH_SIZE_SUS;       ///< Groesse der Flashdaten
    unsigned int MDE_EK_SIZE;          ///< Groesse der MDE - Eingabeketten
    unsigned int ENTRANCE_EK_SIZE;     ///< Groesse der Zutrittskontroll - Eingabeketten
    unsigned char LIST_KEY_FIELD[20];  ///< Fuer alle 20 Moeglichen LSBs je ein Byte fuer ein Schluesselfeld reservieren. 0=Keines zugewiesen, 1..MAX_FELD+1=Feldindex
    unsigned char unused_3;
    StructAutoKeys AUTO_KEYS;          ///< Tastenumschaltung
    unsigned int SETUP_VERSION;        ///< Ab: 03.01.30.00; Es werden in vier Byte die Versionsnummer des Setup mit Buildnumber abgelegt
    unsigned char DEVICE_TYPE;         ///< Ab: 03.01.31.00; Es wird in einem Byte der Geraetetyp abgespeichert, fuer den dieses Setup gespeichert wird. 
    unsigned int LIST_ACCESS;          ///< Zeiger auf Strukturen der Zutrittskontrolle: FW 04.01.01
    unsigned int IC_ACCESS;            ///< Zeiger auf Strukturen der Zutrittskontrolleingabeketten : FW 04.01.01
    unsigned short FILE_VERSION;       ///< Ab: 04.01.06.09; Es wird eine Dateiversion der Form 0xDF01 (16kByte Setupspeicher), 0xDF02 (48kByte Setupspeicher) abgelegt.
    unsigned int HEADER_SIZE;          ///< Ab: 04.02.03/04; sizeof(TAESHeader) fuer die aktuell verwendete Headerstruktur.
    unsigned int EXTENDED_SIZE;        ///< Ab: 04.02.03/04; Groesse des Erweiterten Speicherbereichs.
    unsigned char unused_4[260];
};

/////////////////////////////////////////////////////////////////////
// Strukturdeklarationen
// Es werden immer erst die defines eingefuehrt, welche in den Strukturen
// und allen weiteren evntl. benoetigt werden. Danach folgt die jeweilige Strukturdeklaration.

///////////////////////////////////////
// Struct: StructTransponder

// Flags fuer TP StructTransponder.tbModeX
#define BDTP_ACTIVE          0x80 ///< StructTransponder.tpMode: Einstellung fuer Transponder aktiviert.
//  lesen / schreiben
#define BDTP_WRITE           0x40 ///< StructTransponder.tpMode: Der Wert wird ins Segment geschrieben, ansonsten gelesen.

#define BDTP_DEC             0x00 ///< StructTransponder.tpMode: Behandlung der Daten als Dezimalwert.
#define BDTP_HEX             0x10 ///< StructTransponder.tpMode: Behandlung der Daten als Hexadezimalwert.
#define BDTP_ASCII           0x20 ///< StructTransponder.tpMode: Behandlung der Daten als ASCII - Zeichenfolge.
#define BDTP_HEX_TURNED      0x30 ///< StructTransponder.tpMode: Behandlung der Daten als Hexadezimalwert aber umgekehrte Bytefolge.

// Untere 4bit sind Anzahl feste stellen

/** Diese Struktur dient bei Angabe eines Hitagtransponders, der zu verwendenden Segmente, Ablageformate sowie Länge.

Diese Struktur kommt als Erweiterung in der Struktur StructExNumFeld zur Anwendung. Sie wird auch verwendet
um die globale Angabe sowie die Angabe bei der Zutrittskontrolle im StructConfig abzulegen.
*/
struct StructTransponder
{
    unsigned char tpSegment1;  ///< Gibt das erste Segment an, aus dem der Wert gelesen / oder in den der Wert geschieben werden soll.
    unsigned char tpMode1;     ///< Gibt den Modus des Wertes an. Oberes Nibble beinhaltet oben definierte Flags, unteres die feste Laengenangabe. 
    unsigned char tpSegment2;  ///< siehe Beschreibung zu tpSegment1, hier fuer ein zweites Segment.
    unsigned char tpMode2;     ///< siehe Beschreibung zu tpMode1, hier fuer einen zweiten Mode.
    unsigned char tpSegment3;  ///< siehe Beschreibung zu tpSegment1, hier fuer ein drittes Segment.
    unsigned char tpMode3;     ///< siehe Beschreibung zu tpMode1, hier fuer einen dritten Mode.
}; // 6 Byte

/** Diese Struktur dient bei Angabe eines Mifaretransponders, der zu verwendende Sektor, Ablageformat sowie Auswahl.

Diese Struktur kommt als Erweiterung in der Struktur StructExNumFeld zur Anwendung. Sie wird auch verwendet
um die globale Angabe sowie die Angabe bei der Zutrittskontrolle im StructConfig abzulegen.
*/
struct StructTransponderMifare
{
    unsigned char sectorBlock;      ///< Gibt im oberen Nibble den Sektor, im unteren den Block an, 
                                    ///< aus dem der Wert gelesen / oder in den der Wert geschieben werden soll.
    unsigned char mode;             ///< Oberes Nibble beinhaltet oben definierte Flags fuer das Datenformat, 
                                    ///< unteres die feste Laengenangabe. Bei Hex-Format ist der Wert mal 2 zu nehmen.
    unsigned char cut;              ///< Gibt im oberen Nibble die Startposition des ersten zu lesenden Bytes der 16 moeglichen an.
                                    ///< Das untere Nibble ist frei.
    unsigned char passwordMode;     ///< Gibt den zu verwendenden Passwortbereich an.
    unsigned char securityLevel;    ///< Gibt den zu verwendenden Security-Level des Mifare-Plus ausweises an.
    unsigned char unused;           ///< Frei.
}; // 6 Byte

/** Diese Struktur dient bei Angabe eines HID iClass Transponders, der zu verwendende Ausweistyp, Ablageformat sowie Auswahl.

Diese Struktur kommt als Erweiterung in der Struktur StructExNumFeld zur Anwendung. Sie wird auch verwendet
um die globale Angabe sowie die Angabe bei der Zutrittskontrolle im StructConfig abzulegen.
*/
struct StructTransponderHIDiClass
{
    unsigned char type;         ///< Gibt den zu verwendenden Ausweistyp an.
                                ///< 0 -> H10301
                                ///< 1 -> H10302
                                ///< 2 -> H10304
                                ///< 3 -> Freies Datenformat
    unsigned char format;       ///< Ablageformat.
                                ///< 0 -> Dezimal
                                ///< 1 -> Hexadezimal
                                ///< 2 -> Reserviert für ASCII nicht unterstützt!
                                ///< 3 -> Hexadezimal gedreht
    unsigned char length;       ///< Ablage feste Laengenangabe.
                                ///< 0 -> Keine feste Länge
                                ///< n -> Anzahl Zeichen ggf. mit Vornullen auf die angegebene Länge bringen.
    unsigned char unused[3];    ///< Frei.
}; // 6 Byte

///////////////////////////////////////
// Struct: StructTransponderLegicAdvantOld

/** Diese Struktur dient bei Angabe eines Legic Advant Transponders in den Globalen Einstellungen, 
das zu verwendende Segment, Ablageformat sowie Auswahl.

Diese Struktur wird verwendet um globale Angaben fuer Legic Advant Ausweise in der Firmwareversionen
04.02.02 aufzunehmen. Ab Version 04.02.03 wird auf die StructTransponderLegicAdvant ausgewichen.
*/
struct StructTransponderLegicAdvantOld
{
    unsigned char segment;          ///< Gibt das zu verwendende Segment an. Segment 0 bedeutet, dass der Ausweis nicht segmentiert ist.
    unsigned short pos;             ///< Position des zuerst gelesenen Datenbyte.
    unsigned char length;           ///< Anzahl zu lesender Datenbyte.
    unsigned char format;           ///< Gibt das abzulegende Werteformat an.
                                    ///< 0 -> Dezimal
                                    ///< 1 -> Hexadezimal
                                    ///< 2 -> ASCII
                                    ///< 3 -> Hexadezimal gedreht
    unsigned char formatLength;     ///< Gibt die abzulegende Werteformatlaenge an.
                                    ///< 0 -> Keine feste Laengenangabe
                                    ///< n -> Anzahl der abzulegenden Zeichen ggf. mit Vornullen auf die gewuenschte Laenge bringen.
    unsigned char typeFlags;        ///< Enthält die gewuenschten Ausweistypen und zusaetzliche Flags.
                                    ///< Bit 8 und 7 (Maske 0xC0) beinhaltet den Ausweistyp
                                    ///<    0 -> Nur Advant Ausweise
                                    ///<    1 -> Nur Prime Ausweise
                                    ///<    2 -> Advant und Prime Ausweise
                                    ///<    3 -> Frei
                                    ///< Bit 6 und 5 (Maske 0x30) CRC Pruefung
                                    ///<    0 -> Keine Pruefung
                                    ///<    1 -> 8Bit
                                    ///<    2 -> 16Bit
                                    ///<    3 -> Frei
                                    ///< Bit 4. Bei gesetztem Bit ist Access Segment Definition aktiv.
                                    ///< Bit 3. Bei gesetztem Bit sind ISO14443 Ausweise zugelassen.
                                    ///< Bit 2. Bei gesetztem Bit sind ISO15693 Ausweise zugelassen.
                                    ///< Bit 1. Bei gesetztem Bit sind Seriennummern von Prime Ausweisen nach dem Advant Format zu liefern.
    unsigned char valType;          ///< Zu Lesender Wertetyp.
                                    ///< 0 -> Seriennummer
                                    ///< 1 -> Freie Datenauswahl
                                    ///< 2 -> Kaba group Header von Prime Ausweisen lesen.
                                    ///< 3 -> Segment mittels Suche finden.
                                    ///< ... -> Frei
    unsigned short crcAddress;      ///< Position des CRC
    unsigned char kghSearch[13];    ///< Zu suchende Bytefolge fuer den Kaba group header
    unsigned char kghSearchLength;  ///< Laenge der zu suchenden Bytefolge des Kaba group header
    unsigned char flags;            ///< Enthaelt zusaetzliche Flags
                                    ///< Bit1: Bei gesetztem Bit wird nach KGH gesucht. Nur bei Ausweisen vom Typ Legic Prime.
                                    ///< Bit2 - 8: Frei.
    unsigned char unused[5];        ///< Reserve für Erweiterungen
}; // 30 Byte


///////////////////////////////////////
// Struct: StructTransponderLegicAdvantHeader

/** Diese Struktur enthaelt zusaetzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen fuer die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von count unsigned short Offsets, die die Offsets auf die
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhaengig.
*/
struct StructTransponderLegicAdvantHeader
{
    unsigned char structSize;       ///< Groesse dieser Struktur (ist gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray)
    unsigned char structFlags;      ///< Flags fuer die Strukturuebergreifende Verarbeitung.
                                    ///< Bit1: Passwoerter der Strukturen werden mittels RC4 und DatafoxKey verschluesselt.
                                    ///< Bit2-8: Unused
    unsigned char transponderType;  ///< Typ des Leseverfahrens.
    unsigned char count;            ///< Anzahl der Eintraege in dem Offsetarray, was auf diese Struktur folgt.
    unsigned char mode;             ///< Enthaelt die gewuenschten Ausweistypen.
                                    ///< 0 -> Nur Advant Ausweise
                                    ///< 1 -> Nur Prime Ausweise
                                    ///< 2 -> Advant und Prime Ausweise
    unsigned char flags;            ///< Enthaelt zusaetzliche Flags.
                                    ///< Bit1: Bei gesetztem Bit sind ISO14443 Ausweise zugelassen.
                                    ///< Bit2: Bei gesetztem Bit sind ISO15693 Ausweise zugelassen.
                                    ///< Bit3: Bei gesetztem Bit ist Access Segment Definition aktiv.
                                    ///< Bit4: Bei gesetztem Bit wird nach KGH gesucht. Nur bei Ausweisen vom Typ Legic Prime.
}; // 6 Byte

/** Diese Struktur dient bei Angabe eines Legic Advant Transponders in den Globalen Einstellungen, 
das zu verwendende Segment, Ablageformat sowie Auswahl.

Diese Struktur kommt als Erweiterung in der Struktur StructExNumFeld zur Anwendung. Sie wird auch verwendet
um die globale Angabe sowie die Angabe bei der Zutrittskontrolle im StructConfig abzulegen.
*/
struct StructTransponderLegicAdvant
{
    unsigned char rwType;           ///< Zu Lesender Wertetyp.
                                    ///< 0 -> Seriennummer
                                    ///< 1 -> Freie Datenauswahl
                                    ///< 2 -> Kaba group Header mittels Suche finden.
                                    ///< ... -> Frei
    unsigned char rwFlags;          ///< Enthaelt zusaetzliche Flags.
                                    ///< Bit1: Bei gesetztem Bit sind Seriennummern von Prime Ausweisen nach dem Advant Format zu liefern.
    unsigned char rwSegment;        ///< Gibt das zu verwendende Segment an. Segment 0 bedeutet, dass der Ausweis nicht segmentiert ist.
    unsigned short rwOffset;        ///< Position des zuerst gelesenen Datenbyte.
    unsigned short rwLength;        ///< Anzahl zu lesender Datenbyte.
    unsigned char kghSearch[13];    ///< Zu suchende Bytefolge fuer den Kaba group header
    unsigned char kghSearchLength;  ///< Laenge der zu suchenden Bytefolge des Kaba group header
    unsigned char crcCheck;         ///< Art der CRC Pruefung
                                    ///< 0 -> Keine Pruefung
                                    ///< 1 -> 8Bit
                                    ///< 2 -> 16Bit
    unsigned short crcAddress;      ///< Position des CRC
    unsigned char outputFormat;     ///< Gibt das abzulegende Werteformat an.
                                    ///< 0 -> Dezimal
                                    ///< 1 -> Hexadezimal
                                    ///< 2 -> ASCII
                                    ///< 3 -> Hexadezimal gedreht
                                    ///< 4 -> Dezimal gedreht
    unsigned short outputLength;    ///< Gibt die abzulegende Werteformatlaenge an.
                                    ///< 0 -> Keine feste Laengenangabe
                                    ///< n -> Anzahl der abzulegenden Zeichen ggf. mit Vornullen auf die gewuenschte Laenge bringen.
    unsigned char unused[3];        ///< Reserve für Erweiterungen
}; // 30 Byte


///////////////////////////////////////
// Struct: StructTransponderMifareDesfireHeader

/** Diese Struktur enthaelt zusaetzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen fuer die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von count unsigned short Offsets, die die Offsets auf die
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhaengig.
*/
struct StructTransponderMifareDesfireHeader
{
    unsigned char structSize;       ///< Groesse dieser Struktur (ist gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray)
    unsigned char structFlags;      ///< Flags fuer die Strukturuebergreifende Verarbeitung.
                                    ///< Bit1: Passwoerter der Strukturen werden mittels RC4 und DatafoxKey verschluesselt.
                                    ///< Bit2-8: Unused
    unsigned char transponderType;  ///< Typ des Leseverfahrens.
    unsigned char count;            ///< Anzahl der Eintraege in dem Offsetarray, was auf diese Struktur folgt.
}; // 4 Byte

/** Diese Struktur dient bei Angabe eines Mifare Desfire Transponders in den Globalen Einstellungen, 
das zu verwendende Segment, Ablageformat sowie Auswahl.

*/
struct StructTransponderMifareDesfire
{
    unsigned char rwFlags;              ///< Gibt durch Flags folgendes an.
                                        ///< Bit1: Gesetzt bedeutet, UID lesen. (Seriennummer)
                                        ///< Bit2: Gesetzt bedeutet, Echte UID bei Random UID Ausweisen lesen.
                                        ///< Bit3: Gesetzt bedeutet, UID auch von anderen Mifare Ausweisen lesen.
                                        ///< Bit4: Gesetzt bedeutet, UID mit einer Laenge von 4 Byte liefern.
                                        ///< Bit5-8: Unused
    unsigned int applicationNumber;     ///< Nummer der zu verwendenden Applikation.
    unsigned char fileNumber;           ///< Nummer der zu verwendenden Datei.
    unsigned short rwOffset;            ///< Offset in die Daten der Datei.
    unsigned short rwLength;            ///< Anzahl zu lesender Datenbyte aus der Datei.
    unsigned char fileType;             ///< Type der Datei.
                                        ///< 0 -> Standard
                                        ///< 1 -> Backup
    unsigned char communicationMode;    ///< Modus der Kommunikation zwischen Ausweis und Leser.
                                        ///< 0 -> Plain
                                        ///< 1 -> MACed
                                        ///< 2 -> Crypt
    unsigned char keyNumber;            ///< Nummer des zu verwendenden Schluessels.
    unsigned char keyFlags;             ///< Passwortflags
                                        ///< Bit1: Passwort liegt im Verschluesselten Format vor.
                                        ///< Bit2-8: Unused
    unsigned char applicationKey[32];   ///< Zu verwendender Schluessel.
    unsigned char cryptType;            ///< Typ der Verschluesselung.
                                        ///< 0 -> DF4    (16 Byte Schluessel)
                                        ///< 1 -> TDES   (16 Byte Schluessel)
                                        ///< 2 -> AES    (16 Byte Schluessel)
                                        ///< 3 -> 3KTDES (24 Byte Schluessel)
    unsigned char outputFormat;         ///< Gibt das abzulegende Werteformat an.
                                        ///< 0 -> Dezimal
                                        ///< 1 -> Hexadezimal
                                        ///< 2 -> ASCII
                                        ///< 3 -> Hexadezimal gedreht
                                        ///< 4 -> Dezimal gedreht
    unsigned short outputLength;        ///< Gibt die abzulegende Werteformatlaenge an.
                                        ///< 0 -> Keine feste Laengenangabe
                                        ///< n -> Anzahl der abzulegenden Zeichen ggf. mit Vornullen auf die gewuenschte Laenge bringen.
    unsigned char unused[10];           ///< Reserve fuer Erweiterungen
}; // 60 Byte


///////////////////////////////////////
// Struct: StructTransponderMifarePlusHeader

/** Diese Struktur enthaelt zusaetzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen fuer die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von count unsigned short Offsets, die die Offsets auf die
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhaengig.
*/
struct StructTransponderMifarePlusHeader
{
    unsigned char structSize;       ///< Groesse dieser Struktur (ist gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray)
    unsigned char structFlags;      ///< Flags fuer die Strukturuebergreifende Verarbeitung.
                                    ///< Bit1: Passwoerter der Strukturen werden mittels RC4 und DatafoxKey verschluesselt.
                                    ///< Bit2-8: Unused
    unsigned char transponderType;  ///< Typ des Leseverfahrens.
    unsigned char count;            ///< Anzahl der Eintraege in dem Offsetarray, was auf diese Struktur folgt.
    unsigned char securityLevel;    ///< Anzuwendender Sicherheitslevel.
                                    ///< 0 -> Security Level 0 (Ausweis ist blanko)
                                    ///< 1 -> Security Level 1
                                    ///< 2 -> Security Level 2
                                    ///< 3 -> Security Level 3
    unsigned char iso14443A;        ///< Gibt im Security Level 1/2 an welche Ausweistypen von ISO14443A akzeptiert werden.
                                    ///< Bit1: ISO14443A Mini
                                    ///< Bit2: ISO14443A Classic 1k
                                    ///< Bit3: ISO14443A Classic 4k
                                    ///< Bit4: ISO14443A Plus S
                                    ///< Bit5: ISO14443A Plus X
}; // 6 Byte

/** Diese Struktur dient bei Angabe eines Mifare Plus Transponders in den Globalen Einstellungen, 
das zu verwendende Segment, Ablageformat sowie Auswahl.

*/
struct StructTransponderMifarePlus
{
    unsigned char rwFlags;              ///< Gibt durch Flags folgendes an.
                                        ///< Bit1: Gesetzt bedeutet, UID lesen. (Seriennummer)
                                        ///< Bit2: Gesetzt bedeutet, Echte UID bei Random UID Ausweisen lesen. Nur im Security level 3.
                                        ///< Bit3: Gesetzt bedeutet, UID auch von anderen Mifare Ausweisen lesen.
                                        ///< Bit4: Gesetzt bedeutet, UID mit einer Laenge von 4 Byte liefern.
                                        ///< Bit5-8: Unused
    unsigned char rwSector;             ///< Nummer des zu verwendenden Sektors.
    unsigned char rwBlock;              ///< Nummer des zu verwendenden Blocks.
    unsigned char rwOffset;             ///< Offset in den Datenblock.
    unsigned short rwLength;            ///< Anzahl zu lesender Datenbyte aus dem Sektor/Block.
    unsigned char passwordUsage;        ///< Passwortverwendung fuer Lesen / Schreiben.
                                        ///< 0 -> KeyA / KeyA
                                        ///< 1 -> KeyA / KeyB
                                        ///< 2 -> KeyB / KeyB
    unsigned char passwordFlags;        ///< Passwortflags
                                        ///< Bit1: Fuer Lesen auch die Standardpasswoerter verwenden.
                                        ///< Bit2: passwordA_Crypto1 liegt im Verschluesselten Format vor.
                                        ///< Bit3: passwordB_Crypto1
                                        ///< Bit4: passwordA_AES liegt im Verschluesselten Format vor.
                                        ///< Bit5: passwordB_AES liegt im Verschluesselten Format vor.
                                        ///< Bit6: instID liegt im Verschluesselten Format vor.
                                        ///< Bit7-8: Unused
    unsigned char passwordA_Crypto1[6]; ///< Zu verwendendes Passwort A fuer Crypto 1 (nur im Security Level 1/2)
    unsigned char passwordB_Crypto1[6]; ///< Zu verwendendes Passwort B fuer Crypto 1 (nur im Security Level 1/2)
    unsigned char passwordA_AES[16];    ///< Zu verwendendes Passwort A im Security level 2 / 3
                                        ///< Zu verwendender Virtual Card Polling Encryption key im SL 3 bei Random UID
    unsigned char passwordB_AES[16];    ///< Zu verwendendes Passwort B
                                        ///< Zu verwendender Virtual Card Polling MAC key im SL 3 bei Random UID
    unsigned char instID[16];           ///< Zu verwendender Installation Identifier im SL 3 bei Random UID
    unsigned char outputFormat;         ///< Gibt das abzulegende Werteformat an.
                                        ///< 0 -> Dezimal
                                        ///< 1 -> Hexadezimal
                                        ///< 2 -> ASCII
                                        ///< 3 -> Hexadezimal gedreht
                                        ///< 4 -> Dezimal gedreht
    unsigned short outputLength;        ///< Gibt die abzulegende Werteformatlaenge an.
                                        ///< 0 -> Keine feste Laengenangabe
                                        ///< n -> Anzahl der abzulegenden Zeichen ggf. mit Vornullen auf die gewuenschte Laenge bringen.
    unsigned char unused[9];            ///< Reserve fuer Erweiterungen
}; // 80 Byte


///////////////////////////////////////
// Struct: StructTransponderMifareUltralightHeader

/** Diese Struktur enthaelt zusaetzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen fuer die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von count unsigned short Offsets, die die Offsets auf die
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhaengig.
*/
struct StructTransponderMifareUltralightHeader
{
    unsigned char structSize;       ///< Groesse dieser Struktur (ist gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray)
    unsigned char structFlags;      ///< Flags fuer die Strukturuebergreifende Verarbeitung.
                                    ///< Bit1: Passwoerter der Strukturen werden mittels RC4 und DatafoxKey verschluesselt.
                                    ///< Bit2-8: Unused
    unsigned char transponderType;  ///< Typ des Leseverfahrens.
    unsigned char count;            ///< Anzahl der Eintraege in dem Offsetarray, was auf diese Struktur folgt.

}; // 4 Byte

/** Diese Struktur dient bei Angabe eines Mifare Ultralight Transponders in den Globalen Einstellungen, 
das zu verwendende Segment, Ablageformat sowie Auswahl.

*/
struct StructTransponderMifareUltralight
{
    unsigned char rwFlags;          ///< Gibt durch Flags folgendes an.
                                    ///< Bit1: Gesetzt bedeutet, UID lesen. (Seriennummer)
                                    ///< Bit2-8: Unused.
    unsigned char rwBlock;          ///< Nummer des zu verwendenden Blocks.
    unsigned short rwOffset;        ///< Offset in den Block.
    unsigned short rwLength;        ///< Anzahl zu lesender Datenbyte.
    unsigned char keyUsage;         ///< Verwenddung des Schluessels.
                                    ///< 0 -> Kein Passwort.
                                    ///< 1 -> Ausweise ohne und mit diesem Passwort akzeptieren.
                                    ///< 2 -> Nur Ausweise mit diesem Passwort akzeptieren.
    unsigned char keyFlags;         ///< Passwortflags
                                    ///< Bit1: Passwort liegt im Verschluesselten Format vor.
                                    ///< Bit2-8: Unused
    unsigned char key[16];          ///< Zu verwendender Schluessel.
    unsigned char outputFormat;     ///< Gibt das abzulegende Werteformat an.
                                    ///< 0 -> Dezimal
                                    ///< 1 -> Hexadezimal
                                    ///< 2 -> ASCII
                                    ///< 3 -> Hexadezimal gedreht
                                    ///< 4 -> Dezimal gedreht
    unsigned short outputLength;    ///< Gibt die abzulegende Werteformatlaenge an.
                                    ///< 0 -> Keine feste Laengenangabe
                                    ///< n -> Anzahl der abzulegenden Zeichen ggf. mit Vornullen auf die gewuenschte Laenge bringen.
    unsigned char unused[3];        ///< Reserve fuer Erweiterungen
}; // 30 Byte


///////////////////////////////////////
// Struct: StructTransponderiCodeHeader

/** Diese Struktur enthaelt zusaetzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen fuer die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von count unsigned short Offsets, die die Offsets auf die
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhaengig.
*/
struct StructTransponderiCodeHeader
{
    unsigned char structSize;       ///< Groesse dieser Struktur (ist gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray)
    unsigned char structFlags;      ///< Flags fuer die Strukturuebergreifende Verarbeitung.
                                    ///< Bit1: Passwoerter der Strukturen werden mittels RC4 und DatafoxKey verschluesselt.
                                    ///< Bit2-8: Unused
    unsigned char transponderType;  ///< Typ des Leseverfahrens.
    unsigned char count;            ///< Anzahl der Eintraege in dem Offsetarray, was auf diese Struktur folgt.
}; // 4 Byte

/** Diese Struktur dient bei Angabe eines iCode Transponders in den Globalen Einstellungen, 
das zu verwendende Segment, Ablageformat sowie Auswahl.

*/
struct StructTransponderiCode
{
    unsigned char rwFlags;          ///< Gibt durch Flags folgendes an.
                                    ///< Bit1: Gesetzt bedeutet, UID lesen. (Seriennummer)
                                    ///< Bit2-8: Unused.
    unsigned char rwBlock;          ///< Nummer des zu verwendenden Blocks.
    unsigned short rwOffset;        ///< Offset in den Block.
    unsigned short rwLength;        ///< Anzahl zu lesender Datenbyte.
    unsigned char outputFormat;     ///< Gibt das abzulegende Werteformat an.
                                    ///< 0 -> Dezimal
                                    ///< 1 -> Hexadezimal
                                    ///< 2 -> ASCII
                                    ///< 3 -> Hexadezimal gedreht
                                    ///< 4 -> Dezimal gedreht
    unsigned short outputLength;    ///< Gibt die abzulegende Werteformatlaenge an.
                                    ///< 0 -> Keine feste Laengenangabe
                                    ///< n -> Anzahl der abzulegenden Zeichen ggf. mit Vornullen auf die gewuenschte Laenge bringen.
    unsigned char unused[6];        ///< Reserve fuer Erweiterungen
}; // 15 Byte


///////////////////////////////////////
// Struct: StructTransponderJavaCardHeader

/** Diese Struktur enthaelt zusaetzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen fuer die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von count unsigned short Offsets, die die Offsets auf die
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhaengig.
*/
struct StructTransponderJavaCardHeader
{
    unsigned char structSize;       ///< Groesse dieser Struktur (ist gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray)
    unsigned char structFlags;      ///< Flags fuer die Strukturuebergreifende Verarbeitung.
                                    ///< Bit1: Passwoerter der Strukturen werden mittels RC4 und DatafoxKey verschluesselt.
                                    ///< Bit2-8: Unused
    unsigned char transponderType;  ///< Typ des Leseverfahrens.
    unsigned char count;            ///< Anzahl der Eintraege in dem Offsetarray, was auf diese Struktur folgt.
}; // 4 Byte

/** Diese Struktur dient bei Angabe eines JavaCard Transponders in den Globalen Einstellungen, 
das zu verwendende Segment, Ablageformat sowie Auswahl.

*/
struct StructTransponderJavaCard
{
    unsigned char rwFlags;          ///< Gibt durch Flags folgendes an.
                                    ///< Bit1: Gesetzt bedeutet, UID lesen. (Seriennummer)
                                    ///< Bit2-8: Unused.
    unsigned char rwBlock;          ///< Nummer des zu verwendenden Blocks.
    unsigned short rwOffset;        ///< Offset in den Block.
    unsigned short rwLength;        ///< Anzahl zu lesender Datenbyte.
    unsigned char outputFormat;     ///< Gibt das abzulegende Werteformat an.
                                    ///< 0 -> Dezimal
                                    ///< 1 -> Hexadezimal
                                    ///< 2 -> ASCII
                                    ///< 3 -> Hexadezimal gedreht
                                    ///< 4 -> Dezimal gedreht
    unsigned short outputLength;    ///< Gibt die abzulegende Werteformatlaenge an.
                                    ///< 0 -> Keine feste Laengenangabe
                                    ///< n -> Anzahl der abzulegenden Zeichen ggf. mit Vornullen auf die gewuenschte Laenge bringen.
    unsigned char unused[6];        ///< Reserve fuer Erweiterungen
}; // 15 Byte


///////////////////////////////////////
// Struct: StructTransponderMyDHeader

/** Diese Struktur enthaelt zusaetzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen fuer die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von count unsigned short Offsets, die die Offsets auf die
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhaengig.
*/
struct StructTransponderMyDHeader
{
    unsigned char structSize;       ///< Groesse dieser Struktur (ist gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray)
    unsigned char structFlags;      ///< Flags fuer die Strukturuebergreifende Verarbeitung.
                                    ///< Bit1: Passwoerter der Strukturen werden mittels RC4 und DatafoxKey verschluesselt.
                                    ///< Bit2-8: Unused
    unsigned char transponderType;  ///< Typ des Leseverfahrens.
    unsigned char count;            ///< Anzahl der Eintraege in dem Offsetarray, was auf diese Struktur folgt.
    unsigned char isoFlags;         ///< Enthaelt die zugelassenen Ausweistypen.
                                    ///< Bit1: Bei gesetztem Bit sind ISO14443 Ausweise zugelassen.
                                    ///< Bit2: Bei gesetztem Bit sind ISO15693 Ausweise zugelassen.
                                    ///< Bit3-8: Unused
}; // 5 Byte

/** Diese Struktur dient bei Angabe eines MyD Transponders in den Globalen Einstellungen, 
das zu verwendende Segment, Ablageformat sowie Auswahl.

*/
struct StructTransponderMyD
{
    unsigned char rwFlags;          ///< Gibt durch Flags folgendes an.
                                    ///< Bit1: Gesetzt bedeutet, UID lesen. (Seriennummer)
                                    ///< Bit2-8: Unused.
    unsigned short rwBlock;         ///< Nummer des zu verwendenden Blocks.
    unsigned short rwOffset;        ///< Offset in den Block.
    unsigned short rwLength;        ///< Anzahl zu lesender Datenbyte.
    unsigned char keyUsage;         ///< Verwenddung des Schluessels.
                                    ///< 0 -> Kein Passwort.
                                    ///< 1 -> Ausweise ohne und mit diesem Passwort akzeptieren.
                                    ///< 2 -> Nur Ausweise mit diesem Passwort akzeptieren.
    unsigned char keyFlags;         ///< Passwortflags
                                    ///< Bit1: keyA liegt im Verschluesselten Format vor.
                                    ///< Bit2: keyB liegt im Verschluesselten Format vor.
                                    ///< Bit3-8: Unused
    unsigned char keyA[8];          ///< Zu verwendender Schluessel A.
    unsigned char keyB[8];          ///< Zu verwendender Schluessel B.
    unsigned char outputFormat;     ///< Gibt das abzulegende Werteformat an.
                                    ///< 0 -> Dezimal
                                    ///< 1 -> Hexadezimal
                                    ///< 2 -> ASCII
                                    ///< 3 -> Hexadezimal gedreht
                                    ///< 4 -> Dezimal gedreht
    unsigned short outputLength;    ///< Gibt die abzulegende Werteformatlaenge an.
                                    ///< 0 -> Keine feste Laengenangabe
                                    ///< n -> Anzahl der abzulegenden Zeichen ggf. mit Vornullen auf die gewuenschte Laenge bringen.
    unsigned char unused[2];        ///< Reserve fuer Erweiterungen
}; // 30 Byte


///////////////////////////////////////
// Struct: StructTransponderISOTagsHeader

/** Diese Struktur enthaelt zusaetzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen fuer die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von count unsigned short Offsets, die die Offsets auf die
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhaengig.
*/
struct StructTransponderISOTagsHeader
{
    unsigned char structSize;       ///< Groesse dieser Struktur (ist gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray)
    unsigned char structFlags;      ///< Flags fuer die Strukturuebergreifende Verarbeitung.
                                    ///< Bit1: Passwoerter der Strukturen werden mittels RC4 und DatafoxKey verschluesselt.
                                    ///< Bit2-8: Unused
    unsigned char transponderType;  ///< Typ des Leseverfahrens.
    unsigned char count;            ///< Anzahl der Eintraege in dem Offsetarray, was auf diese Struktur folgt.
}; // 4 Byte

/** Diese Struktur dient bei Angabe eines ISO Tags Transponders in den Globalen Einstellungen, 
das zu verwendende Segment, Ablageformat sowie Auswahl.

*/
struct StructTransponderISOTags
{
    unsigned char isoType;          ///< Gibt durch Flags an, welche ISO Ausweise gelesen werden sollen.
                                    ///< Bit1: ISO14443A
                                    ///< Bit2: ISO14443B
                                    ///< Bit3: ISO14443-2
                                    ///< Bit4: ISO14443-3
                                    ///< Bit5: ISO15693
                                    ///< Bit6-8: Unused
    unsigned char iso14443A;        ///< Gibt an welche Ausweistypen von ISO14443A gelesen werden sollen.
                                    ///< Bit1: ISO14443A Mini
                                    ///< Bit2: ISO14443A Classic 1k
                                    ///< Bit3: ISO14443A Classic 4k
                                    ///< Bit4: ISO14443A Plus S
                                    ///< Bit5: ISO14443A Plus X
                                    ///< Bit6: ISO14443A Ultralight
                                    ///< Bit7: ISO14443A DESFire
                                    ///< Bit8: ISO14443A LegicAdvant
    unsigned char iso14443A_2;      ///< Gibt an welche Ausweistypen von ISO14443A gelesen werden sollen.
                                    ///< Bit1: ISO14443A Andere (SmartMX, usw.)
                                    ///< Bit2-8: Unused
    unsigned char iso15693;         ///< Gibt an welche Ausweistypen von ISO15693 gelesen werden sollen.
                                    ///< Bit1: ISO15693i Code SL
                                    ///< Bit2: ISO15693 Tag-it HF-I
                                    ///< Bit3: ISO15693 My-D vicinity
                                    ///< Bit4: ISO15693 Andere (LegicAdvant usw.)
                                    ///< Bit5: ISO15693 24LR16 / 24LR64 von ST
                                    ///< Bit6: ISO15693 Fujitsu MB89R118 / MB89R119
                                    ///< Bit7: ISO15693 LRI2k / LRI64k von ST
                                    ///< Bit8: Unused
    unsigned char outputFormat;     ///< Gibt das abzulegende Werteformat an.
                                    ///< 0 -> Dezimal
                                    ///< 1 -> Hexadezimal
                                    ///< 2 -> unbenutzt (ASCII)
                                    ///< 3 -> Hexadezimal gedreht
                                    ///< 4 -> Dezimal gedreht
    unsigned char outputLength;     ///< Gibt die abzulegende Werteformatlaenge an.
                                    ///< 0 -> Keine feste Laengenangabe
                                    ///< n -> Anzahl der abzulegenden Zeichen ggf. mit Vornullen auf die gewuenschte Laenge bringen.
    unsigned char unused[4];        ///< Reserve fuer Erweiterungen
}; // 10 Byte


///////////////////////////////////////
// Struct: StructTransponderWiegandClockDataHeader

/** Diese Struktur enthaelt zusaetzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen fuer die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von count unsigned short Offsets, die die Offsets auf die
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhaengig.
*/
struct StructTransponderWiegandClockDataHeader
{
    unsigned char structSize;       ///< Groesse dieser Struktur (ist gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray)
    unsigned char structFlags;      ///< Flags fuer die Strukturuebergreifende Verarbeitung.
    ///< Bit1: Passwoerter der Strukturen werden mittels RC4 und DatafoxKey verschluesselt.
    ///< Bit2-8: Unused
    unsigned char transponderType;  ///< Typ des Leseverfahrens.
    unsigned char count;            ///< Anzahl der Eintraege in dem Offsetarray, was auf diese Struktur folgt.
}; // 4 Byte

/** Diese Struktur dient bei Angabe eines  Wiegand Clock Data  Transponders, der zu verwendende Ausweistyp, Ablageformat sowie Auswahl.

Diese Struktur kommt als Erweiterung in der Struktur StructExNumFeld zur Anwendung. Sie wird auch verwendet
um die globale Angabe sowie die Angabe bei der Zutrittskontrolle im StructConfig abzulegen.
*/
struct StructTransponderWiegandClockData
{
    unsigned char flags;                                ///< Flags, die die nachfolgenden Eigenschaften angeben.
    ///< Bit1: Steuerleitung (CLS) vorhanden: Gesetzt => vorhanden; Nicht gesetzt => nicht vorhanden.
    ///< Bit2: Parität prüfen: Gesetzt => prüfen; Nicht gesetzt => nicht prüfen.
    ///< Bit3: Firmencode prüfen: Gesetzt => prüfen; Nicht gesetzt => nicht prüfen.
    ///< Bit4-8: Unused.
    unsigned char clsWireActiveState;                   ///< Gibt den Zustand der Steuerleitung an, wenn diese aktiv ist (sofern vorhanden; siehe flags-Bit1).
    ///< 0 => Low-Aktiv.
    ///< 1 => High-Aktiv.
    unsigned char bitFormat;                            ///< Gibt das Bit-Format (Anzahl der vom Transponder gelieferten Bits) des Transponders an.
    ///< 0 => Individuelles Einstellen der Bitanzahl und der Eigenschaften Parität bzw. der Paritätsbits.
    ///< 1 => 26-Bit-Standardformat.
    ///< 2 => 32-Bit-Standardformat
    ///< 3 => 34-Bit-Standardformat.
    ///< 4 => 35-Bit-Standardformat.
    ///< 5 => 37-Bit-Standardformat.
    ///< 6 => 38-Bit-Standardformat.
    ///< 7 => 42-Bit-Standardformat.
	///< 8 => 66-Bit-Standardformat.
    unsigned short numberOfBits;                        ///< Gibt die Anzahl der Bits an, die eingestellt werden können, wenn das individuelle Bitformat gewählt ist (siehe bitFormat Wert 0).

    unsigned char parityFlags01;                        ///< Flags, die die nachfolgenden Eigenschaften angeben.
    ///< Bit1: Paritätsberechnung wird verwendet (wenn die Position des Paritätsbits größer Null ist): Gesetzt => Parität prüfen; Nicht gesetzt => Parität nicht prüfen.
    ///< Bit2: Gibt an, ob es sich um eine (un-)gerade Parität handelt: Gesetzt => ungerade Parität; Nicht gesetzt => gerade Parität.
    ///< Bit4-8: Unused.
    unsigned short parityBitPos01;                      ///< Gibt die Position des Paritätsbits an.
    unsigned short parityStart01;                       ///< Gibt die Position des Bits an, bei welchem der Paritätsbereich beginnt.
    unsigned short parityEnd01;                         ///< Gibt die Position des Bits an, bei welchem der Paritätsbereich endet.

    unsigned char parityFlags02;                        ///< Flags, die die nachfolgenden Eigenschaften angeben.
    ///< Bit1: Paritätsberechnung wird verwendet (wenn die Position des Paritätsbits größer Null ist): Gesetzt => Parität prüfen; Nicht gesetzt => Parität nicht prüfen.
    ///< Bit2: Gibt an, ob es sich um eine (un-)gerade Parität handelt: Gesetzt => ungerade Parität; Nicht gesetzt => gerade Parität.
    ///< Bit4-8: Unused.
    unsigned short parityBitPos02;                      ///< Gibt die Position des Paritätsbits an.
    unsigned short parityStart02;                       ///< Gibt die Position des Bits an, bei welchem der Paritätsbereich beginnt.
    unsigned short parityEnd02;                         ///< Gibt die Position des Bits an, bei welchem der Paritätsbereich endet.


    unsigned char facilityCode[8];                     ///< Gibt den Wert des Firmencodes an, gegen den der Wert des Firmencodes vom Transponder geprüft werden soll.
    unsigned char facilityCodeFormat;                   ///< Gibt das Format des eingegebenen Firmencodes an.
    ///< 0 => Dezimal.
    ///< 1 => Hexadezimal.
    ///< 2 => ASCII.
    ///< 3 => Hexadezimal gedreht.
    ///< 4 => Dezimal gedreht.
    ///< 5 => Binär.
    unsigned short facilityCodeBitPos;                  ///< Gibt die Position des Startbits des Firmencodes an.
    unsigned short facilityCodeNumberOfBits;            ///< Gibt die Anzahl der Bits des Firmencodes an.
    unsigned short cardDataBitPos;                      ///< Gibt die Position des Startbits der Kartendaten an.
    unsigned short cardDataNumberOfBits;                ///< Gibt die Anzahl der Bits der Kartendaten an.
    unsigned char outputData;                           ///< Gibt an, welche Daten von dem Transponder übernommen werden sollen.
    ///< 0 => Nur die Kartennummer.
    ///< 1 => Firmencode + Kartennummer.
    unsigned char outputFormat;                         ///< Gibt das Ablageformat der Daten von dem Transponder an.
    ///< 0 => Dezimal.
    ///< 1 => Hexadezimal.
    ///< 2 => ASCII.
    ///< 3 => Hexadezimal gedreht.
    ///< 4 => Dezimal gedreht.
    ///< 5 => Binär.
    unsigned char outputLength;                         ///< Gibt die Länge der Ausgabedaten an.
    ///< 0 => Keine feste Länge.
    ///< n => Anzahl Zeichen zwischen 1 und 11; Ggf. mit Vornullen auf die angegebene Länge bringen.
    unsigned char tmp[27];                              ///< Reserve fuer Erweiterungen.

}; // 66 Byte


///////////////////////////////////////
// Struct: StructTransponderETSITagsHeader

/** Diese Struktur enthaelt zusaetzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen fuer die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von count unsigned short Offsets, die die Offsets auf die
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhaengig.
*/
struct StructTransponderETSITagsHeader
{
    unsigned char structSize;       ///< Groesse dieser Struktur (ist gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray)
    unsigned char structFlags;      ///< Flags fuer die Strukturuebergreifende Verarbeitung.
                                    ///< Bit1: Passwoerter der Strukturen werden mittels RC4 und DatafoxKey verschluesselt.
                                    ///< Bit2-8: Unused
    unsigned char transponderType;  ///< Typ des Leseverfahrens.
    unsigned char count;            ///< Anzahl der Eintraege in dem Offsetarray, was auf diese Struktur folgt.
}; // 4 Byte

/** Diese Struktur dient bei Angabe eines ETSI Tag Transponders in den Globalen Einstellungen, 
das zu verwendende Segment, Ablageformat sowie Auswahl.

*/
struct StructTransponderETSITags
{
    unsigned char strength;         ///< Gibt die Feldstaerke in % an.
    unsigned char rwFlags;          ///< Gibt durch Flags folgendes an.
                                    ///< Bit1: Gesetzt bedeutet, EPC lesen. Ansonsten soll der angegebene Bank gelesen werden.
                                    ///< Bit2: Gesetzt bedeutet, Mehrachbuchung bei EPC. Es wird für bis zu 10 im Feld befindliche Ausweise eine separate Buchung durchgeführt.
                                    ///< Bit3-8: Unused
    unsigned char rwBank;           ///< Nummer der zu verwendenden Bank (0 - 3).
    unsigned short rwOffset;        ///< Offset in den Datenblock.
    unsigned short rwLength;        ///< Anzahl zu lesender Datenbyte aus dem Sektor/Block.
    unsigned int password;          ///< Passwortverwendung fuer Lesen / Schreiben.
    unsigned char outputFormat;     ///< Gibt das abzulegende Werteformat an.
                                    ///< 0 -> Dezimal
                                    ///< 1 -> Hexadezimal
                                    ///< 2 -> unbenutzt (ASCII)
                                    ///< 3 -> Hexadezimal gedreht
                                    ///< 4 -> Dezimal gedreht
    unsigned short outputLength;    ///< Gibt die abzulegende Werteformatlaenge an.
                                    ///< 0 -> Keine feste Laengenangabe
                                    ///< n -> Anzahl der abzulegenden Zeichen ggf. mit Vornullen auf die gewuenschte Laenge bringen.
    unsigned char unused[4];        ///< Reserve fuer Erweiterungen
}; // 18 Byte


///////////////////////////////////////
// Struct: StructTransponderQ5Header

/** Diese Struktur enthaelt zusaetzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen fuer die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von count unsigned short Offsets, die die Offsets auf die
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhaengig.
*/
struct StructTransponderQ5Header
{
    unsigned char structSize;       ///< Groesse dieser Struktur (ist gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray)
    unsigned char structFlags;      ///< Flags fuer die Strukturuebergreifende Verarbeitung.
                                    ///< Bit1: Passwoerter der Strukturen werden mittels RC4 und DatafoxKey verschluesselt.
                                    ///< Bit2-8: Unused
    unsigned char transponderType;  ///< Typ des Leseverfahrens.
    unsigned char count;            ///< Anzahl der Eintraege in dem Offsetarray, was auf diese Struktur folgt.
    unsigned char isoFlags;         ///< Enthaelt die zugelassenen Ausweistypen.
                                    ///< Bit1: Bei gesetztem Bit sind ISO14443 Ausweise zugelassen.
                                    ///< Bit2: Bei gesetztem Bit sind ISO15693 Ausweise zugelassen.
                                    ///< Bit3-8: Unused
}; // 5 Byte

/** Diese Struktur dient bei Angabe eines Q5 Transponders in den Globalen Einstellungen, 
das zu verwendende Segment, Ablageformat sowie Auswahl.

*/
struct StructTransponderQ5
{
    unsigned char rwFlags;          ///< Gibt durch Flags folgendes an.
                                    ///< Bit1: Gesetzt bedeutet, UID lesen. (Seriennummer)
                                    ///< Bit2-8: Unused.
    unsigned short rwBlock;         ///< Nummer des zu verwendenden Blocks.
    unsigned short rwOffset;        ///< Offset in den Block.
    unsigned short rwLength;        ///< Anzahl zu lesender Datenbyte.
    unsigned char keyUsage;         ///< Verwenddung des Schluessels.
                                    ///< 0 -> Kein Passwort.
                                    ///< 1 -> Ausweise ohne und mit diesem Passwort akzeptieren.
                                    ///< 2 -> Nur Ausweise mit diesem Passwort akzeptieren.
    unsigned char keyFlags;         ///< Passwortflags
                                    ///< Bit1: keyA liegt im Verschluesselten Format vor.
                                    ///< Bit2: keyB liegt im Verschluesselten Format vor.
                                    ///< Bit3-8: Unused
    unsigned char keyA[8];          ///< Zu verwendender Schluessel A.
    unsigned char keyB[8];          ///< Zu verwendender Schluessel B.
    unsigned char outputFormat;     ///< Gibt das abzulegende Werteformat an.
                                    ///< 0 -> Dezimal
                                    ///< 1 -> Hexadezimal
                                    ///< 2 -> ASCII
                                    ///< 3 -> Hexadezimal gedreht
                                    ///< 4 -> Dezimal gedreht
    unsigned short outputLength;    ///< Gibt die abzulegende Werteformatlaenge an.
                                    ///< 0 -> Keine feste Laengenangabe
                                    ///< n -> Anzahl der abzulegenden Zeichen ggf. mit Vornullen auf die gewuenschte Laenge bringen.
    unsigned char unused[2];           ///< Reserve fuer Erweiterungen
}; // 30 Byte

///////////////////////////////////////
// Struct: StructDatensatzBeschreibung / StructTable / StructTable2
// Beschreibung einer Tabelle

#define MAX_DATENSATZ_BESCHREIBUNG 20 ///< Maximale Anzahl der Datensatzbeschreibungen und auch max. Listenbeschreibungen
#define MAX_FELD                   25 ///< Maximale Anzahl Felder Felder in einem Datensatz.
#define MAX_DATENSATZ             241 ///< Maximale Groesse eines Datensatzes (255 - DS_KOPF_CRC)
#define MAX_UNTERPUNKTE           100 ///< Maximale Anzahl Unterpunkte im Baum (Bedienung) 

/** Diese Struktur beschreibt mit ihren Grundangaben eine Tabellendefinition.

Die Informationen einer Datensatz- sowie Listenbeschreibung, werden in dieser Struktur abgelegt.
Die Ablagereihenfolge in der Setupdatei (*.aes) ist wie folgt:
Es kommt zuerst ein Zeiger - Array einer Laenge von MAX_DATENSATZ_BESCHREIBUNG * unsigned int,
diese Zeiger verweisen jeweils auf eine der Maximal MAX_DATENSATZ_BESCHREIBUNG moeglichen.
*/
struct StructDatensatzBeschreibung
{
    unsigned char Felder; ///< Gibt die Anzahl der Tabellenfelder an.
    char Name[17];        ///< Gibt den verwendeten Tabellenname an. 
    unsigned int P;       ///< Gibt den Zeiger auf das Array der Zeiger fuer die Feldbeschreibungen (Typ "StructDatensatzFeld") an.
};

/** Diese Struktur ist eine Uebergangsstruktur von StructDatensatzBeschreibung auf StructTable2. 

Naehere Informationen siehe StructTable2.
*/
struct StructTable
{
    unsigned char count;  ///< Anzahl Felder im Datensatz 
    char name[17];        ///< Name der Datensatzbeschreibung
    unsigned int ptr;     ///< Zeiger auf Array fuer Zeiger auf Feldstrukturen
    unsigned char num;    ///< Nummer der Tabelle im System
    unsigned char key;    ///< Schluesselspalte
    unsigned char sort;   ///< Sortierspalte
    unsigned short flags; ///< Flags
};

/** Diese Struktur beschreibt mit ihren Grunddaten eine Tabellendefinition.

Diese Struktur ersetzt die nicht mehr ausreichende StructDatensatzBeschreibung.
Eine Beschreibung zur Ablage der Daten in der Setupdatei (*.aes) ist dort gegeben.
*/

#define TF_SORT   0x0001 // Tabelle soll lexiographisch sortiert werden und -sort- gibt nullindex der Spalte an
#define TF_SORTED 0x0002 // Listendaten sind lexiographisch nach spaltenindex -sort- sortiert.

struct StructTable2
{
    unsigned char num;    ///< Nummer der Tabelle im System
    char name[32];        ///< Name der Tabelle.
    unsigned char count;  ///< Anzahl Tabellenfelder
    unsigned char key;    ///< Schluesselspalte
    unsigned char sort;   ///< Sortierspalte
    unsigned short flags; ///< Flags
};


///////////////////////////////////////
// Struct: StructDatensatzFeld / StructTableField / StructTable2Field
// Beschreibung eines Tabellenfeldes 

//	Feldtypen mit Laengenangabe
#define TF_ULONG                                           1 ///< StructDatensatzFeld.Typ: Unsigned int (4 Byte)
#define TF_ULONG_LEN                                       4 ///< StructDatensatzFeld.Cnt: Unsigned int (4 Byte)
#define TF_DT                                              2 ///< StructDatensatzFeld.Typ: DateTime (7 Byte, 4 Byte Date, 3 Byte Time)
#define TF_DT_LEN                                          7 ///< StructDatensatzFeld.Cnt: DateTime (7 Byte, 4 Byte Date, 3 Byte Time)
#define TF_NUM                                             3 ///< StructDatensatzFeld.Typ: ASCII (nur Ziffern)
#define TF_A_NUM                                           4 ///< StructDatensatzFeld.Typ: ASCII
#define TF_DATE                                            5 ///< StructDatensatzFeld.Typ: Date (4 Byte; Y100, Y, M, D)
#define TF_DATE_LEN                                        4 ///< StructDatensatzFeld.Cnt: Date (4 Byte; Y100, Y, M, D)
#define TF_TIME                                            6 ///< StructDatensatzFeld.Typ: Time (3 Byte; h, m, s)
#define TF_TIME_LEN                                        3 ///< StructDatensatzFeld.Cnt: Time (3 Byte; h, m, s)
#define TF_FINGERTEMPLATE_DIN                              7 ///< StructDatensatzFeld.Typ: Fingertemplate (161 Byte; DIN V66400)
#define TF_FINGERTEMPLATE_DIN_LEN                        161 ///< StructDatensatzFeld.Cnt: Fingertemplate (161 Byte; DIN V66400)
#define TF_FINGERTEMPLATE_IDENCOM_COMPACT                  8 ///< StructDatensatzFeld.Typ: Fingertemplate (216 Byte; Idencom Compact)
#define TF_FINGERTEMPLATE_IDENCOM_COMPACT_LEN            216 ///< StructDatensatzFeld.Cnt: Fingertemplate (216 Byte; Idencom Compact)
#define TF_BINARY                                          9 ///< StructDatensatzFeld.Typ: Binaere Daten

/** Diese Struktur beschreibt mit ihren Grunddaten eine Tabellenfelddefinition.


Die Informationen einer Datensatzfeld- sowie Listenfeldbeschreibung, werden in dieser Struktur abgelegt.
Die Ablagereihenfolge in der Setupdatei (*.aes) ist wie folgt:
Es kommt zuerst ein Zeiger - Array einer Laenge von StructDatensatzBeschreibung.Felder * unsigned int,
diese Zeiger verweisen jeweils auf eine der StructDatensatzBeschreibung.Felder vorhandenen.
*/
struct StructDatensatzFeld
{
    unsigned char Typ;            ///< Feldtyp 
    unsigned char Cnt;            ///< Feldlaenge inc. anbschließende Null bei Strings 
    char Name[17];                ///< Feldname
};

/** Diese Struktur ist eine Uebergangsstruktur von StructDatensatzFeld auf StructTable2Field.

Naeheres siehe StructTable2Field.
*/
struct StructTableField
{
    unsigned char type;   ///< Feldtyp 
    unsigned char size;   ///< Feldlaenge inc. anbschlieszende Null bei Strings 
    char name[17];        ///< Feldname
    unsigned char num;    ///< Nummer des Tabellenfeldes
    unsigned short flags; ///< Flags
    char format[32];      ///< Formatstring fuer Feldwertmoeglichkeit
};

/** Diese Struktur beschreibt mit ihren Grunddaten eine Tabellenfelddefinition.

Diese Struktur ersetzt die nicht mehr ausreichende StructDatensatzFeld.
Eine Beschreibung zur Ablage der Daten in der Setupdatei (*.aes) ist dort gegeben.
*/
struct StructTable2Field
{
    unsigned char num;    ///< Nummer des Tabellenfeldes
    char name[32];        ///< Feldname
    unsigned char type;   ///< Feldtyp
    unsigned char size;   ///< Feldlaenge inc. anbschlieszende Null bei Strings 
    unsigned short flags; ///< Flags
    char format[32];      ///< Formatstring fuer Feldwertmoeglichkeit
};


/////////////////////////////////////////////////////////////////////
// Bediensteuerung 
// Hautpmenue mit Menueeintraegen, Eingabeketten und Feldern

///////////////////////////////////////
// Struct: StructBedien

// Typen der Bedienung 
#define TB_MENUE             1 ///< StructBedien.Typ: Folgestruktur ist von Typ: StructMenue
#define TB_DATENSATZ         2 ///< StructBedien.Typ: Folgestruktur ist von Typ: StructDatensatzEinlesen
#define TB_SYSTEM            3 ///< StructBedien.Typ: Folgestruktur ist von Typ: Undefiniert
#define TB_KONFIG_ROOT       4 ///< StructBedien.Typ: Folgestruktur ist von Typ: Undefiniert
#define TB_LISTE             5 ///< StructBedien.Typ: Folgestruktur ist von Typ: Undefiniert
#define TB_CLR_LAST          6 ///< StructBedien.Typ: Folgestruktur ist von Typ: Undefiniert

/** Informationsstruktur zur Datensteuerung der Bedienung.

In der Geraetebedienung kann von festen Informationskonstrukten gesprochen werden, an stellen wo
meherer Wege eingeschlagen werden koennten, kommt diese Struktur zum Einsatz. Es kann auf die
Bedienungswurzel z. B. sowohl eine Menuestruktur oder ein anderer Typ folgen. In einem Menue selbst sind
hingegen nur Menueeintraege erlaubt. In diesen jedoch wieder verschiednene Typen wie Eingabeketten etc.
Um Mehrdeutigkeiten aufzuloesen kommt somit vor jeder StructMenue und StructDatensatzEinlesen eine StructBedien.
*/
struct StructBedien
{
    unsigned char Typ;            ///< Typ der Folgestruktur 
    unsigned char d1;             ///< Reserve 
    unsigned char d2;             ///< Reserve
    unsigned int P;               ///< Zeiger auf Folgestruktur, welche durch "Typ" angegeben wird. 
};


///////////////////////////////////////
// Struct: StructMenue

/** Diese Struktur beinhaltet Informationen zum Hauptmenue / Untermenues.


*/
struct StructMenue
{
    unsigned char Flags;          ///< Flags wie Zutrittskontrollabfrage aktiviert usw..
                                  ///< Bit 0: Flag fuer die zweispaltige Anzeige beim AE-MasterIV. 
                                  ///< Bit 1: Menüauswahl per Barcode zulassen.
                                  ///< Bit 2: Menüauswahl per Tranponder zulassen.
    unsigned char EintragCnt;     ///< Anzahl Eintraege im Menue 
    char Name1[17];               ///< Ueberschrift1 des Menues 
    char Name2[17];               ///< Ueberschrift2 des Menues
    unsigned char K1Txt;          ///< Text in Kopfzeile 1 Bit 0...3 Nummer-Globale-Variable nur für das Hauptmenü gültig sonst 254
    unsigned char K2Txt;          ///< Text in Kopfzeile 2 Bit 0...3 Nummer-Globale-Variable nur für das Hauptmenü gültig sonst 254
    unsigned char GlobVar;        ///< GlobVar in die der Wert des Barcodes oder Transponders zur Auswahl des Menüeintrags kopiert werden soll.
    unsigned char EscV;           ///< Verhalten bei ESC-Taste, Sprungziel (Bedeutung siehe StructDatensatzEinlesen)
    unsigned char unused[4];      ///< Reserve
    unsigned int P;               ///< Zeiger auf Liste der Eintraege 
};


///////////////////////////////////////
// Struct: StructKonfigRoot

// Ein Varhalten 
#define EVERH_AKTUELL 0 ///< StructKonfigRoot.EinVerhalten: Nach Ein auf der aktuellen Pos vor Aus bleiben 
#define EVERH_MENUE   1 ///< StructKonfigRoot.EinVerhalten: Nach Einschalten zurueck in das Menue
///<                           Eintrag aus Hauptmenue starten

/** Diese Struktur beinhaltet Informationen zum Haupteintrag der Bedienung.

Dieser Struktur folgt ein Zeigerarray mit EintragCnt (* sizeof(unsigned int)) Zeigern, die auf folgende
StructMenue verweisen.
*/
struct StructKonfigRoot
{
    unsigned short AutoAus;     ///< Zeit in Sekunden bis Timeboy abgeschaltet wird, wenn keine Bedienung erfolgt
                                ///< 0 -> kein Automatische Abschaltung 
    unsigned char EintragCnt;   ///< Anzahl Eintraege in KonfigRoot 
    unsigned char EinVerhalten; ///< Verhalten nach Einschalten
    unsigned char unused[8];    ///< Reserve 
    unsigned int P;             ///< Zeiger auf Zeigerarray mit "EintragCnt" Eintraegen 
};


///////////////////////////////////////
// Struct: StructMenueEintrag

// Flags fuer Menueeintrag
#define BDME_MDE_0 0x00 ///< StructMenueEintrag.Flags: Normal
#define BDME_MDE_1 0x01 ///< StructMenueEintrag.Flags: Anmeldung
#define BDME_MDE_2 0x02 ///< StructMenueEintrag.Flags: Abmeldung
#define BDME_MDE_3 0x03 ///< StructMenueEintrag.Flags: Taste klemmen
#define BDME_MDE_4 0x04 ///< StructMenueEintrag.Flags: Taste klemmen und kummulieren
#define BDME_MDE_5 0x05 ///< StructMenueEintrag.Flags: Buchung
#define BDME_MDE_ONLY_BY_ORDER 0x80 ///< StructMenueEintrag.Flags: Ausfuehrung ist nur bei angemeldetem Auftrag moeglich.

/** Diese Struktur enthaelt Informationen zu einem Menueeintrag.

Dieser Struktur folgt ein Zeigerarray mit EintragCnt (* sizeof(unsigned int)) Zeigern, die auf folgende
StructDatensatzEinlesen oder StructMenue verweisen.
*/
struct StructMenueEintrag
{
    char Name[17];            ///< Menue Text 
    unsigned char EintragCnt; ///< Anzahl Eintraege in KonfigRoot 
    unsigned short Pass;      ///< Passwort
    unsigned char Flags;      ///< Parameter fuer MDE 
    unsigned char machine;    ///< Zugeordnete Maschine bei Taste Klemmen
    unsigned char unused[4];  ///< Reserve 
    unsigned int P;           ///< Zeiger auf Aktion nach Auswahl des Menuepunktes 
};


///////////////////////////////////////
// Struct: StructDatensatzEinlesen

// Einlesemetoden fuer Datensaetze 
#define TBDA_NORM 0                    ///< StructDatensatzEinlesen.Typ: Standard 

// Flags Datensatz einlesen 
#define BDEF_W_UEBER            0x01 ///< StructDatensatzEinlesen.Flags: Bei Schleifen bereits eingegebene Werte als 
                                     ///<                                Vorgabe fuer Eingabe uebernehmen 
#define BDEF_NICHT_SPEICHERN    0x02 ///< StructDatensatzEinlesen.Flags: Datensatz nicht speichern (nur Globale Variable) 
#define BDEF_ZUSATZ_RETURN      0x04 ///< StructDatensatzEinlesen.Flags: Abfrage "Speichern ?" am Ende der Eingabekette 
#define BDEF_RUECKSPRUNG_MENUE  0x08 ///< StructDatensatzEinlesen.Flags: Nach Einlesen eines Datensatzes Ruecksprungmenue anzeigen 
#define BDEF_ENTRANCE_ACCESS    0x10 ///< StructDatensatzEinlesen.Flags: Zutrittskontrolle vor Abarbeitung der Eingabekette durchfuehren.
#define BDEF_RELAIS             0x20 ///< StructDatensatzEinlesen.Flags: Relais fuer bestimmte Zeit schalten.
#define BDEF_RECORD_DOUBLE      0x40 ///< StructDatensatzEinlesen.Flags: Eingabekette vor evntl. Eingabe von Werten ausfuehren, danach "Normale" Abarbeitung. 
#define BDEF_NO_SERVER_ONLINE   0x80 ///< StructDatensatzEinlesen.Flags: Im Modus "Server online" wird das Verhalten, dass auf die Antwort und Sendmessage gewaretet wird, fuer diesen Datensatz deaktiviert.

/** Diese Struktur enthaelt Informationen zu einer Eingabekette.

Dieser Struktur folgt ein Zeigerarray mit Felder (* sizeof(unsigned int)) Zeigern die auf folgende 
StructFeldEinlesen verweisen. Ihr selbst ist immer eine StructBedien vorangestellt.
*/
struct StructDatensatzEinlesen
{
    unsigned char DsNum;            ///< Nummer der zugeordneten Datensatzbeschreibung (ab 0, 255 = Keine Zuordnung) 
    unsigned char Felder;           ///< Anzahl Felder
    unsigned char CrV;              ///< Verhalten bei Enter-Taste, Sprungziel 
                                    //Wert     Bedeutung
                                    //---------------------
                                    //0        Uebergeordnetes U-Menue
                                    //1        Hauptmenue (war mal ausschalten)
                                    //253      Reserviert. Naechstes Eingabekettenfeld.
                                    //254      Hauptmenue
                                    //255      Naechste E-Kette bzw. U-Menue
                                    //2-111    (1-109)-tes E-Feld der Eingabekette
                                    //112-127  (1-15)-te F–Taste des Hauptmenue
                                    //200-229  (1-30)-te E-Kette bzw. U-Menue im gleichen Menueeintrag.
                                    //230-252  Reserviert. Wird zu 0 konvertiert, wenn Bedeutung unbekannt.
    char Name[17];                  ///< Ueberschrift 
    unsigned char Typ;              ///< Bedien-Typ des Einlesens 
    unsigned char Flags;            ///< Diverse Flags Definiert als BDEF_.... 
    unsigned char K1Txt;            ///< Text in Feld K2 Bit 0...3 Nummer-Globale-Variable 
                                    ///<                 Bit 4..7 0-> Konstanter Text >0 -> Glob.-Var. 
    unsigned char EscV;             ///< Verhalten bei ESC-Taste, Sprungziel
                                    //Wert     Bedeutung
                                    //---------------------
                                    //0        Uebergeordnetes U-Menue
                                    //1        Hauptmenue (war mal ausschalten)
                                    //253      Reserviert. Naechstes Eingabekettenfeld.
                                    //254      Hauptmenue
                                    //255      Naechste E-Kette bzw. U-Menue
                                    //2-111    (1-109)-tes E-Feld der Eingabekette
                                    //112-127  (1-15)-te F–Taste des Hauptmenue
                                    //200-229  (1-30)-te E-Kette bzw. U-Menue im gleichen Menueeintrag.
                                    //230-252  Reserviert. Wird zu 0 konvertiert, wenn Bedeutung unbekannt.
    unsigned char vehicleRelease;   ///< Fahrzeugfreigabe
                                    //Bit7: Wenn gesetzt, dann ist dieser Wert fuer Fahrzeugfreigabe aktiv.
                                    //Bit6: Wenn gesetzt, dann soll eine Freigabe erfolgen, ansonsten zurueckgenommen werden.
                                    //Bit5: Reserve
                                    //Bit4-0: Freigabetimeout. 0 -> Unbegrenzt, 1...24 Stunden
    unsigned char ucRelaisNum;      ///< Nummer des anzusprechenden Relais.
                                    //0-3: Sind die internen Relais 1 .. 4
                                    //4-11: Sind die externen Relais des IO-Moduls 1 .. 8
                                    //12-23: Sind die internen Relais 5 .. 16
    unsigned short usRelaisTime;    ///< Zeit fuer die das ausgewaehlte Relais schliesst.
    unsigned char unused;           ///< Reserve
    unsigned int P;                 ///< Zeiger auf Liste der Strukturen fuer Einlesen der Felder 
                                    ///< (Anzahl und Reihenfolge muß mit Datensatzbeschreibung uebereinstimmen) 
};


///////////////////////////////////////
// Struct: StructExBestaetigung

/** Diese Struktur enthaelt die Textzeilen eines Bestaetigungsfeldes.

Diese Struktur kommt im union-Teil der Struktur StructFeldEinlesen zur Anwendung.
*/
struct StructExBestaetigung
{
    char Text1[17]; ///< Hinweistext der ersten Zeile.
    char Text2[17]; ///< Hinweistext der zweiten Zeile.
    char Text3[17]; ///< Hinweistext der dritten Zeile.
}; // 51 Byte

///////////////////////////////////////
// Struct: StructExText

/** Diese Struktur enthaelt Text einer Feldfunktion. Z. B. SMS versenden.

Diese Struktur kommt im union-Teil der Struktur StructFeldEinlesen zur Anwendung.
*/
struct StructExText
{
    char Text[51]; ///< Text der ersten Zeile.
}; // 51 Byte


///////////////////////////////////////
// Struct: StructCompare

#define FECT_NONE            0x00 ///< StructCompare.type: Kein Sprung
#define FECT_FORMAT_STRING   0x01 ///< StructCompare.type: Sprung mit Formatstring konstant.
#define FECT_FORMAT_GV       0x02 ///< StructCompare.type: Sprung mit Formatstring in globaler Variablen
#define FECT_FORMAT_STRING_2 0x03 ///< StructCompare.type: Sprung mit Formatstring konstant. FW >= 04.01.04
#define FECT_FORMAT_GV_2     0x04 ///< StructCompare.type: Sprung mit Formatstring in globaler Variablen. FW >= 04.01.04

/** Diese Struktur dient mit Ihren Informationen der Angabe eines Sprunges in einer Eingabekette.

Diese Struktur kommt als Erweiterung der Struktur StructExNumFeld zur Anwendung.
*/
struct StructCompare
{
    unsigned char type;            ///< Typ fuer die Vergleichsart; Formatstring || GV
    unsigned char formatGV;        ///< Wenn type GV: Bit 0..3: Arraynummer der GV, anderen Bits fuer Comboboxindex
    char formatString[17];         ///< Wenn type Formatstring: Der Formatstring.
    unsigned char negate;          ///< Verzweigen wenn Vergleich NICHT zutraf.
    unsigned char jumpDestination; ///< Sprungziel, als Feldindex (0..n), von Eingabekette.
}; // 21 Byte

///////////////////////////////////////
// Struct: StructExNumFeld

// Quellen fuer Eingabe von Daten (Maske in Var. Flag) 
#define NFF_TASTATUR    0x80 ///< StructExNumFeld.Flags: Werteingabe ueber Tastatur moeglich.
#define NFF_BARCODE     0x40 ///< StructExNumFeld.Flags: Werteingabe ueber Barcode moeglich.
#define NFF_TRANSPONDER 0x20 ///< StructExNumFeld.Flags: Werteingabe ueber Transponder moeglich.
#define NFF_NO_RET      0x01 ///< StructExNumFeld.Flags: CR nach Barcodeeingabe notwendig.
#define NFF_ANZEIGEN    0x0E ///< StructExNumFeld.Flags: 3 Bit Maske fuer 8 Werte der Anzeigedauer, wenn kein Return notwendig. 
#define NFF_BUZZER      0x10 ///< StructExNumFeld.Flags: Kein Aktustisches Signal zur Besaetigung

/** Diese Struktur dient in Eingabekettenfeldern bei allen Einlesemethoden ausser Liste und Bestaetigung, 
zur Angabe weiterer Informationen.

Diese Struktur kommt im union-Teil der Struktur StructFeldEinlesen zur Anwendung.
*/
struct StructExNumFeld
{
    unsigned char Flags;       ///< Flags z.B. Einlesen ueber Barcode erlaubt 
    StructTransponder tag;     ///< Transpondereinlesen bei Hitag, bei diesem Feld abaendern. Abweichend von der globalen Einstellung.
    union
    {
        unsigned char fingerprintParam;       ///< Parametervariable fuer Fingerprint
                                              ///< Bit 1+2: Destination: (0)Fingerprint modul, (1)Ausweis
                                              ///< Bit 3+4: Delete Method: (0)Fingertemplate, (1)PID
                                              ///< Bit 5+6: Fingertemplate: (0)Nr 1, (1)Nr 2
        unsigned char inputModuleTimeout;     ///< Zeitangabe wie lange der BC oder TP eingeschaltet bleibt.
        unsigned char operantLeft;            ///< Linker Operant bei TBFE_OPERATION
        unsigned char relayNumberExtended;    ///< Nummer des Relais. Wird verwendet bei TBFE_SWITCH_RELAY, wenn in relayNumber die Nummer des Relais >= 12 ist.
        unsigned char digitalStatusAsDecimal; ///< Bei der Feldfunktion TBFE_DIGITAL_WERT soll der aktuelle Status der digitalen Eingänge als 32bit Wert geliefert werden.
    };
    union
    {
        unsigned char fingerprintPID;   ///< Index der GV fuer PID-Quelle
        unsigned char operantRight;     ///< Rechter Operant bei TBFE_OPERATION.
    };

    union
    {
        unsigned char fingerprintTeachFlags; // Zusätliche Funktionen zum bei Saturn-Flächensensor ( z.B. Übungsmodus, Optimierungsmodus Traingsmodus ) 8 Bit frei somit
        unsigned char operation;        ///< Mathematische oder Logische Operation ausführen (Fuer TBFE_OPERATION)
        ///< Aufbau: Bit(7,6) = Linker Operant, Bit(5,4,3,2) = Operation, Bit(1,0) = Rechter Operant
        ///< Operant: 0 = Konst. 1 = Feld, 2 = GV
        ///< Operation: 0 = +, 1 = -, 2 = Not
    };
    //unsigned char relayFlagsNum;    ///< Flags und verwendete Relaisnummer (Fuer TBFE_SWITCH_RELAY, TBFE_SWITCH_LED, TBFE_SWITCH_BUZZER)
    unsigned char relayNumber : 4;
    unsigned char relayAction : 4;
                                    ///< Oberen Nibble bei Relais und Buzzer: 
                                    ///< Bit(3,2) = Aktion vor Zeitablauf; Bit (1,0) = Aktion nach Zeitablauf
                                    ///< Aktion: 0 = Ausschalten, 1 = Einschalten, 2 = Umschalten, 3 = Keine
                                    ///< Oberes Nibble bei LED:
                                    ///< Aktion: 0 = beide aus, 1 rot an grün aus, etc.
                                    ///< Unteres Nibble: Zu schaltende Relaisnummer oder bei LEDs die Maschinennummer.
                                    ///< Unteres Nibble bei Buzzer: Lautstaerke in 10er Schritten fuer % der Lautstaerke.
    unsigned short relayTime;       ///< Zeit fuer Schaltdauer in Sekunden.
    char KonstAscii[17];            ///< Wert der Konstante bei Einlesemethode "Konstante uebernehmen".
    StructCompare compare;          ///< Sprung in einer Eingabekette definieren.
}; // 51 Byte


///////////////////////////////////////
// Struct: StructExListAuswahl

#define BDLB_SEL_BARCODE         0x01 ///< StructExListAuswahl.SelBarcode: Selection in Liste mit Barcode freigeben 
#define BDLB_BESTAETIGEN         0x02 ///< StructExListAuswahl.SelBarcode: Ausgewaehlten Eintrag bestaetigen mit Ret 
#define BDLB_AUSBLENDEN          0x04 ///< StructExListAuswahl.SelBarcode: Automatisches Ausblenden der Liste nach n Sekunden. 

#define BDLB_SEL_TRANSPONDER     0x01 ///< StructExListAuswahl.Flags: Selection in Liste mit Transponder freigeben 
#define BDLB_SEL_TASTATUR        0x02 ///< StructExListAuswahl.Flags: Selection in Liste mit Tastatur freigeben 
#define BDLB_DONT_SHOW_BIG_WATCH 0x04 ///< StructExListAuswahl.Flags: Bei Data on Card die Große Uhr ausblenden und Nachricht fuer Schreiben auf Ausweis anzeigen. 
#define BDLB_INCREMENTAL_SEARCH  0x10 ///< StructExListAuswahl.Flags: Inkrementelle Suche in Liste (TimeboyIV)

/** Diese Struktur dient in Eingabekettenfeldern bei der Einlesemethode Liste, 
zur Angabe von weiteren Informationen.

Diese Struktur kommt im union-Teil der Struktur StructFeldEinlesen zur Anwendung.
*/
struct StructExListAuswahl
{
    unsigned char Sel;                   ///< Flag Listenauswahl mit Selection 
    unsigned char SelDFeld;              ///< Feld im Datensatz fuer Selection (nur wenn "Sel" = TRUE) 
    unsigned char SelLFeld;              ///< Feld in der Liste nach dem selectiet wird (nur wenn "Sel" = TRUE) 
    unsigned char List;                  ///< Nummer der Liste 
    unsigned char AuswahlFeld;           ///< Feld in der Liste nach dem der Benutzer auswaehlt 
    unsigned char DatFeld;               ///< Feld in der Liste das im Datensatz gespeichert wird 
    unsigned char SelBarcode;            ///< Die unteren drei Bit sind Flags, die oberen der Index der ausgewaehlten Anzeigedauer.
    unsigned char Flags;                 ///< Eingabequellen: Transponder, Tastatur
    unsigned char NoValueCheck;          ///< Wert nicht in Liste
    unsigned char jumpSelNoEntry;        ///< Sprungziel wenn SelNoEntry == 3
    unsigned char inputModuleTimeout;    ///< Auszeit wie lange das Modul angeschaltet bleibt, wenn aktiviert.
    unsigned char transponderConfig;     ///< Zu verwendende Konfiguration fuer lesen/schreiben auf Ausweise.
                                         ///< Oberes Nibble ist Index zum Lesen eines Wertes,
                                         ///< unters Nibble ist Index zum Schreiben eines Wertes.
    unsigned char unused_1[2];           ///< Tempfeld um hinter StructExNumFeld zu kommen.
    unsigned char doubleColumn;          ///< War mal: MapFKey, Die ersten zehn Listeneintraege auf Funktionstasten legen.
                                         ///< Ist Jetzt: Zweispaltige Anzeige beim AE-MasterIV
    unsigned char SelDFeld2;             ///< wie SelDFeld
    unsigned char SelLFeld2;             ///< wie SelLFeld
    unsigned char SelNoEntry;            ///< Verhalten wenn kein Eintrag vorhanden.
    unsigned char usedFontSize;          ///< Zu verwendente Schriftgroesze bei Listenanzeige
    unsigned char unused_2[11];          ///< Reserve.
    StructCompare compare;
}; // 51 Byte


///////////////////////////////////////
// Struct: StructExData

/** Diese Struktur enthält zusätzliche Daten zu einer Feldfunktion.

Diese Struktur kommt im union-Teil der Struktur StructFeldEinlesen zur Anwendung.
*/
struct StructExData
{
    unsigned char type;
    unsigned char flags; // 1 Bit für ergebnisorientiert mit Ok = True, ESC = False; 
                         // 2 Bit Option nächstes Eingabefeld = True; und False für die anderen Optionen 
    unsigned short time; // 0 = Immer eingeschatet
    unsigned char redOK;
    unsigned char greenOK;
    unsigned char blueOK;
    unsigned char whiteOK;
    unsigned char redESC; 
    unsigned char greenESC;
    unsigned char blueESC;
    unsigned char whiteESC;

    unsigned char unused[39]; ///< Unbenutzt
}; // 51 Byte

///////////////////////////////////////
// Struct: StructExGVToHID

/** Diese Struktur enthält zusätzliche Daten zu einer Feldfunktion.

Diese Struktur kommt im union-Teil der Struktur StructFeldEinlesen zur Anwendung.
*/
struct StructExGVToHID
{
    unsigned char nullByte; // Die Feldfunktion wurde zuvor mit StructExText implementiert,
                            // zur Abwärtskompatibilität wird hier eine Nullterminierung als Text vorgegeben.
    unsigned char ending;   // Abschließendes Zeichen der GV hinzufügen.
                            // 0 = Keine
                            // 1 = CR
                            // 2 = CR/LF
                            // 3 = Tabulator
    unsigned char interfacePort; // Gewählte Schnittstelle über die versendet werden soll.
                                 // 0 = HID
                                 // 1 = Zusätzlicher COM-Port.

    unsigned char unused[48]; ///< Unbenutzt
}; // 51 Byte

///////////////////////////////////////
// Struct: StructExSwitchModuleEnergie

/** Diese Struktur enthält zusätzliche Daten zu einer Feldfunktion.

Diese Struktur kommt im union-Teil der Struktur StructFeldEinlesen zur Anwendung.
Die Daten sind gleich mit der POWERMODULE-Struktur aus Power.h.
*/
struct StructExSwitchModuleEnergie
{
    unsigned char module;         ///< Gewahltes Modul. GPS, Display, etc..
    unsigned char item;           ///< Auswahl der Energieoption EIN, AUS usw.
    unsigned int time;            ///< Wert der Zeit in Sekunden
    unsigned char value;          ///< Wert für Helligkeit oder andere Größen
    unsigned int interval;        ///< Wert des Einschaltintervall der Zeit in Sekunden (value = 0) oder als Uhrzeit (value = 1)

    unsigned char unused[40]; ///< Unbenutzt
}; // 51 Byte

///////////////////////////////////////
// Struct: StructExSystemMessage

/** Diese Struktur enthält zusätzliche Daten zu einer Feldfunktion.

Diese Struktur kommt im union-Teil der Struktur StructFeldEinlesen zur Anwendung.
*/
struct StructExSystemMessage
{
    unsigned char value;          ///< Wert der übernommen werden soll.
                                  ///< 0 = Art
                                  ///< 1 = Id
                                  ///< 2 = Id als Text
                                  ///< 3 = Parameter 1
                                  ///< 4 = Parameter 2
                                  ///< 5 = Parameter 3
    unsigned char unused[50]; ///< Unbenutzt
}; // 51 Byte

///////////////////////////////////////
// Struct: StructExTakePhoto

/** Diese Struktur enthält zusätzliche Daten zu einer Feldfunktion.

Diese Struktur kommt im union-Teil der Struktur StructFeldEinlesen zur Anwendung.
*/
struct StructExTakePhoto
{
    unsigned char flags;         ///< Flags für Optionen.
                                 ///< Bit1: Wird die Kamera im Stichprobenmodus betrieben, kann durch setzten des Flags die Aufnahme erzwungen werden.
                                 ///< Bit2-8: Unbenutzt.
    unsigned char showTime;      ///< Anzeigedauer der Nachricht.
                                 ///< 0 = Keine Anzeige.
                                 ///< 1 - 15 = Sekunden.
                                 ///< 255 = Bestätigung durch ESC oder Enter.
    unsigned char showPeep;      ///< Piepsignal, Wert wie bei Textmeldung anzeigen.
    unsigned char unused[8];     ///< Unbenutzt
    char text[40];               ///< Wert für Helligkeit oder andere Größen
}; // 51 Byte

///////////////////////////////////////
// Struct: StructFeldEinlesen

// Einlesemethoden fuer Felder 
#define TBFE_NORM                        1 ///< Standardmethode.
#define TBFE_GET_TIME                    2 ///< Aktuelles Datum-Uhrzeit uebernehmen.
#define TBFE_LIST_SELECT_GET             3 ///< Aus Liste auswaehlen um Wert von Liste zu uebernehmen.
#define TBFE_KONSTANTE                   4 ///< Konstante in den Datensatz schreiben 
#define TBFE_BESTAETIGUNG                5 ///< Bestaetigung abfragen (fuellt kein Datenfeld).
#define TBFE_COPY_GV_TO_FIELD            6 ///< Globale Variable in Feld kopieren.
#define TBFE_SERIENNUMMER                7 ///< Seriennummer uebernehmen 
#define TBFE_ANALOG_WERT                 8 ///< Analogen Messwert in Feld uebernehmen.
#define TBFE_DIGITAL_WERT                9 ///< Digitalen Messwert in Feld uebernehmen.
#define TBFE_ZAEHLER_WERT               10 ///< Zaehlerstand in Feld uebernehmen.
#define TBFE_ENTRANCE_PNR               11 ///< Eingelesene Personalnummer uebernehmen.
#define TBFE_ENTRANCE_STATUS            12 ///< Status von Zutritt uebernehmen.
#define TBFE_FUNC                       13 ///< Funktionswert uebernehmen.
#define TBFE_SPS_WERT                   14 ///< Wert von SPS uebernehmen.
#define TBFE_GPRS_ALIVE_COUNTER         15 ///< Aufgelaufene Alivedatensaetze (Zaehler) uebernehmen.
#define TBFE_FIRMWARE_VERSION           16 ///< Firmware-Version uebernhemen.
#define TBFE_ONLINE_STATUS              17 ///< String "online" oder "offline" als -Server Online- Zustand uebernehmen
#define TBFE_ENTRANCE_ZM_NUM            18 ///< Zutrittsmasternummer uebernehmen.
#define TBFE_ENTRANCE_READER_NUM        19 ///< Lesernummer uebernehmen.
#define TBFE_SUMMER_WINTER_STATUS       20 ///< (S) fuer Sommerzeit oder (W) fuer Winterzeit uebernehmen.
#define TBFE_GPS_RECORD_FIX             21 ///< GPS - Daten uebernehmen. Feste Vorgabe des Aufbaus
#define TBFE_ENTRANCE2_IDENTIFICATION   22 ///< Zutrittspruefung durchfuehren. 
#define TBFE_COPY_FIELD_TO_GV           23 ///< Feldwert in Globale Variable kopieren.
#define TBFE_FINGERPRINT_SCANN          24 ///< Finger scannen und Fingertemplate liefern
#define TBFE_FINGERPRINT_CARD           25 ///< Fingertemplate(s) von Karte lesen.
#define TBFE_FINGERPRINT_TEACH          26 ///< Finger einlernen
#define TBFE_FINGERPRINT_IDENTIFICATION 27 ///< Identifikation durchfuehren
#define TBFE_FINGERPRINT_VERIFICATION   28 ///< Verifikation durchfuehren
#define TBFE_FINGERPRINT_DELETE         29 ///< Fingertemplate loeschen
#define TBFE_TIMEBOY_COPY_GV_TO_FIELD   30 ///< Globale Variable von Timeboy uebernehmen
#define TBFE_ANALOG_STATUS              31 ///< Status eines Analogen Eingangs uebernehmen
#define TBFE_LIST_SELECT_SET            32 ///< Aus Liste auswaehlen um Wert in Liste zu uebernehmen.
#define TBFE_OPERATION                  33 ///< Mathematisch oder Logische Operation ausfuehren.
#define TBFE_TIMEBOY_COPY_FIELD_TO_GV   34 ///< Feldwert in Globale Variable des Timeboy uebernehmen.
#define TBFE_SWITCH_RELAY               35 ///< Relais schalten
#define TBFE_GPS_RECORD_VARIABLE        36 ///< GPS - Daten uebernehmen. Variable Auswahl aus Daten.
#define TBFE_TIMER                      37 ///< Timer starten/stoppen
#define TBFE_TRANSPONDER_WRITE          38 ///< Transponderwert auf Karte schreiben
#define TBFE_SWITCH_LED                 39 ///< LED einer Maschine schalten
#define TBFE_GPS_DISTANCE               40 ///< Zurückgelegte Entfernung in m übernehmen.
#define TBFE_MOBILE_PARAMETER           41 ///< Parameter des Mobilfunks übernehmen. Ordsbestimmung.
#define TBFE_SWITCH_BUZZER              42 ///< Buzzer schalten
#define TBFE_KONSTANTE_TYPE_DEVICE_NAME 43 ///< Geraetetypname fuer welches dieses Setup ist, als Konstante uebernehmen.
#define TBFE_DATA_ON_CARD               44 ///< Datenarray auf Transponderkarten schreiben.
#define TBFE_SEND_SMS                   45 ///< SMS versenden mit maximal 50 Zeichen Text.
#define TBFE_SCRIPT                     46 ///< Scripttext mit maximal 50 Zeichen.
#define TBFE_MOBILMASTERIV_MENUENTRY    47 ///< Menueeintrag fuer TimeboyIV bei einer Mobil-MasterIV Anbindung.
#define TBFE_SWITCH_COMMUNICATION       48 ///< Hautpkommunikation umschalten.
#define TBFE_GV_TO_INTERFACE            49 ///< Werte von globaler Variable über Schnittstelle (HID, COM, etc.) senden. 
#define TBFE_SWITCH_BACKLIGHT           50 ///< Hintergrundbeleuchtung schalten.
#define TBFE_SHOW_MESSAGE               51 ///< Nachricht anzeigen.
#define TBFE_SWITCH_MODULE_ENERGIE      52 ///< Energiezustand eines Moduls einstellen.
#define TBFE_SYSTEM_MESSAGE             53 ///< Einen der Werte von einer Systemmeldung übernehmen.
#define TBFE_TAKE_PHOTO                 54 ///< Ein Photo erstellen.
#define TBFE_TIMER_STATE                55 ///< Status der Zeitschaltuhr übernehmen     
#define TBFE_FINGERPRINT_ADMNISTARTION  56 ///< Für das Saturmodul benötig da hier alle Funktionen von dem Gerät übernommen werden     
#define TBFE_VIBRATION_VALUE            57 ///< Vibrationszähler übernehmen


#define TBFE_MAX                        58 ///< Maximale Anzahl Funktionen fuer Arraygroesse

#define TBFE_DEBUG                     255 ///< Einlesemethode fuer interne Zwecke

// FeldEinlesenFlag
#define FEF_APPEND_TO_FIELD 0x01 ///< StructFeldEinlesen.flags: Der ermittelte Wert an den Feldwert anfuegen.
#define FEF_APPEND_TO_GV    0x02 ///< StructFeldEinlesen.flags: Der ermittelte Wert an die globale Variable anfuegen.

// Verhalten am Ende einer Eingabekette oder bei Eingabekettenfeld. 
#define EV_UPPER_MENUE    0 ///< Uebergeordnetes Untermenue 
#define EV_NEXT_FIELD   253 ///< Naechstes Eingabekettenfeld
#define EV_MAIN_MENUE   254 ///< Hauptmenue
#define EV_NEXT_SIBLING 255 ///< Naechste Eingabekette bzw. U-Menue. 

/** Diese Struktur enthaelt Informationen zu einem Eingabekettenfeld.

Der enthaltene union-Teil wird abhaengig von dem verwendeten Typ genutzt.
*/
struct StructFeldEinlesen
{
    unsigned char Typ;         ///< Typ der Einlesemetode "TBFE..." 
    unsigned char DFNum;       ///< Nummer des Datensatzfeldes (ab 0) 
    unsigned char EscV;        ///< Verhalten bei ESC-Taste (Bedeutung siehe StructDatensatzEinlesen)
    char Name[17];       ///< Ueberschrift 
    unsigned char K2Txt;       ///< Text in Feld K2 Bit 0..3 Nummer-Globale-Variable 
                               ///< Bit 4..7 = 0 -> Konstanter Text, 4..7 > 0 -> Glob.-Var. 
    unsigned char GlobVar;     ///< Feld-Wert in Globale Var. kopieren (Aufbau wie K2Txt)

    unsigned char SysVar;       /// Bit 7 = 1 -> Wert einer Systemvariablen wird in Zeile 4 der Anzeige angezeigt
                                ///              das Offset, welcher Typ angezeit wird steht in K2Txt
                                ///              die Adresse der Systemvariable steht in MessVar1
                                /// Bit 6 = 1 -> Wert einer Systemvariablen wird geschrieben
                                ///              das Offset, welcher Typ angezeit wird steht in GlobVar
                                ///              die Adresse der Systemvariable steht in MessVar2
                                /// Bit 5 = 1 -> Wert von Bit 7 liegt als String vor. 
                                /// Bit 4 = 1 -> Wert von Bit 6 liegt als String vor.
                                /// Ansonsten handelt es sich um eine Binaerarray. 
                                ///    Timeout  Zählerteiler
                                /// E1[00000001 00000002 ect.]
                                /// Hierbei wird in Messvar2 der Grundoffset E1 angegeben und GlobVar
                                /// Bit 3,2 -> 0 Keine Vorbelegung, 1 = Vorbelegung durch Feld, 2 = Vorbelegung durch GV.
                                /// Bit 1: Durch den Buzzer TP- oder BC- Aktivierung signalisieren.
                                /// Bit 0: BC oder TP Sofort einschalten (sofort scannen bei TBIV)

    unsigned char BarcodeVon;  ///< Bereich aus Wert ausschneiden 
    unsigned char BarcodeBis;  ///< Bereich aus Wert ausschneiden
                                /// BarcodeVon == 0: Ist BarcodeBis == 0, dann ist das Ausschneiden deaktiviert, ansonsten werden von Rechts BarcodeBis Zeichnen des Wertes geliefert.
                                /// BarcodeVon == 255: Es werden BarcodeBis Zeichen von Links aus dem Wert geliefert.
                                /// BarcodeVon > 0 && BarcodeBis < 255: Es werden von der Position BarcodeVon bis zur Position BarcodeBis die Zeichen mitten aus dem Wert geliefert.
    unsigned char InputTypCnt; ///< Bei Methode Normal Feldlaenge und Typ angeben.
    unsigned char MessVar1;    ///< 
    unsigned char MessVar2;    ///< 

    union
    {
        StructExText Text;
        StructExBestaetigung Bestaetigung; ///< 
        StructExNumFeld NumFeld;           ///< 
        StructExListAuswahl ListAuswahl;   ///< 
        StructExData Data;
        StructExGVToHID GVToHID;
        StructExSwitchModuleEnergie SwitchModuleEnergie;
        StructExSystemMessage SystemMessage;
        StructExTakePhoto TakePhoto;
    } Ex;
};


///////////////////////////////////////
// Struct: StructGlobVar

/** Diese Struktur enthaelt die Namen der definierten Globalen Variablen.

Diese Struktur zeigt mit P auf den naechsten Eintrag. Sie selbst ist standardmaessig die erste Struktur 
in den Setupdaten.
*/
struct StructGlobVar
{
    char SFeld[8][17]; ///< Namen der GlobVars 
    unsigned int P;          ///< Zeiger auf Root StructBedien 
};


///////////////////////////////////////
// Struct: StructConfig

// Transpondertypen: TRANSPONDER_TYPE
#define TRANSPONDER_NONE                     0x3F
#define TRANSPONDER_UNIQUE                   0
#define TRANSPONDER_HITAG2                   1
#define TRANSPONDER_LEGIC                    2
#define TRANSPONDER_HITAG1                   3
#define TRANSPONDER_HITAGS_H32               4
#define TRANSPONDER_MIFARE                   5
#define TRANSPONDER_Q5                       6
#define TRANSPONDER_HITAGS_H56               7
#define TRANSPONDER_HITAGS_H48               8
#define TRANSPONDER_IBUTTON                  9
#define TRANSPONDER_SIMONSVOSS              10
#define TRANSPONDER_NEDAP                   11
#define TRANSPONDER_TITAN                   12
#define TRANSPONDER_HITAG1_CRYPT            13
#define TRANSPONDER_HITAG1_SECURITY         14
#define TRANSPONDER_HID_PROXKEY             15
#define TRANSPONDER_HID_ICLASS              16
#define TRANSPONDER_LEGIC_ADVANT            17
#define TRANSPONDER_MIFARE_DESFIRE          18
#define TRANSPONDER_MIFARE_PLUS             19
#define TRANSPONDER_MIFARE_ULTRALIGHT       20
#define TRANSPONDER_ICODE                   21
#define TRANSPONDER_MYD                     22
#define TRANSPONDER_ISO_TAGS                23
#define TRANSPONDER_BLUEBOX_ID_TRONIC       24
#define TRANSPONDER_UNIQUE_FIT              25
#define TRANSPONDER_JAVACARD                26
#define TRANSPONDER_OCL                     27
#define TRANSPONDER_WIEGAND                 28
#define TRANSPONDER_CLOCK_DATA              29

/** Diese Struktur enthaelt die Grundeinstellungen zum Geraet.

Diese Struktur beinhaltet alle Gruneinstellungen des Geraets, sowie globale Einstellungen die ggf. an anderen
Stellen angepasst werden koennen.
*/
struct StructConfig
{
    unsigned char unused_1[3];                   ///< (0x00) Busnummer
                                                 ///< (0x02) Frei (war hibyte von LCD_LIGHT)
    unsigned char LCD_LIGHT;                     ///< (0x03) Hintergrundbeleuchtung LCD
    unsigned short ONLINE_WAIT;                  ///< (0x04) Wartezeit auf Antwort vom PC
    unsigned char unused_2;                      ///< (0x06) Frei (war hibyte von SERVER_ONLINE)
    unsigned char SERVER_ONLINE;                 ///< (0x07) Offlinebetrieb
    unsigned short SYSTEMMENU_PASSWORD;          ///< (0x08) Passwort fuer bios
    unsigned char unused_3[7];                   ///< (0x0A) MDE_DIVIDER beim AEIII+
                                                 ///< (0x0C) MDE_TIMEOUT beim AEIII+
                                                 ///< (0x0E) MDE_ZYKLUS beim AEIII+
                                                 ///< (0x10) Frei (war hibyte von TERMINAL_MODE)
    unsigned char TERMINAL_MODE;                 ///< (0x11) normal, PZE1, PZE2, MDE
    unsigned char unused_4[11];                  ///< (0x12) Unikatsmodus (-> Es wird die UID der Karte gelesen)
                                                 ///< (0x13) Unterscheidung zwischen segmentiert und unsegmentiert
                                                 ///< (0x14) ACTION_E1_UP beim AEIII+
                                                 ///< (0x16) ACTION_E1_DOWN beim AEIII+
                                                 ///< (0x18) ACTION_E2_UP beim AEIII+
                                                 ///< (0x1A) ACTION_E2_DOWN beim AEIII+
                                                 ///< (0x1C) Frei (war hibyte von ONOFF_DEACTIVATE)
    unsigned char ONOFF_DEACTIVATE;              ///< (0x1D) Ein-/Austaste aktivieren
    unsigned char unused_5;                      ///< (0x1E) Frei (war hibyte von INFO_TABLEAU)
    unsigned char INFO_TABLEAU;                  ///< (0x1F) Infoseite anzeigen
    unsigned char UNIQUE_Mode;                   ///< (0x20) Darstellungsformat (Dec., Hex. usw.)
    unsigned char UNIQUE;                        ///< (0x21) Modus transponder (4/5 Byte)
    // Frueherer MDE - Bereich fuer Transponder verwenden; SW03.01.32
    StructTransponder TP_GLOBAL;                 ///< (0x22) globale Struktur, wie eine Karte gelesen wird
    StructTransponder TP_ACCESS_CONTROL;         ///< (0x28) globale Struktur, wie bei Zutritt gelesen wird
    unsigned char HITAG_AUTOINC_FLAGS;           ///< (0x2E) Flags fuer Autoinkrement bei Karten schreiben
    unsigned short HITAG_AUTOINC_VALUE;          ///< (0x2F) 16BitWert um den Inkrementiert wird
    unsigned char TRANSPONDER_CUT_FLAGS;         ///< (0x31) Flags fuer Zuschneidefunktion der Legic/Mifare Leser
    unsigned char TRANSPONDER_CUT_FROM;          ///< (0x32) Start fuer Ausschneiden von Transponderwert bei Legic & Mifare
    unsigned char TRANSPONDER_CUT_LENGTH;        ///< (0x33) Laenge des auszuschneidenen Transponderwerts 
    unsigned char unused_6;                      ///< (0x34) Frei (war hibyte von IOMODULE_ACTIVE)
    unsigned char IOMODULE_ACTIVE;               ///< (0x35)  1 ist aktiv, 0 ist deaktiviert. 
    unsigned char TRANSPONDER_SEGMENT;           ///< (0x36) Segmentnummer bei segmentierten Karten
    unsigned char TRANSPONDER_TYPE;              ///< (0x37) Transponder Type: 0:Unique, 1:Hightech, 2:Legic (Highbit gibt Segmentverwendung an, naechstes bit fuer -nur Seriennummer lesen-)
    unsigned char unused_7;                      ///< (0x38) (war hibyte von KEY_SOUND)
    unsigned char KEY_SOUND;                     ///< (0x39) Tastenton
    unsigned char unused_8[5];                   ///< (0x3A) Display - Kontrast
                                                 ///< (0x3C) ? frei
                                                 ///< (0x3E) Frei (war hibyte
    unsigned char LED_PERMANENT;                 ///< (0x3F) Letzte FTaste durch blinken abzeigen, wie AE3
    unsigned char unused_9[2];                   ///< (0x40) TimeOut fuer MB
    unsigned int HITAG_INITIAL_VALUE_3;          ///< (0x42) Startwert fuer Kartensegment 3 beschreiben (ist im AEIII+ die IP-Adresse)
    unsigned char ENABLE_ENTRANCE;               ///< (0x46) Zutrittskontolle Version 1 aktivieren
    unsigned char EXT_READER;                    ///< (0x47) externer Leser verwenden
    unsigned char DOOR_SIGNAL_TIME;              ///< (0x48) Dauer fuer Tueroeffner
    unsigned char DOOR_MONITORING;               ///< (0x49) Tuerueberwachung aktivieren
    unsigned char DOOR_ALARM_TIME;               ///< (0x4A) Alarmzeit fuer geoeffnete Tuer
    unsigned char ENTRANCE_MAIN_MENU;            ///< (0x4B) Bit 0: Ueberpruefen von Zutritt im Hauptmenue

    unsigned char ENABLE_CUT;                    ///< (0x4C) Zuschneiden aktivieren
    unsigned char CUT_AT;                        ///< (0x4D) Zuschnitt von
    unsigned char CUT_TO;                        ///< (0x4E) Zuschnitt bis
    unsigned char unused_10[5];                  ///< (0x4F) Kartenpraefix fuer Hitag
                                                 ///< (0x53) Segment in/aus welches dieses geschrieben/gelesen wird
    unsigned int HITAG_INITIAL_VALUE_1;          ///< (0x54) Startwert fuer Kartensegment 1 beschreiben                                             
    unsigned char ENTRANCE_FLAG;                 ///< (0x58) Bit 0: Prüfung der Feiertage bei Zeitsteuerung der digitalen Ausgänge
                                                 ///< (0x58) War Typ fuer Wert des Kartenpraefix
    unsigned char EVER_BIG_WATCH;                ///< (0x59) Grosze Uhr immer anzeigen
    unsigned char TIMEOUT_PZE;                   ///< (0x5A) Timeout nach Eingaben zur Pruefung der Tastenumschalttabelle
    unsigned char MDE_LAMP_FLAGS;                ///< (0x5B) Flags fuer die (rote) Lampensteuerung der MDE
    unsigned char PEEP_FLAGS;                    ///< (0x5C) bit(0): wenn gesetzt, kein Bestaetigungspeep fuer OK; bit(1): wenn gesetzt, kein Bestaetigungspeep fuer ERROR
                                                 ///< (0x5C) War vor 04.01.05: Feste Stellenanzahl fuer Kartenwert; 0 => aus; wird links mit 0en aufgefuellt
    unsigned short TRANSPONDER_USER_PASSWORD;    ///< (0x5D) Passwort fuer Benutzermenue (Kartenprogrammiermenue/MMC/USB-Host etc.); 0 => aus; 16bit
    unsigned int HITAG_INITIAL_VALUE_2;          ///< (0x5F) Startwert fuer Kartensegment 2 beschreiben
    unsigned char ENABLE_ENTRANCE2;              ///< (0x63) Zutrittskontrolle Version 2 aktivieren
}; // 100 Byte

// ########################################################
// Erweiterter Flash

///////////////////////////////////////
// Struct: StructExtendedFlash

#define EXTENDED_CHECK 0xAAAAAAAE  ///< StructExtendedFlash.ulCheck: Pruefwert fuer Gueltigkeit (Versionskennung)

#define EXTF_EMPTY     0x00000000  ///< StructExtendedFlash.flags: Nicht vorhanden.
#define EXTF_NEW       0x00000001  ///< StructExtendedFlash.flags: Struktur angelegt.
#define EXTF_DOWNLOAD  0x00000010  ///< StructExtendedFlash.flags: Aus Geraet ausgelesene Daten.

/** Diese Struktur enthaelt die Kopfdaten des erweiterten Speichers.

Diese Struktur enthaelt die Adressen der einzelnen Bloecke. Sie ist die erste und einzige Struktur ihrer art
im Erweiterten Speicherblock (16kByte).
*/
struct StructExtendedFlash
{
    unsigned int check;                                    ///< muss immer mit EXTENDED_CHECK belegt werden.
    unsigned int size;                                     ///< groesse der Stuktur sizeof(StructExtendedFlash)
    unsigned int offsetIOModule;                           ///< Offset auf IOModul - Struktur
    unsigned int offsetIOModule8DigitalInputs;             ///< Offset auf die 8 StructDigitalInput - Strukturen des IOModul
    unsigned int offsetIntern4DigitalInputs;               ///< Offset auf die 4 StructDigitalInput - Strukturen fuer intern.
    unsigned int offsetEventChains;                        ///< Offset auf PointerArray der Eingabeketten fuer MDE
    unsigned int offsetFreeScope;                          ///< Offset auf naechsten freien Speicherbereich im ExtendedFlashMem
    unsigned int offsetAccessControlTablesV1;              ///< Offset auf die Tabellen der Zutrittskontrolle Version 1.
    unsigned int offsetAccessControlEventChainsV1;         ///< Offset auf OffsetArray der Eingabeketten fuer Zutrittskontrolle Version 1.
    unsigned int offsetGlobalVariableValues;               ///< Offset auf Struktur mit Standardwerten der GVs
    unsigned int offsetAccessControlTablesV2;              ///< Offset auf die Tabellen fuer Zutrittskontrolle Version 2.
    unsigned int offsetAccessControlEventChainsV2;         ///< Offset auf OffsetArray der Eingabeketten fuer Zutrittskontrolle Version 2.
    unsigned int offsetExtendedStruct;                     ///< Offset auf Struktur fuer Erweiterte Funktionalitaeten
    unsigned int offsetIntern4DigitalInputsPlus;           ///< Offset auf die weiteren 4 StructDigitalInput - Strukturen fuer intern.
    unsigned int offsetIntern2AnalogInputs;                ///< Offset auf die 2 StructAnalogInput - Strukturen fuer intern.
    unsigned int offsetEnergieSchemas;                     ///< Offset auf die Energieschemen
    unsigned int offsetMachineDefinitions;                 ///< Offset auf die Maschienendefinitionen
    unsigned int offsetEnergieMemo;                        ///< Offset auf Zeichenfolge für Memo der Energieverwaltung
    unsigned int offsetTransponderConfiguration;           ///< Offset auf die Transponderkonfigurationen
    // Die folgenden Zeiger werden von DatafoxStudioIV-Versionen < 04.03.03.02 ebenfalls gelesen, aber überschrieben.
    unsigned char unused_1[20];                            ///< Reserviert. Auf 100 Byte Strukturgroesse bringen.
    unsigned int flags;                                    ///< Flags fuer Struktur und Erweiterten Speicher
    // Ab der Version 04.03.03.01 auf 200 Byte vergrößert.
    // Die folgenden Zeiger werden von DatafoxStudioIV-Versionen < 04.03.03.02 nicht gelesen.
    unsigned int offsetInternDigitalInputs;                ///< Offset auf StructHeaderOffsetArray für die StructDigitalInput - Strukturen intern (ersetzt pulMDEDigital).
    unsigned int offsetInternDigitalInputsExt;             ///< Offset auf StructHeaderOffsetArray für die StructDigitalInputExt - Strukturen intern (ersetzt pulMDEDigitalExt).
    unsigned int offsetInternAnalogInputs;                 ///< Offset auf StructHeaderOffsetArray für die StructAnalogInput - Strukturen intern (ersetzt pulMDEAnalog).
    unsigned int offsetTimers;                             ///< Offset auf StructHeaderOffsetArray für die StructTimer - Stukturen (ersetzt pulExtended.timer[4]).
    unsigned int offsetVibrationSensor;                    ///< Offset auf eine StructVibrationSensor für den Vibrationssensor.
    unsigned int offsetSystemMessages;                     ///< Offset auf StructHeaderOffsetArray für die StructSystemMessage - Strukturen für Systemmeldungen.
    unsigned char unused_2[76];
};

#define sizeof_StructExtendedFlash_1    100 // Strukturgroesse der Version 1 ist: 100Byte
#define sizeof_StructExtendedFlash_2    200 // Strukturgroesse der Version 2 ist: 200Byte

///////////////////////////////////////
// Struct: StructHeaderOffsetArray

/** Diese Struktur enthaelt die Kopfdaten für folgende OffsetArray - Strukturdaten.

Diese Struktur ist eine vereinheitlichte Hilfstruktur für OffsetArray-Strukturdaten,
wie z. B. für StructDigitalInput, StructDigitalInputExt, StructAnalogInput, StructTimer, etc.
*/
struct StructHeaderOffsetArray
{
    unsigned int size;        ///< Version / Groesse der Stuktur sizeof(StructHeaderOffsetArray)
    unsigned int count;       ///< Anzahl der Pointer in dem Array
    unsigned int offsetArray; ///< Basisoffset auf die Sturkturbasis, zum Pointerarray.
};

///////////////////////////////////////
// Struct: StructIOModule

/** Diese Struktur enthaelt die Informationen zum IO-Module.

Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructIOModule
{
    // Spannungseingaenge
    unsigned char ucActiveU;       ///< Flags welcher Eingang aktiv.
    char szNameU[8][17];     ///< Bezeichner der Eingaenge.
    float fMinU[8];                ///< Minimalwert fuer Berechnung.
    float fMaxU[8];                ///< Maximalwert fuer Berechnung.
    char szUnitU[8][5];            ///< Einheit - suffix der Berechnung.
    unsigned char ucOnScreenU;     ///< Flags ob Ausgabe auf Display erlaubt.

    // Stromeingaenge
    unsigned char ucActiveI;       ///< Flags welcher Eingang aktiv.
    char szNameI[8][17];     ///< Bezeichner der Eingaenge
    float fMinI[8];                ///< Minimalwert fuer Berechnung.
    float fMaxI[8];                ///< Maximalwert fuer Berechnung.
    char szUnitI[8][5]; ///< Einheit - suffix der Berechnung.
    unsigned char ucOnScreenI;     ///< Flags ob Ausgabe auf Display erlaubt.
};


///////////////////////////////////////
// Struct: StructDigitalInput

/** Diese Struktur enthaelt die Informationen fuer die Behandlung eines digitalen Einganges.

Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructDigitalInput
{
    unsigned char ucUseAs;         ///< Verwendung als Start/Stop, Zaehler etc..
    unsigned char ucStartLowHigh;  ///< Auszufuehrende Eingabekette bei Flankenwechsel low -> high
    unsigned char ucStartHighLow;  ///< Auszufuehrende Eingabekette bei Flankenwechsel high -> low
    unsigned short uiCountDivisor; ///< Bei Verwendung als Zaehler, dessen Teiler
    unsigned short uiCycle;        ///< Bei Verwendung als Zaehler, dessen Zyklus.
    unsigned short uiTimeOut;      ///< Bei Verwendung als Zaehler mit Start/Stop, dessen Timeout.
    unsigned int ulCountDivisor32; ///< Bei Verwendung als Zaehler, dessen Teiler als 32bit
    unsigned short uiDebounceHigh; ///< Endprellzeit fuer Flankenwechsel Low-High
    unsigned short uiDebounceLow;  ///< Endprellzeit fuer Flankenwechsel High-Low
    unsigned char ucInvokeInputChainCountDivisor; ///< Auszufuehrende Eingabekette bei erreichtem Zaehler
    unsigned char ucInvokeInputChainCycle;        ///< Auszufuehrende Eingabekette bei erreichtem Zyklus

    unsigned char unused[2];       ///< Reserviert.
}; // 21 Byte

///////////////////////////////////////
// Struct: StructDigitalInputExt

/** Diese Struktur enthaelt zusaetzliche Informationen fuer die Behandlung eines digitalen Einganges.

Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructDigitalInputExt
{
    unsigned short uiCavity;   ///< Die zu verwendende Kavitaet.
    unsigned short uiStrokes;  ///< Die zu verwendende Hubzahl.

    unsigned char unused[17];  ///< Reserviert.
}; // 21 Byte

///////////////////////////////////////
// Struct: StructAnalogInput

/** Diese Struktur enthaelt die Informationen fuer die Behandlung eines analogen Einganges.

Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructAnalogInput
{
    unsigned char useAs;               ///< Eingangsverwendung als ...
    unsigned char show;                ///< Flags ob Ausgabe auf Display erlaubt.
    unsigned char sampleRate;          ///< Abtastrate fuer den analogen Eingang
    char name[17];               ///< Bezeichner der Eingaenge.
    char unit[5];           ///< Einheit - suffix der Berechnung.
    float minLimit;                    ///< Minimalwert fuer Berechnung.
    float maxLimit;                    ///< Maximalwert fuer Berechnung.
    float minInput;                    ///< Minimalwert fuer Eingangsspannung.
    float maxInput;                    ///< Maximalwert fuer Eingangsspannung.
    float limitUpperAlarm;             ///< Wert fuer die obere Alarmgrenze
    float limitUpperWarning;           ///< Wert fuer die obere Warngrenze
    float limitLowerWarning;           ///< Wert fuer die untere Warngrenze
    float limitLowerAlarm;             ///< Wert fuer die untere Alarmgrenze
    unsigned char limitTolerance;      ///< Grenzwertabweichung
    unsigned char invokeInputChain[5]; ///< Auszufuehrende Eingabekette bei Statuswechsel

    unsigned char unused[27];          ///< Reserviert.
}; // 92 Byte


///////////////////////////////////////
// Struct: StructVibrationSensor

/** Diese Struktur enthaelt die Informationen fuer die Behandlung eines Vibrationssensors.

Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructVibrationSensor
{
    unsigned char useAs;               ///< Eingangsverwendung als ...
    unsigned int  level;               ///< Wert fuer das auslösen Vibration erkannt
    unsigned short timeout;            ///< Erkennungszeitraum in Sekunden
    unsigned char invokeInputChain[2]; ///< Auszufuehrende Ketten eine für Start und eine für Stopp der Vibration

    unsigned char unused[23];          ///< Reserviert.
}; // 32 Byte


///////////////////////////////////////
// Struct: StructValueGVx

/** Diese Struktur enthaelt die ggf. zu setzenden Standardwerte der globalen Variablen.

Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructValueGVx
{
    unsigned char ucFlags;            ///< Bitfeld gibt an welche Variablen gesetzt werden sollen.
    char szValue[8][21]; ///< Zu setzender Standardwert der Variablen.
}; // 169 Byte


///////////////////////////////////////
// Struct: StructFingerprint

/** Diese Struktur enthaelt die Einstellparameter fuer ein Fingerprint-Modul

Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructFingerprint
{
    unsigned char method;           ///< Methode des Fingerprints
    unsigned char segment;          ///< Startsegment der abgelegten Fingertemplates auf Ausweis
    unsigned char securityLevel;    ///< Sicherheitslevel; der sich aus Falschrueckweisungrate et Falschakzeptanzrate ergibt.
    float securityFRR;              ///< Falschrueckweisungsrate
    float securityFAR;              ///< Falschakzeptanzrate
    unsigned char templateFormat;   ///< 0 = DIN V66400 (161 Byte)
                                    ///< 1 = Idencom-Compact (216 Byte)
                                    ///< 2 = Idencom-Standard (561 Byte)
                                    ///< 3 = Staturn-Standard (635 Byte) 
    unsigned char quality;          ///< Mindestbildqualitaet beim scannen
    unsigned char minutiaeCount;    ///< Mindestanzahl der Minutiae beim scannen
    unsigned char passwordGroup;    ///< Zu verwendende Passwortgruppe beim Schreiben des Ausweises
    unsigned char type;             ///< Typ des Fingerprintmodul 
                                    ///<  0 = Idencom Modul
                                    ///<  1 = Saturn01 Flächensensor
    unsigned char segment2;         ///< Hier ist bei Ausweisen z.B. MifareDesfire wo die Templates nicht hintereinander eingetragen sind 
                                    ///< die Position des zweiten Template eingetragen
    char unused[3];                 ///< Reserve fuer evntl. Erweiterungen
}; // 20 Byte

// Zusatztypen (161 Byte)
struct StructFingertemplateDIN
{
    unsigned int pidHi;
    unsigned int pidLo;
    unsigned char fid;
    unsigned char quality;
    unsigned char count;
    unsigned char data[50][3];
}; // 161 Byte

// Zusatztypen (216 Byte)
struct StructFingertemplateIdencom
{
    unsigned int pidHi;
    unsigned int pidLo;
    unsigned char fid;
    unsigned char quality;
    unsigned char count;
    unsigned char data[41][5];
}; // 216 Byte

// Zusatztypen (561 Byte)
struct StructFingertemplateIdencomStandard
{
    unsigned int pidHi;
    unsigned int pidLo;
    unsigned char fid;
    unsigned char quality;
    unsigned char count;
    unsigned char data[50][11];
}; // 561 Byte

// Zusatztypen (11 + 624 Byte)
struct StructFingertemplateSaturn01
{
    unsigned int pidHi;
    unsigned int pidLo;
    unsigned char fid;
    unsigned char quality;
    unsigned char count;
    unsigned char data[39][16];
}; // 635 Byte

struct StructFingertemplateId
{
    char pid[21];
    char fid[4];
}; // 25 Byte

///////////////////////////////////////
// Struct: StructTBAdaption

/** Diese Struktur enthaelt die Einstellparameter fuer eine Timeboyadaption an MasterIV-Geraete

Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructTimeboyAdaption
{
    unsigned char method;               ///< Methode der Adaption
    unsigned char count;                ///< Anzahl der Steckplaetze bei Mehrfachdocking
    unsigned char records;              ///< Datenablage auf MasterIV oder Timeboy
    unsigned char invokeInputChainPush; ///< Auszufuehrende Eingabekette, wenn Timeboy gesteckt wird
    unsigned char invokeInputChainPull; ///< Auszufuehrende Eingabekette, wenn Timeboy gezogen wird
    char unused[15];                    ///< Reserve fuer evntl. Erweiterungen
}; // 20 Byte

///////////////////////////////////////
// Struct: StructGPRS

/** Diese Struktur enthaelt die Einstellparameter fuer eine GPRS-Kommunikation

Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructGPRS
{
    unsigned char method;               ///< Methode der GPRS-Kommunikation
    unsigned char invokeInputChain;     ///< Auszufuehrende Eingabekette, wenn Alive-Timeout abgelaufen
    char unused[8];                     ///< Reserve fuer evntl. Erweiterungen
}; // 10 Byte

///////////////////////////////////////
// Struct: StructEnergieOption

/** Diese Struktur enthaelt die Einstellparameter fuer die EnergieOption zum Modbil-MasterIV.

Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructEnergieOption
{
    unsigned char mode;             ///< Aktivierung der Option
    unsigned char digitalIn;        ///< Eingabekette fuer das An/Aus der Zuendung
    unsigned short timeout;         ///< Timout, nach dem die Module nach einem An, eingeschaltet werden
    unsigned char invokeRelease;    ///< Eingabekette zur Fahrzeugfreigabe.
    unsigned char invokeGPS;        ///< Eingabekette bei gueltigen GPS-Daten.
    unsigned char digitalOut;       ///< Zu schaltendes Relais fuer Selbstabschaltung
    unsigned char flags;            ///< Flags fuer die Energieoption
                                    ///< Bit0: Einzel-(0) oder Dauerfreischaltung(1).
                                    ///< Bit1: GPS-Start/Stopp für An/Aus der Zündung nutzen.
                                    ///< Bit2: Immer Bewegung, Erkennung fuer An/Aus der Zündung deaktiviert.
                                    ///< Bit3-7: unused
    unsigned char powerLow;         ///< Tiefentladeschutz. Wert in Volt. 10.2V wird als 102 also *10 abgelegt
    unsigned char unused[7];        ///< Reserve
}; // 16 Byte

///////////////////////////////////////
// Struct: StructTimer

/** Diese Struktur enthaelt die Einstellparameter fuer die Timer
Die Struktur ist im extendet Speicherblock abgelegt.
*/
struct StructTimer
{
    unsigned char ucUseAs;          ///< Verwendung als Timer in Eingabeketten etc..
    unsigned short uiCycle;         ///< Bei Verwendung als Timer, dessen Zyklus.
    unsigned short uiTimeOut;       ///< Bei Verwendung als Timer, dessen Timeout.
    unsigned char invokeInputChain; ///< Auszufuehrende Eingabekette, wenn Timeout abgelaufen, oder danach der Zyklus abgelaufen.

    unsigned char unused[10];          ///< Reserve
}; // 16 Byte

///////////////////////////////////////
// Struct: StructGPS

/** Diese Struktur enthaelt die Einstellparameter fuer die GPS-Signalverarbeitung.
Die Struktur ist im extendet Speicherblock abgelegt.
*/
struct StructGPS
{
    unsigned char startSpeed;             ///< Zu erreichende Geschwindigkeit als Starterkennung. Einheit: km/h
    unsigned short startMove;             ///< Zu erreichende Bewegungsentfernung als alternative Starterkennung. Einheit: Meter
    unsigned char invokeInputChainStart;  ///< Auszuführende Eingabekette für Start Erkennung, 0 = keine Zuordnung
    unsigned short stopTimeout;           ///< Timeoutzeit für die stopSpeed unterschritten sein muss, damit ein Stopperkennung stattfindet. Einheit: Sekunden
    unsigned char stopSpeed;              ///< Geschwindigkeit die für erzeugen einer Stopperkennung unterschritten werden muss. Einheit: km/h
    unsigned char invokeInputChainStop;   ///< Auszuführende Eingabekette für Stopperkennung, 0 = keine Zuordnung
    unsigned short cycle;                 ///< Zykluszeit, in der ein Zyklusdatensatz erzeugt werden soll, wenn keine anderen Fahrdatensätze erzeugt werden. Einheit: Sekunden.
    unsigned char invokeInputChainCycle;  ///< Auszuführende Eingabekette für Zyklusdatensatz, 0 = keine Zuordnung
    unsigned char course;                 ///< Wert für Kursänderung.
    unsigned char invokeInputChainCourse; ///< Auszuführende Eingabekette für Kursdatensatz, 0 = keine Zuordnung
    unsigned short move;                  ///< Wert für Bewegung. Einheit: Meter.
    unsigned char invokeInputChainMove;   ///< Auszuführende Eingabekette für Bewegungsdatensatz, 0 = keine Zuordnung
    unsigned char unused[2];              ///< Reserve
}; // 18 Byte

///////////////////////////////////////
// Struct: StructMachineDefinition

/** Diese Struktur enthaelt Informationen ueber die Definition einer Maschine.

Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructMachineDefinition
{
    unsigned char flags;    ///< Gibt Informationsbits an. 
                            ///< Bit 0: 0 = Deaktiviert, 1 = Aktiviert.
    char name[17];    ///< Bezeichnung der Maschine
    unsigned char inputs;   ///< Gibt die zugeordneten digitalen Eingänge an. 
                            ///< Bit 0: E1
                            ///< ...
                            ///< Bit 7: E8
    unsigned char unused[5];///< Reserviert.
}; // 24 Byte

///////////////////////////////////////
// Struct: struct_password_box

struct struct_key_box
{
    unsigned char r;
    unsigned char flags; // Flagwerte.
    char key[17];        // Schlüssel + Nullterminierung
    unsigned short crc;  // CRC über die Struktur
}; // 21 Byte

///////////////////////////////////////
// Struct: StructExtended

/** Diese Struktur enthaelt zusaetzliche Informationen.

Diese Struktur ist im erweiterten Speicherblock abgelegt.
In dieser Struktur werden alle weiteren Informationen abgelegt.
*/
struct StructExtended
{
    unsigned short doubleInputTime;  ///< Zeit fuer "Doppelte Eingabe" PZE1/PZE2 Modus in Sekunden
    unsigned short confirmationTime; ///< Zeit fuer "Bestätigungsfelder" in Sekunden
    // Sommer/Winterzeitumschaltung
    unsigned char daylightActive;    ///< Aktivieren der Sommer-/Winterzeitumschaltung (daylight saving)
                                     ///< 0->Deaktiviert, 1->Alte Aktivierung, 2->Aktiv und von FW auszuwerten
    unsigned char daylight1Month;    ///< Monat in dem Umgestellt wird (1 - 12)
    unsigned char daylight1Week;     ///< Monatswoche in der Umgestellt wird (1 - 5, wobei 5 letzte im Monat bedeutet)
    unsigned char daylight1Day;      ///< Tag der Umschaltung (1 - 7, wobei 1 = Montag)
    unsigned char daylight1Hour;     ///< Stunde fuer Umschaltung (0 - 23)
    unsigned char daylight2Month;    ///< Monat in dem Umgestellt wird (1 - 12)
    unsigned char daylight2Week;     ///< Monatswoche in der Umgestellt wird (1 - 5, wobei 5 letzte im Monat bedeutet)
    unsigned char daylight2Day;      ///< Tag der Umschaltung (1 - 7, wobei 1 = Montag)
    unsigned char daylight2Hour;     ///< Stunde fuer Umschaltung (0 - 23)
    char transponderPassword32[9];   ///< Passwort fuer Hitag2 (32bit) in ASCII (Hex) abgelegt.
    char transponderPasswordA0[13];  ///< Passwort fuer Mifare (48bit) in ASCII (Hex) abgelegt.
    unsigned char phgAESKey[16];     ///< Schluessel fuer Kommunikationsverschluesselung der PHG Leser
    StructFingerprint fingerprint;        ///< Datenstruktur zum Fingerprint
    StructTimeboyAdaption adaption;       ///< Anbindungsstruktur von Timboys an MasterIV
    StructGPRS gprs;                      ///< Struktur fuer Zusatzangaben bezueglich GPRS, wie z. B. Alive.
    StructEnergieOption energieOption;    ///< Struktur fuer Energieoption bei Mobil-MasterIV
    StructTimer timer[4];                 ///< Struktur fuer Timer in der Signalverarbeitung (vorgesehen sind 4 Timer a 16 Byte)
    unsigned char transponderPasswordFlag;  ///< Passwortflag fuer Passwortverwendung von Mifare
                                            ///< Bit 0, 1: Passwortverwendung für Lesen/Schreiben A0/B0; 0=KeyA/KeyA, 1=KeyA/KeyB, 2=KeyB/KeyB
                                            ///< Bit 2, 3: Passwortverwendung für Lesen/Schreiben A1/B1; 0=KeyA/KeyA, 1=KeyA/KeyB, 2=KeyB/KeyB
                                            ///< Bit 4, 5: Passwortverwendung für Lesen/Schreiben A2/B2; 0=KeyA/KeyA, 1=KeyA/KeyB, 2=KeyB/KeyB
                                            ///< Bit 6: Frei
                                            ///< Bit 7: Autologin; Verwendung aller Passwörter für Lesen
    char transponderPasswordB0[13];  ///< Passwort fuer Mifare (48bit) in ASCII (Hex) abgelegt.
    char transponderPasswordA1[13];  ///< Passwort fuer Mifare (48bit) in ASCII (Hex) abgelegt.
    char transponderPasswordB1[13];  ///< Passwort fuer Mifare (48bit) in ASCII (Hex) abgelegt.
    char transponderPasswordA2[13];  ///< Passwort fuer Mifare (48bit) in ASCII (Hex) abgelegt.
    char transponderPasswordB2[13];  ///< Passwort fuer Mifare (48bit) in ASCII (Hex) abgelegt.
    int transponderSectorTrailer;    ///< Zugriffseinstellungen für den globale Sektor.
    unsigned char transponderInitialValue[17]; ///< Startwert von Mifare-Ausweis
    unsigned char memoryUsage;           ///< Speicheraufteilung zwischen Daten/Listen in %. Wert gibt die % für Daten reserviert an.
    unsigned char entranceOnline;        ///< Betriebsmodus der Zutrittskontrolle. 0=Offline, 1=Online, 2=Online/Offline mit Timeout.
    unsigned char entranceOnlineTimeout; ///< Timeout fuer die Umschaltung zwischen Online/Offline Betrieb der ZK.
    StructDigitalInputExt digitalExtE[8];     ///< Erweiterte Strukturen für die Digitalen Eingaenge.
    char fileName[44];                   ///< Dateiname der Setupdatei. Wird vor der Übertragung durch Laderoutine der Setupdatei gesetzt.
    char fileTimeStamp[20];              ///< Modifizierungszeitstempel der Setupdatei. Wird vor der Übertragung durch Laderoutine der Setupdatei gesetzt.
    unsigned char passwordSetup[17];     ///< Passwort für die Kommunikation
    StructGPS gps;                       ///< Struktur für GPS Datensatz Erzeugung
    unsigned char userPasswordEnable;    ///< Benutzerpasswort verwenden, ansonsten wird das Systempasswort fürs Userpasswort verwendet.
    unsigned char escJumpToMainmenu;     ///< Im PZE-Modus wird durch drücken der ESC-Taste das Hauptmenü angezeigt.
    unsigned char ledPerFieldFunction;   ///< AE/BDE-MasterIV: LED steuerung mit Feldfunktion "LED schalten" steuern.
    unsigned short dataWriteOKTime;      ///< Die Anzeigezeit der Meldung "Datenspeicherung erfolgreich". 0=aus, 500ms default,bereich 0,500-5000 in 250ms Schritten; Einheit ms
    unsigned char dataWriteOKFontType;   ///< Der verwendete Zeichensatztyp bei Meldung "Datenspeicherung erfolgreich". 0=
    unsigned int powerActiv;             ///< Gibt an ob die Energieverwaltung des Setups aktiviert ist, inclusive Flags für die aktivierten Module.
                                         ///< siehe PAF_x (P)ower(A)ctiv(F)lag
    unsigned char powerClockTimerCount;  ///< Gibt an wie viele Timer die zu einer Bestimmten Uhrzeit auslösen existieren dürfen.
    unsigned char spsQuery;              ///< Abfrage einer angeschlossenen SPS. Feldfunktion SPS-Abfrage.
    unsigned char dataOnCard;                 ///< Datenarrays auf Transponderkarten schreiben.
    unsigned char zkExtension;                ///< Erweiterte Zutrittskontrolle verwenden.
    unsigned short zkTagTimeToClose;          ///< Die Zeit wie lange der Ausweis vor einen ZK Leser gehalten werden muss bis eine Schliessung erfolgt.
    unsigned short zkAlarmTimeout;            ///< Auszeit nachdem ein Alarmdatensatz generiert wird, weil ein Digitaler Eingang zu lange offen war.
    unsigned char akkuLoadOptimization;       ///< Ladeerhaltung voller Akkus optimieren.
    unsigned char showSmallWatch;             ///< Zusaetzlich zur Anzeige der großen Uhr wird eine kleine Uhr mit Sekundenanzeige eingeblendet.
    unsigned char dontDimBacklightForCharge;  ///< Die Hintergrundbeleuchtung soll nicht gedimmt werden um die Aufladung der Akku's zu beschleunigen.
    unsigned char flagsEnergie;             ///< Zusaetzliche Flags fuer bestimmtes Geraeteverhalten in der Energieverwaltung.
                                            ///< Bit1: TimeboyIV soll GPS abschalten, wenn er sich in der Dockingstation befindet.
                                            ///< Bit2: TimeboyIV soll Mobilfunk abschalten, wenn er sich in der Dockingstation befindet.
                                            ///< Bit3-8: Reserve.
    unsigned short displayOffAfter;         ///< Display ausschalten nach [Wert] Minuten ohne Bedienung. 0 = niemals aus bis 990 = 16,5 Stunden  
    unsigned char flagsShowSetupNote;       ///< Flag ob ein Hinweis zu Änderungen im Setup angezeigtwerden soll 
                                            ///< Bit1: Energiesparoption "Display aus nach n Minuten ohne Bedienung."
                                            ///< Bit2: Standardwert wurde auf 120 Minuten gesetzt
    unsigned char phgCryptUseFlags;         ///< Flags für die Zusatzsicherheitsoptionen bei ZK-Anlagen mit PHG-Lesern.
                                            ///< Bit1: Initialisierungs-Vector soll zusätzliche zur Verschlüsselung verwendet werden.
                                            ///< Bit2: Session-Keys sollen zusätzlich zur Verschlüsselung verwendet werden.
    unsigned char readerPollingFlags;       ///< Flags für das Polling der Leser im Bus.
                                            ///< Bit1: Maximale Pollingrate.
                                            ///<       Achtung: Dieses Bit wird ab der Version 04.03.09 nicht weiter unterstützt und von der Firmware als fest gesetzt ausgewertet.
                                            ///< Bit2: Nur Leser aus der Reader-Liste im Bus suchen.
    unsigned char recordPriorityColumn[20]; ///< Spaltenindex fuer Prioritaet des Datensatzes zur Datensatzbeschreibung. 0=Keines zugewiesen, 1..MAX_FELD+1=Feldindex
    unsigned char cameraUse;                ///< Verwendung der evntl. integrierten Kamera.
                                            ///< 0 = Keine Verwendung.
                                            ///< 1 = Einfache Bildaufnahme per Feldfunktion.
                                            ///< 2 = Einfache und Stichprobenartige Bildaufnahme per Feldfunktion.
    unsigned char cameraGrab;               ///< Bei cameraUse == 2 gibt es den Prozentwert der Stichprobe an.

    unsigned char memoryUsageExtData;        ///< Für Daten zu Datensätzen ( Kamera Bilder in diesem Moment erstmal ) 0 - 90 
    unsigned char dateTimeFormatSettings;    ///< Zur Einstellung des des Datum und Zeit Format an den Geräten
                                             ///< 0 = Augenblickliches StandardFormat = Tag.Monat.Jahr und Uhr 24 Stunden Anzeige
                                             ///< 1 = Intenationales Datumsformat Jahr-Monat-Tag und Uhr 24 Stunden Anzeige
                                             ///< 2 = USA Datumsformat Monat/Tag/Jahr und Uhr 24 Stunden Anzeige
                                             ///< 3 = USA Datumsformat Monat-Tag-Jahr und Uhr 24 Stunden Anzeige
                                             ///< 4 = GB Datumsformat Tag/Monat/Jahr und Uhr 24 Stunden Anzeige
                                             ///< 5 = GB Datumsformat Tag-Monat-Jahr und Uhr 24 Stunden Anzeige
                                             ///< 16 = Deutsches Datumsformat Tag.Monat.Jahr und Uhr 12 Stunden Anzeige und AM/PM
                                             ///< 17 = Intenationales Datumsformat Jahr-Monat-Tag und Uhr 12 Stunden Anzeige und AM/PM
                                             ///< 18 = USA Datumsformat Monat/Tag/Jahr und Uhr 12 Stunden Anzeige und AM/PM
                                             ///< 19 = USA Datumsformat Monat-Tag-Jahr und Uhr 12 Stunden Anzeige und AM/PM
                                             ///< 20 = GB Datumsformat Tag/Monat/Jahr und Uhr 12 Stunden Anzeige und AM/PM
                                             ///< 21 = GB Datumsformat Tag-Monat-Jahr und Uhr 12 Stunden Anzeige und AM/PM
    struct_key_box protection_password;      ///< Passwort für die Verschlüsselte Wertablage.


    unsigned char  unused[414];             ///< Frei fuer Erweiterungen
}; // 1024 Byte

///////////////////////////////////////
// Struct: StructSystemMessage

/** Diese Struktur enthaelt zusaetzliche Informationen fuer die Behandlung einer Systemmeldung.

    Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
enum class EnumSystemMessageType : unsigned char
{
    None            = 0,
    Information     = 1, ///< 1 = Information
    Error           = 2  ///< 2 = Fehler
};
enum class EnumSystemMessageInformationId : unsigned char
{
    None            = 0,
    GetListPerHttp  = 100 ///< 100 = Übernahme von Listen per HTTP

};
enum class EnumSystemMessageErrorId : unsigned char
{
    None            = 0,
    GetListPerHttp  = 100 ///< 100 = Übernahme von Listen per HTTP
};
union UnionSystemMessageId
{
    EnumSystemMessageInformationId information;
    EnumSystemMessageErrorId error;
};
struct StructSystemMessage
{
    EnumSystemMessageType type;     ///< Der Nachrichten-Type.
    UnionSystemMessageId id;        ///< Die Nachrichtenkennung, abhängig von der Typangabe.
    unsigned int flags;             ///< Reservierte Flagvariable.
    unsigned char invokeInputChain; ///< Auszufuehrende Ereigniskette, wenn diese Meldung eintritt.
}; // 8 Byte


// Power

#define POWER_PROFIL_COUNT                       4  ///< Anzahl der möglichen Energieschemen
#define POWER_PROFIL_UNITS                       14 ///< Anzahl der auswählbaren Module CPU, Display usw.
#define POWER_PROFIL_ITEMS                       14 ///< Anzahl der auswählbaren Energieoptionen für AN, AUS usw.
#define POWER_EVENT_COUNT                        22 ///< Anzahl der definierten Events
#define POWER_EVENT_MAX                          22 ///< Anzahl der max. selektierbaren Events pro Energieschema
#define POWER_EVENT_TIMER_A_MAX  POWER_PROFIL_COUNT ///< Anzahl der Systemtimer, mögliche mit Timeout
#define POWER_EVENT_TIMER_B_MAX                   8 ///< Anzahl der Systemtimer, mögliche per Uhrzeit

//#define POWER_PROFILE_NAME_0                     _T("Betrieb")
//#define POWER_PROFILE_NAME_1                     _T("Sparmodus")
//#define POWER_PFOFILE_NAME_TURN_OFF              _T("Abschalten")
//#define POWER_PROFILE_NAME_0                    QT_TR_NOOP( "Power", "Betrieb" )
//#define POWER_PROFILE_NAME_1                    QT_TR_NOOP( "Power", "Sparmodus" ) 
//#define POWER_PFOFILE_NAME_TURN_OFF             QT_TR_NOOP( "Power", "Abschalten" )


#define POWER_DEFAULT_PROFILE                    hal_POWER_DEFAULT_PROFILE

//Auswahlpunkte der Module
#define POWER_ITEM_NR_OFF                        0  ///< Aus
#define POWER_ITEM_NR_ON                         1  ///< Ein
#define POWER_ITEM_NR_KEEP                       2  ///< Beibehalten aus vorausgegangenem Schema
#define POWER_ITEM_NR_SLEEP                      3  ///< Sleep-Mode
#define POWER_ITEM_NR_SERVICE_TIMEOUT            4  ///< Bedienung-TimeOut
#define POWER_ITEM_NR_CHAIN_FIELD                5  ///< EIN, wenn Modul in aktiver Feldfunktion benutzt wird
#define POWER_ITEM_NR_DISPLAY_DARK               6  ///< Beleuchtung Display Dunkel
#define POWER_ITEM_NR_SEND_RECORDS               7  ///< Kommunikation "Ein" wenn Datensätze vorhanden sind, "Aus" nach Timeout
#define POWER_ITEM_NR_COM_ON                     8  ///< Kommunikation Zyklisch "Ein", nach Datenübertragung wieder "Aus" nach TimeOut, Aktivierungszeit in h oder als Uhrzeit
#define POWER_ITEM_NR_USBHOST_MENUE              9  ///< Nur Ein im BIOS-Menü
#define POWER_ITEM_NR_GPS_CYCLE                  10 ///< Zyklischer Betrieb Ein bis gültige Daten vorhanden sind und Zeitwert für Aus muss angegeben werden, Aktivierungszeit in h oder als Uhrzeit
#define POWER_ITEM_NR_MACHINE_RUNS               11 ///< Maschine läuft(Start)
#define POWER_ITEM_NR_MACHINE_STOP               12 ///< Maschine steht(Stopp)
#define POWER_ITEM_NR_TB_TIMEOUT                 13 ///< Timeboy Steckevent EIN und nach TimeOut z.B. 10h wieder AUS
#define POWER_ITEM_NR_DIGITAL_IN                 11 ///< Digitaler Eingang wird verwendet
#define POWER_ITEM_NR_HIGH_OFF                    0 ///< Der High Pegel des verwendeten digitalen Eingangs schaltet das GPS Modul AUS
#define POWER_ITEM_NR_HIGH_ON                     1 ///< Der High Pegel des verwendeten digitalen Eingangs schaltet das GPS Modul EIN
#define POWER_ITEM_NR_TOFF                       14 ///< EIN, wenn digitaler Eingang x min. 1s HIGH wird. AUS wenn digitaler Eingang x wieder min. 1s LOW ist und eingestellter Timeout abgelaufen.
#define POWER_ITEM_NR_TON                        15 ///< EIN, wenn digitaler Eingang x min. 1s HIGH ist und eingestellter Timeout abgelaufen. AUS wenn digitaler Eingang x wieder min. 1s LOW ist.
#define POWER_ITEM_NR_STANDBY                    16 ///< Modul wird nur bei Bedarf aktiviert. Kann eventl. Verzögerungen hervorrufen.
#define POWER_ITEM_NR_TB_SLOT                    17 ///< EIN, wenn digitaler Eingang x min. 1s HIGH wird. AUS wenn digitaler Eingang x wieder min. 1s LOW ist und eingestellter Timeout abgelaufen.
///< Zusaetzlich wird bei AUS, wenn Listendaten fuer Timeboys vorliegen, alle 2 Minuten nach zu aktualisierenden Timeboys geschaut.


//Auswahlpunkte der Module
//#define TEXT_POWER_ITEM_NR_OFF                        _T("Aus")           ///< AUS
//#define TEXT_POWER_ITEM_NR_ON                         _T("Ein")           ///< EIN
//#define TEXT_POWER_ITEM_NR_KEEP                       _T("Beibehalten")   ///< Beibehalten aus vorausgegangenem Schema
//#define TEXT_POWER_ITEM_NR_SLEEP                      _T("Sleep-Mode")    ///< Sleep-Mode
//#define TEXT_POWER_ITEM_NR_SERVICE_TIMEOUT            _T("Eingabe")       ///< Bedienung-TimeOut
//#define TEXT_POWER_ITEM_NR_CHAIN_FIELD                _T("Feldfunktion")  ///< EIN, wenn Modul in aktiver Feldfunktion benutzt wird
//#define TEXT_POWER_ITEM_NR_DISPLAY_DARK               _T("Helligkeit")    ///< Beleuchtung Display Dunkel
//#define TEXT_POWER_ITEM_NR_SEND_RECORDS               _T("Datensatz")     ///< Kommunikation "Ein" wenn Datensätze vorhanden sind, "Aus" nach Timeout
//#define TEXT_POWER_ITEM_NR_COM_ON                     _T("Zyklisch")      ///< Kommunikation Zyklisch "Ein", nach Datenübertragung wieder "Aus" nach TimeOut, Aktivierungszeit in h oder als Uhrzeit
//#define TEXT_POWER_ITEM_NR_USBHOST_MENUE              _T("Bios")          ///< Nur Ein im BIOS-Menü
//#define TEXT_POWER_ITEM_NR_GPS_CYCLE                  _T("Zyklisch")      ///< Zyklischer Betrieb Ein bis gültige Daten vorhanden sind und Zeitwert für Aus muss angegeben werden, Aktivierungszeit in h oder als Uhrzeit
//#define TEXT_POWER_ITEM_NR_MACHINE_RUNS               _T("Start")         ///< Maschine läuft(Start)
//#define TEXT_POWER_ITEM_NR_MACHINE_STOP               _T("Stopp")         ///< Maschine steht(Stopp)
//#define TEXT_POWER_ITEM_NR_TB_TIMEOUT                 _T("Timeboy")       ///< Timeboy Steckevent EIN und nach TimeOut z.B. 10h wieder AUS
//#define TEXT_POWER_ITEM_NR_DIGITAL_IN                 _T("Dig.-Eingang")  ///< Digitaler Eingang wird verwendet
//#define TEXT_POWER_ITEM_NR_HIGH_OFF                   _T("High->AUS")     ///< Der High Pegel des verwendeten digitalen Eingangs schaltet das GPS Modul AUS
//#define TEXT_POWER_ITEM_NR_HIGH_ON                    _T("High->EIN")     ///< Der High Pegel des verwendeten digitalen Eingangs schaltet das GPS Modul EIN
//#define TEXT_POWER_ITEM_NR_TOFF                       _T("Aus nach")      ///< EIN, wenn digitaler Eingang x min. 1s HIGH wird. AUS wenn digitaler Eingang x wieder min. 1s LOW ist und eingestellter Timeout abgelaufen.
//#define TEXT_POWER_ITEM_NR_CPU_TOFF                   _T("Sleep nach")    ///< EIN, wenn digitaler Eingang x min. 1s HIGH wird. AUS wenn digitaler Eingang x wieder min. 1s LOW ist und eingestellter Timeout abgelaufen.
//#define TEXT_POWER_ITEM_NR_TON                        _T("Ein nach")      ///< EIN, wenn digitaler Eingang x min. 1s HIGH ist und eingestellter Timeout abgelaufen. AUS wenn digitaler Eingang x wieder min. 1s LOW ist.
//#define TEXT_POWER_ITEM_NR_STANDBY                    _T("Standby")       ///< EIN, nur bei Bedarf. Verzoegerungen durch Hochlaufzeit moeglich.
//#define TEXT_POWER_ITEM_NR_TB_SLOT                    _T("Docking")       ///< EIN, wenn digitaler Eingang x min. 1s HIGH wird. AUS wenn digitaler Eingang x wieder min. 1s LOW ist und eingestellter Timeout abgelaufen.
///< Zusaetzlich wird bei AUS, wenn Listendaten fuer Timeboys vorliegen, alle 2 Minuten nach zu aktualisierenden Timeboys geschaut.
// #define TEXT_POWER_ITEM_NR_OFF                        QT_TR_NOOP( "Power", "Aus" )           ///< AUS
// #define TEXT_POWER_ITEM_NR_ON                         QT_TR_NOOP( "Power", "Ein" )           ///< EIN
// #define TEXT_POWER_ITEM_NR_KEEP                       QT_TR_NOOP( "Power", "Beibehalten" )   ///< Beibehalten aus vorausgegangenem Schema
// #define TEXT_POWER_ITEM_NR_SLEEP                      QT_TR_NOOP( "Power", "Sleep-Mode" )    ///< Sleep-Mode
// #define TEXT_POWER_ITEM_NR_SERVICE_TIMEOUT            QT_TR_NOOP( "Power", "Eingabe" )       ///< Bedienung-TimeOut
// #define TEXT_POWER_ITEM_NR_CHAIN_FIELD                QT_TR_NOOP( "Power", "Feldfunktion" )  ///< EIN, wenn Modul in aktiver Feldfunktion benutzt wird
// #define TEXT_POWER_ITEM_NR_DISPLAY_DARK               QT_TR_NOOP( "Power", "Helligkeit" )    ///< Beleuchtung Display Dunkel
// #define TEXT_POWER_ITEM_NR_SEND_RECORDS               QT_TR_NOOP( "Power", "Datensatz" )     ///< Kommunikation "Ein" wenn Datensätze vorhanden sind, "Aus" nach Timeout
// #define TEXT_POWER_ITEM_NR_COM_ON                     QT_TR_NOOP( "Power", "Zyklisch" )      ///< Kommunikation Zyklisch "Ein", nach Datenübertragung wieder "Aus" nach TimeOut, Aktivierungszeit in h oder als Uhrzeit
// #define TEXT_POWER_ITEM_NR_USBHOST_MENUE              QT_TR_NOOP( "Power", "Bios" )          ///< Nur Ein im BIOS-Menü
// #define TEXT_POWER_ITEM_NR_GPS_CYCLE                  QT_TR_NOOP( "Power", "Zyklisch" )      ///< Zyklischer Betrieb Ein bis gültige Daten vorhanden sind und Zeitwert für Aus muss angegeben werden, Aktivierungszeit in h oder als Uhrzeit
// #define TEXT_POWER_ITEM_NR_MACHINE_RUNS               QT_TR_NOOP( "Power", "Start" )         ///< Maschine läuft(Start)
// #define TEXT_POWER_ITEM_NR_MACHINE_STOP               QT_TR_NOOP( "Power", "Stopp" )         ///< Maschine steht(Stopp)
// #define TEXT_POWER_ITEM_NR_TB_TIMEOUT                 QT_TR_NOOP( "Power", "Timeboy" )       ///< Timeboy Steckevent EIN und nach TimeOut z.B. 10h wieder AUS
// #define TEXT_POWER_ITEM_NR_DIGITAL_IN                 QT_TR_NOOP( "Power", "Dig.-Eingang" )  ///< Digitaler Eingang wird verwendet
// #define TEXT_POWER_ITEM_NR_HIGH_OFF                   QT_TR_NOOP( "Power", "High->AUS" )     ///< Der High Pegel des verwendeten digitalen Eingangs schaltet das GPS Modul AUS
// #define TEXT_POWER_ITEM_NR_HIGH_ON                    QT_TR_NOOP( "Power", "High->EIN" )     ///< Der High Pegel des verwendeten digitalen Eingangs schaltet das GPS Modul EIN
// #define TEXT_POWER_ITEM_NR_TOFF                       QT_TR_NOOP( "Power", "Aus nach" )      ///< EIN, wenn digitaler Eingang x min. 1s HIGH wird. AUS wenn digitaler Eingang x wieder min. 1s LOW ist und eingestellter Timeout abgelaufen.
// #define TEXT_POWER_ITEM_NR_CPU_TOFF                   QT_TR_NOOP( "Power", "Sleep nach" )    ///< EIN, wenn digitaler Eingang x min. 1s HIGH wird. AUS wenn digitaler Eingang x wieder min. 1s LOW ist und eingestellter Timeout abgelaufen.
// #define TEXT_POWER_ITEM_NR_TON                        QT_TR_NOOP( "Power", "Ein nach" )      ///< EIN, wenn digitaler Eingang x min. 1s HIGH ist und eingestellter Timeout abgelaufen. AUS wenn digitaler Eingang x wieder min. 1s LOW ist.
// #define TEXT_POWER_ITEM_NR_STANDBY                    QT_TR_NOOP( "Power", "Standby" )       ///< EIN, nur bei Bedarf. Verzoegerungen durch Hochlaufzeit moeglich.
// #define TEXT_POWER_ITEM_NR_TB_SLOT                    QT_TR_NOOP( "Power", "Docking" )       ///< EIN, wenn digitaler Eingang x min. 1s HIGH wird. AUS wenn digitaler Eingang x wieder min. 1s LOW ist und eingestellter Timeout abgelaufen.

//Module, Baugruppen
#define POWER_MODULE_NR_CPU                      1  ///< CPU
#define POWER_MODULE_NR_TIMER                    2  ///< Timer 
#define POWER_MODULE_NR_INPUTS                   3  ///< Eingänge
#define POWER_MODULE_NR_GLOBAL                   4  ///< Global Alles
#define POWER_MODULE_NR_12V_EXT                  5  ///< 12V extern
#define POWER_MODULE_NR_5V_EXT                   6  ///< 5V extern
#define POWER_MODULE_NR_COMMUNICATION            7  ///< Kommunikation
#define POWER_MODULE_NR_DISPLAY_LIGHT            8  ///< Display Beleuchtung
#define POWER_MODULE_NR_DISPLAY_TEXT             9  ///< Display Text
#define POWER_MODULE_NR_FINGERPRINT              10 ///< Biokey
#define POWER_MODULE_NR_TP                       11 ///< Transponder
#define POWER_MODULE_NR_GPS                      12 ///< GPS
#define POWER_MODULE_NR_USBHOST                  13 ///< USB-Host
#define POWER_MODULE_NR_MMC                      14 ///< MMC

//#define TEXT_MODULE_NR_SYSTEM                    _T("System")                   ///< NA
//#define TEXT_MODULE_NR_CPU                       _T("CPU")                      ///< CPU
//#define TEXT_MODULE_NR_TIMER                     _T("Timer")                    ///< Timer 
//#define TEXT_MODULE_NR_INPUTS                    _T("Eingänge")                 ///< Eingänge
//#define TEXT_MODULE_NR_GLOBAL                    _T("Global Alles")             ///< Global Alles
//#define TEXT_MODULE_NR_12V_EXT                   _T("12V extern")               ///< 12V extern
//#define TEXT_MODULE_NR_5V_EXT                    _T("5V extern")                ///< 5V extern
//#define TEXT_MODULE_NR_COMMUNICATION             _T("Kommunikation")            ///< Kommunikation
//#define TEXT_MODULE_NR_DISPLAY_LIGHT             _T("Display Beleuchtung")      ///< Display Beleuchtung
//#define TEXT_MODULE_NR_DISPLAY_TEXT              _T("Display Text")             ///< Display Text
//#define TEXT_MODULE_NR_FINGERPRINT               _T("Biokey")                   ///< Biokey
//#define TEXT_MODULE_NR_TP                        _T("Transponder")              ///< Transponder
//#define TEXT_MODULE_NR_GPS                       _T("GPS")                      ///< GPS
//#define TEXT_MODULE_NR_USBHOST                   _T("USB-Host")                 ///< USB-Host
//#define TEXT_MODULE_NR_MMC                       _T("MMC")                      ///< MMC
// #define TEXT_MODULE_NR_SYSTEM                    QT_TR_NOOP( "Power", "System" )                   ///< NA
// #define TEXT_MODULE_NR_CPU                       QT_TR_NOOP( "Power", "CPU" )                      ///< CPU
// #define TEXT_MODULE_NR_TIMER                     QT_TR_NOOP( "Power", "Timer" )                    ///< Timer 
// #define TEXT_MODULE_NR_INPUTS                    QT_TR_NOOP( "Power", "Eingänge" )                 ///< Eingänge
// #define TEXT_MODULE_NR_GLOBAL                    QT_TR_NOOP( "Power", "Global Alles" )             ///< Global Alles
// #define TEXT_MODULE_NR_12V_EXT                   QT_TR_NOOP( "Power", "12V extern" )               ///< 12V extern
// #define TEXT_MODULE_NR_5V_EXT                    QT_TR_NOOP( "Power", "5V extern" )                ///< 5V extern
// #define TEXT_MODULE_NR_COMMUNICATION             QT_TR_NOOP( "Power", "Kommunikation" )            ///< Kommunikation
// #define TEXT_MODULE_NR_DISPLAY_LIGHT             QT_TR_NOOP( "Power", "Display Beleuchtung" )      ///< Display Beleuchtung
// #define TEXT_MODULE_NR_DISPLAY_TEXT              QT_TR_NOOP( "Power", "Display Text" )             ///< Display Text
// #define TEXT_MODULE_NR_FINGERPRINT               QT_TR_NOOP( "Power", "Biokey" )                   ///< Biokey
// #define TEXT_MODULE_NR_TP                        QT_TR_NOOP( "Power", "Transponder" )              ///< Transponder
// #define TEXT_MODULE_NR_GPS                       QT_TR_NOOP( "Power", "GPS" )                      ///< GPS
// #define TEXT_MODULE_NR_USBHOST                   QT_TR_NOOP( "Power", "USB-Host" )                 ///< USB-Host
// #define TEXT_MODULE_NR_MMC                       QT_TR_NOOP( "Power", "MMC" )                      ///< MMC

//Flags Module
#define POWER_MODULE_CPU                         0x00000001 ///< CPU
#define POWER_MODULE_TIMER                       0x00000002 ///< Timer 
#define POWER_MODULE_INPUTS                      0x00000004 ///< Eingänge
#define POWER_MODULE_GLOBAL                      0x00000008 ///< Global Alles
#define POWER_MODULE_5V_EXT                      0x00000010 ///< 5V extern
#define POWER_MODULE_12V_EXT                     0x00000020 ///< 12V extern
#define POWER_MODULE_COMMUNICATION               0x00000040 ///< Kommunikation
#define POWER_MODULE_DISPLAY_LIGHT               0x00000080 ///< Display Beleuchtung
#define POWER_MODULE_DISPLAY_TEXT                0x00000100 ///< Display Text
#define POWER_MODULE_FINGERPRINT                 0x00000200 ///< Biokey
#define POWER_MODULE_TP                          0x00000400 ///< Transponder
#define POWER_MODULE_GPS                         0x00000800 ///< GPS
#define POWER_MODULE_USBHOST                     0x00001000 ///< USB-Host
#define POWER_MODULE_MMC                         0x00002000 ///< MMC

//Events
#define POWER_EVENT_NR_TIMER_A                   0x01 ///< Systemtimer Typ A mit Aktivzeit
#define POWER_EVENT_NR_TIMER_B                   0x02 ///< Systemtimer Typ B mit Uhrzeit
#define POWER_EVENT_NR_KEY_PRESSED               0x03 ///< Eingabeereignis(Tastatur, Biokey, Transponder, Barcode)
#define POWER_EVENT_NR_KEY_TIMEOUT_1             0x04 ///< Eingabe TimeOut 1
#define POWER_EVENT_NR_KEY_TIMEOUT_2             0x05 ///< Eingabe TimeOut 2
#define POWER_EVENT_NR_USB                       0x06 ///< Stromversorgung über USB aktiviert
#define POWER_EVENT_NR_BATTERY                   0x07 ///< Stromversorgung über Batterie aktiviert
#define POWER_EVENT_NR_LINE_IN                   0x08 ///< Stromversorgung über Netzteil aktiviert
#define POWER_EVENT_NR_BAT_LOW                   0x09 ///< Batteriezustand niedrig
#define POWER_EVENT_NR_BAT_EMPTY                 0x0A ///< Batteriezustand leer
#define POWER_EVENT_NR_SWITCH_ON                 0x0B ///< Ein-/Ausschalter schaltet EIN
#define POWER_EVENT_NR_SWITCH_OFF                0x0C ///< Ein-/Ausschalter schaltet AUS
#define POWER_EVENT_NR_TB_PLUGIN                 0x0D ///< Timeboy-Steck-Event
#define POWER_EVENT_NR_TB_PLUGOUT                0x0E ///< Timeboy-Zieh-Event
#define POWER_EVENT_NR_MDE_START                 0x0F ///< Start-Erkennung MDE
#define POWER_EVENT_NR_MDE_STOP                  0x10 ///< Stopp-Erkennung MDE
#define POWER_EVENT_NR_ANALOG_OVER_VALUE         0x11 ///< Analogwertüberwachung Messwert Überschreitung
#define POWER_EVENT_NR_ANALOG_UNDER_VALUE        0x12 ///< Analogwertüberwachung Messwert Unterschreitung
#define POWER_EVENT_NR_MOVE_START                0x13 ///< Bewegungserkennung Start
#define POWER_EVENT_NR_MOVE_STOP                 0x14 ///< Bewegungserkennung Stopp
#define POWER_EVENT_NR_COMMUNICATION             0x15 ///< Kommunikation
#define POWER_EVENT_NR_COMMUNICATION_TIMEOUT     0x16 ///< KommunikationsTimeOut

//#define TEXT_EVENT_NR_TIMER_A                   _T("Systemtimer -> Timeout")                  ///< Systemtimer Typ A mit Aktivzeit
//#define TEXT_EVENT_NR_TIMER_B                   _T("Systemtimer -> Uhrzeit")                  ///< Systemtimer Typ B mit Uhrzeit
//#define TEXT_EVENT_NR_KEY_PRESSED               _T("Eingabe -> Tastatur, Transponder etc.")   ///< Eingabeereignis(Tastatur, Biokey, Transponder, Barcode)
//#define TEXT_EVENT_NR_KEY_TIMEOUT_1             _T("Eingabe -> Timeout")                      ///< Eingabe TimeOut 1
//#define TEXT_EVENT_NR_KEY_TIMEOUT_2             _T("Eingabe -> Timeout 2")                    ///< Eingabe TimeOut 2
//#define TEXT_EVENT_NR_USB                       _T("Stromversorgung -> USB")                  ///< Stromversorgung über USB aktiviert
//#define TEXT_EVENT_NR_BATTERY                   _T("Stromversorgung -> Akku")                 ///< Stromversorgung über Batterie aktiviert
//#define TEXT_EVENT_NR_LINE_IN                   _T("Stromversorgung -> Netzteil")             ///< Stromversorgung über Netzteil aktiviert
//#define TEXT_EVENT_NR_BAT_LOW                   _T("Ladezustand Akku -> Niedrig")             ///< Batteriezustand niedrig
//#define TEXT_EVENT_NR_BAT_EMPTY                 _T("Ladezustand Akku -> Leer")                ///< Batteriezustand leer
//#define TEXT_EVENT_NR_SWITCH_ON                 _T("Ein/Aus-Schalter -> EIN")                 ///< Ein-/Ausschalter schaltet EIN
//#define TEXT_EVENT_NR_SWITCH_OFF                _T("Ein/Aus-Schalter -> AUS")                 ///< Ein-/Ausschalter schaltet AUS
//#define TEXT_EVENT_NR_TB_PLUGIN                 _T("Timeboy -> Einstecken")                   ///< Timeboy-Steck-Event
//#define TEXT_EVENT_NR_TB_PLUGOUT                _T("Timeboy -> Herausziehen")                 ///< Timeboy-Zieh-Event
//#define TEXT_EVENT_NR_MDE_START                 _T("Digitaler Eingang: Low->High")            ///< Start-Erkennung MDE
//#define TEXT_EVENT_NR_MDE_STOP                  _T("Digitaler Eingang: High->Low")            ///< Stopp-Erkennung MDE
//#define TEXT_EVENT_NR_ANALOG_OVER_VALUE         _T("Analogwert -> Überschreitung")            ///< Analogwertüberwachung Messwert Überschreitung
//#define TEXT_EVENT_NR_ANALOG_UNDER_VALUE        _T("Analogwert -> Unterschreitung")           ///< Analogwertüberwachung Messwert Unterschreitung
//#define TEXT_EVENT_NR_MOVE_START                _T("Bewegungserkennung -> Start")             ///< Bewegungserkennung Start
//#define TEXT_EVENT_NR_MOVE_STOP                 _T("Bewegungserkennung -> Stopp")             ///< Bewegungserkennung Stopp
//#define TEXT_EVENT_NR_COMMUNICATION             _T("Kommunikation")                           ///< Kommunikation
//#define TEXT_EVENT_NR_COMMUNICATION_TIMEOUT     _T("Kommunikation -> Ablauf Timeout")         ///< KommunikationsTimeOut
// #define TEXT_EVENT_NR_TIMER_A                   QT_TR_NOOP( "Power", "Systemtimer -> Timeout" )                  ///< Systemtimer Typ A mit Aktivzeit
// #define TEXT_EVENT_NR_TIMER_B                   QT_TR_NOOP( "Power", "Systemtimer -> Uhrzeit" )                  ///< Systemtimer Typ B mit Uhrzeit
// #define TEXT_EVENT_NR_KEY_PRESSED               QT_TR_NOOP( "Power", "Eingabe -> Tastatur, Transponder etc." )   ///< Eingabeereignis(Tastatur, Biokey, Transponder, Barcode)
// #define TEXT_EVENT_NR_KEY_TIMEOUT_1             QT_TR_NOOP( "Power", "Eingabe -> Timeout" )                      ///< Eingabe TimeOut 1
// #define TEXT_EVENT_NR_KEY_TIMEOUT_2             QT_TR_NOOP( "Power", "Eingabe -> Timeout 2" )                    ///< Eingabe TimeOut 2
// #define TEXT_EVENT_NR_USB                       QT_TR_NOOP( "Power", "Stromversorgung -> USB" )                  ///< Stromversorgung über USB aktiviert
// #define TEXT_EVENT_NR_BATTERY                   QT_TR_NOOP( "Power", "Stromversorgung -> Akku" )                 ///< Stromversorgung über Batterie aktiviert
// #define TEXT_EVENT_NR_LINE_IN                   QT_TR_NOOP( "Power", "Stromversorgung -> Netzteil" )             ///< Stromversorgung über Netzteil aktiviert
// #define TEXT_EVENT_NR_BAT_LOW                   QT_TR_NOOP( "Power", "Ladezustand Akku -> Niedrig" )             ///< Batteriezustand niedrig
// #define TEXT_EVENT_NR_BAT_EMPTY                 QT_TR_NOOP( "Power", "Ladezustand Akku -> Leer" )                ///< Batteriezustand leer
// #define TEXT_EVENT_NR_SWITCH_ON                 QT_TR_NOOP( "Power", "Ein/Aus-Schalter -> EIN" )                 ///< Ein-/Ausschalter schaltet EIN
// #define TEXT_EVENT_NR_SWITCH_OFF                QT_TR_NOOP( "Power", "Ein/Aus-Schalter -> AUS" )                 ///< Ein-/Ausschalter schaltet AUS
// #define TEXT_EVENT_NR_TB_PLUGIN                 QT_TR_NOOP( "Power", "Timeboy -> Einstecken" )                   ///< Timeboy-Steck-Event
// #define TEXT_EVENT_NR_TB_PLUGOUT                QT_TR_NOOP( "Power", "Timeboy -> Herausziehen" )                 ///< Timeboy-Zieh-Event
// #define TEXT_EVENT_NR_MDE_START                 QT_TR_NOOP( "Power", "Digitaler Eingang: Low->High" )            ///< Start-Erkennung MDE
// #define TEXT_EVENT_NR_MDE_STOP                  QT_TR_NOOP( "Power", "Digitaler Eingang: High->Low" )            ///< Stopp-Erkennung MDE
// #define TEXT_EVENT_NR_ANALOG_OVER_VALUE         QT_TR_NOOP( "Power", "Analogwert -> Überschreitung" )            ///< Analogwertüberwachung Messwert Überschreitung
// #define TEXT_EVENT_NR_ANALOG_UNDER_VALUE        QT_TR_NOOP( "Power", "Analogwert -> Unterschreitung" )           ///< Analogwertüberwachung Messwert Unterschreitung
// #define TEXT_EVENT_NR_MOVE_START                QT_TR_NOOP( "Power", "Bewegungserkennung -> Start" )             ///< Bewegungserkennung Start
// #define TEXT_EVENT_NR_MOVE_STOP                 QT_TR_NOOP( "Power", "Bewegungserkennung -> Stopp" )             ///< Bewegungserkennung Stopp
// #define TEXT_EVENT_NR_COMMUNICATION             QT_TR_NOOP( "Power", "Kommunikation" )                           ///< Kommunikation
// #define TEXT_EVENT_NR_COMMUNICATION_TIMEOUT     QT_TR_NOOP( "Power", "Kommunikation -> Ablauf Timeout" )         ///< KommunikationsTimeOut

//Flags der Energieverwaltung
#define POWER_EVENT_TIMER_A                      0x00000001 ///< Systemtimer Typ A mit Aktivzeit
#define POWER_EVENT_TIMER_B                      0x00000002 ///< Systemtimer Typ B mit Uhrzeit
#define POWER_EVENT_KEY_PRESSED                  0x00000004 ///< Eingabeereignis(Tastatur, Biokey, Transponder, Barcode)
#define POWER_EVENT_KEY_TIMEOUT_1                0x00000008 ///< Eingabe TimeOut 1
#define POWER_EVENT_KEY_TIMEOUT_2                0x00000010 ///< Eingabe TimeOut 2
#define POWER_EVENT_USB                          0x00000020 ///< Stromversorgung über USB aktiviert
#define POWER_EVENT_BATTERY                      0x00000040 ///< Stromversorgung über Batterie aktiviert
#define POWER_EVENT_LINE_IN                      0x00000080 ///< Stromversorgung über Netzteil aktiviert
#define POWER_EVENT_BAT_LOW                      0x00000100 ///< Batteriezustand niedrig
#define POWER_EVENT_BAT_EMPTY                    0x00000200 ///< Batteriezustand leer
#define POWER_EVENT_SWITCH_ON                    0x00000400 ///< Ein-/Ausschalter schaltet EIN
#define POWER_EVENT_SWITCH_OFF                   0x00000800 ///< Ein-/Ausschalter schaltet AUS
#define POWER_EVENT_TB_PLUGIN                    0x00001000 ///< Timeboy-Steck-Event
#define POWER_EVENT_TB_PLUGOUT                   0x00002000 ///< Timeboy-Zieh-Event
#define POWER_EVENT_MDE_START                    0x00004000 ///< Start-Erkennung MDE
#define POWER_EVENT_MDE_STOP                     0x00008000 ///< Stopp-Erkennung MDE
#define POWER_EVENT_ANALOG_OVER_VALUE            0x00010000 ///< Analogwertüberwachung Messwert Überschreitung
#define POWER_EVENT_ANALOG_UNDER_VALUE           0x00020000 ///< Analogwertüberwachung Messwert Unterschreitung
#define POWER_EVENT_MOVE_START                   0x00040000 ///< Bewegungserkennung Start
#define POWER_EVENT_MOVE_STOP                    0x00080000 ///< Bewegungserkennung Stopp
#define POWER_EVENT_COMMUNICATION                0x00100000 ///< Kommunikation
#define POWER_EVENT_COMMUNICATION_TIMEOUT        0x00200000 ///< KommunikationsTimeOut

/**
@struct POWERSETUPNODE enthält Einstellungen und Adressen der Energieschemen
*/

typedef struct struct_power_setup_node {

    unsigned char version;             ///< Versionserkennung
    unsigned char cntProfile;          ///< Anzahl der benutzten Schemen
    unsigned char cntEvents;           ///< Anzahl der benutzten Events
    unsigned char active;              ///< Schema welches nach dem Start des Setup zuerst aktiviert werden soll
    unsigned char temp[2];             ///< Reserve
    unsigned int ptrProfileArray;      ///< Zeiger auf das 1. vorhandene Schema mit der Struktur POWERSETUP
    unsigned int ptrEventArray;        ///< Zeiger auf das 1. vorhandene Event mit der Struktur POWEREVENT

} POWERSETUPNODE;

/**
@struct POWERMODULE enthält Einstellungen des einzenlen Moduls
*/

typedef struct struct_power_module {

    unsigned char item;                ///< Auswahl der Energieoption EIN, AUS usw.
    unsigned int time;                 ///< Wert der Zeit in Sekunden
    unsigned char value;               ///< Wert für Helligkeit oder andere Größen
    unsigned int interval;             ///< Wert des Einschaltintervall der Zeit in Sekunden (value = 0) oder als Uhrzeit (value = 1)

} POWERMODULE;

/**
@struct POWERSETUP enthält Einstellungen aller Module
*/

typedef struct struct_power_setup
{
    unsigned char name[32];            ///< Name des Schemas
    unsigned char temp[2];             ///< Reserve  
    POWERMODULE cpu;                   ///< Einstellungen für Mircrocontroller
    POWERMODULE timer;                 ///< Einstellungen für Timer der Signalverarbeitung
    POWERMODULE inputs;                ///< Einstellungen für Eingänge der Signalverarbeitung
    POWERMODULE deviceGlobal;          ///< Einstellungen für Globale Spannungsversorgung der Baugruppe
    POWERMODULE U5V;                   ///< Einstellungen für 5V extern
    POWERMODULE U12V;                  ///< Einstellungen für 12V extern
    POWERMODULE communication;         ///< Einstellungen für Kommunikation
    POWERMODULE displayLight;          ///< Einstellungen für Displaybeleuchtung
    POWERMODULE displayText;           ///< Einstellungen für Display Ein/Aus
    POWERMODULE fingerprint;           ///< Einstellungen für Fingerprintmodul Biokey3000
    POWERMODULE transponder;           ///< Einstellungen für Transponderleser
    POWERMODULE gps;                   ///< Einstellungen für GPS Modul Lea 5H von ublox
    POWERMODULE usbHost;               ///< Einstellungen für USB-Host VDrive2
    POWERMODULE mmc;                   ///< Einstellungen für interne MMC/SD-Karte

} POWERSETUP;


/**
@struct POWEREVENT enthält die Definition und Einstellungen eines Events
*/

typedef struct struct_power_event {

    unsigned char type;                ///< Eventtyp
    unsigned char profile;             ///< Event gültig für Energieschema Nr.
    unsigned int time;                 ///< Wert der Zeit in Sekunden, auch bei Format Uhrzeit
    unsigned char nextprofile;         ///< nächstes Energieschema nach Ablauf der Aktivzeit
    unsigned char value;               ///< Wert für Analogwertüberwachung
    unsigned char channel;             ///< Eingang für den Analogwert
    unsigned char tmp[2];              ///< Reserve ab V2 der Strukturen

} POWEREVENT;



typedef struct struct_power_flags
{

    unsigned char busy;                ///< Modul arbeitet gerade, TimeOut-Zähler = 0
    unsigned int timestamp;            ///< Zeitwert ab wann Modul nicht mehr arbeitet
    unsigned int state;                ///< Modul Ein(TRUE) oder Aus(FALSE)

} POWERFLAGS;

typedef struct struct_power_state
{
    POWERFLAGS cpu;                    ///< Status für Mircrocontroller
    POWERFLAGS timer;                  ///< Status für Timer der Signalverarbeitung
    POWERFLAGS inputs;                 ///< Status für Eingänge der Signalverarbeitung
    POWERFLAGS deviceGlobal;           ///< Status für Globale Spannungsversorgung der Baugruppe
    POWERFLAGS U5V;                    ///< Status für 5V extern
    POWERFLAGS U12V;                   ///< Status für 12V extern
    POWERFLAGS communication;          ///< Status für Kommunikation
    POWERFLAGS displayLight;           ///< Status für Displaybeleuchtung
    POWERFLAGS displayText;            ///< Status für Display Ein/Aus
    POWERFLAGS fingerprint;            ///< Status für Fingerprintmodul Biokey3000
    POWERFLAGS transponder;            ///< Status für Transponderleser
    POWERFLAGS gps;                    ///< Status für GPS Modul Lea 5H von ublox
    POWERFLAGS usbHost;                ///< Status für USB-Host VDrive2
    POWERFLAGS mmc;                    ///< Status für interne MMC/SD-Karte

} POWERSTATE;

typedef struct struct_event_timers
{

    unsigned int timerA;               ///< Zeitwert des Systemtimers Typ A
    unsigned int timerB;               ///< Zeitwert des Systemtimers Typ B
    unsigned int timerService;         ///< Zeitwert ab wann keine Bedienung mehr erfolgte
    unsigned int timerTB;              ///< Zeitwert ab wann der Timeboy gesteckt wurde
    unsigned int timerMove;            ///< Zeitwert ab wann Gerät nicht mehr bewegt wurde
    unsigned int timerCom;             ///< Zeitwert ab wann Gerät nicht mehr kommunizierte

}POWEREVENTTIMERS;

/**
@struct POWERPROPERTIES enthält die wichtigsten Grundeinstellungen des Energiemanagements
*/

typedef struct struct_power_properties
{
    unsigned char profileMaxCount;     ///< Anzahl der benutzbaren Schemen
    unsigned char activeProfile;       ///< akuelles Energieschema
    POWERSETUP profile;                ///< Struktur der einzelnen Module mit ihren Optionen
    unsigned int moduleBusy;           ///< Modul arbeitet gerade
    unsigned int moduleIdle;           ///< Modul im Leerlauf
    POWERSTATE modules;                ///< Struktur der einzelnen Module mit aktuellen Werten
    unsigned char eventMaxCount;       ///< Anzahl der Events im Setup
    unsigned char eventCount;          ///< Anzahl der Events im aktuellen Schema
    unsigned int eventStates;          ///< Energie-Events der Module
    POWEREVENTTIMERS timeEvent;        ///< Event-Timer einzelner Module

} POWERPROPERTIES;

#pragma pack()

#endif
