/**
* @file  df_log.cpp
* @brief Implementierung der Klasse CDFLog.
*/
//////////////////////////////////////////////////////////////////////

#include "log/df_log.h"

#if defined(_MSC_VER) || defined(__MINGW32__)
#ifndef WIN32_LEAN_AND_MEAN
#define WIN32_LEAN_AND_MEAN
#endif
#include <windows.h>
#include <direct.h>
#include <io.h>
#else
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>
#endif

#include <fstream>
#include <cstdio>
#include <cstdarg>
#include <cstring>
#include <array>
#include <sstream>
#include <thread>
#include <boost/algorithm/string.hpp>

CDFLog::CDFLog( int connection_id, std::function<void( const std::string & )> change_trace_file_path ) :
    cb_change_trace_file_path( change_trace_file_path )
{
    // Zugewiesene Kanalnummer
    m_connection_id = connection_id;
    m_log_check.m_connection_id = connection_id;

    // Bevorzugt auf die Dockingstation Type 2 prüfen.
    m_docking_prefer_type = 2;

    // Das Oeffnen und Schliessen muss durch den selben Thread erfolgen.
    m_check_open_and_close_by_same_thread = true;

    // Initialisierungsdatei lesen
    //logIniRead(); // führt auch logCheck durch.
}

CDFLog::~CDFLog()
{
}

std::string CDFLog::find_ini_file( bool create_if_not_exist )
{
    std::string ini_file_path;
    // Suchen der Initialisierungsdatei.
    // 1. Bei dem Executable.
    // 2. Umgebungsvariable DFCOM_INI_PATH
    // 3. Umgebungsvariable LOCALAPPDATA unter Windows oder HOME unter GNU/Linux.

    // 1. Versuchen die DFCom.ini in dem Verzeichnis der Anwendung zu finden.
    std::string executable_file_path( 260, '\0' );

#if defined(_MSC_VER) || defined(__MINGW32__)
    // Datei der Anwendung
    if ( GetModuleFileNameA( nullptr, &executable_file_path[0], (DWORD)executable_file_path.size() ) == 0 )
    {
        // Fehler bei Ermittlung
        executable_file_path.clear();
    }
    else
    {
        // Windows akzeptiert auch '/' als Pfadtrenner, somit Gemeinsamkeit mit GNU/Linux herstellen.
        boost::replace_all( executable_file_path, "\\", "/" );
    }
#else
    // Datei der Anwendung
    std::string tmp( 260, '\0' );
    std::sprintf( &tmp[0], "/proc/%d/exe", getpid() );
    if ( readlink( tmp.c_str(), &executable_file_path[0], executable_file_path.size() ) == -1 )
    {
        // Fehler bei Ermittlung
        executable_file_path.clear();
    }
#endif
    // Prüfen ob Inidatei bei Anwendung existiert.
    if ( executable_file_path.size() )
    {
        auto pos = executable_file_path.find_first_of( '\0' );
        if ( pos != std::string::npos )
        {
            // Stringgröße an Inhalt anpassen.
            executable_file_path.resize( pos );
            pos = executable_file_path.find_last_of( '/' );
            if ( pos != std::string::npos )
            {
                ini_file_path = executable_file_path;
                ini_file_path.resize( pos + 1 );
                ini_file_path += "DFCom.ini";

                // Prüfen ob die Datei dort existiert.
                struct stat lstat;
                if ( stat( ini_file_path.c_str(), &lstat ) == -1 || (lstat.st_mode & S_IFMT) != S_IFREG )
                {
                    ini_file_path.clear();
                }
            }
        }
    }

    if ( !ini_file_path.size() )
    {
        // 2. Versuchen die DFCom.ini über die Umgebungsvariable DFCOM_INI_PATH zu finden.
        std::string env_file_path;
        char *dfcom_ini_path = std::getenv( "DFCOM_INI_PATH" );
        if ( nullptr != dfcom_ini_path )
        {
            env_file_path = dfcom_ini_path;
#if defined(_MSC_VER) || defined(__MINGW32__)
            // Windows akzeptiert auch '/' als Pfadtrenner, somit Gemeinsamkeit mit GNU/Linux herstellen.
            boost::replace_all( env_file_path, "\\", "/" );
#endif
            if ( env_file_path.size() )
            {
                if ( '/' == env_file_path[env_file_path.size() - 1] )
                {
                    env_file_path[env_file_path.size() - 1] = 0;
                }

                // Nur, wenn das Verzeichnis existiert und keine Ini vorliegt, wird eine Standard-Ini erzeugt.
                struct stat lstat;
                if ( stat( env_file_path.c_str(), &lstat ) == 0 && (lstat.st_mode & S_IFMT) == S_IFDIR )
                {
                    ini_file_path = env_file_path + "/DFCom.ini";
                    if ( stat( ini_file_path.c_str(), &lstat ) == -1 || (lstat.st_mode & S_IFMT) != S_IFREG )
                    {
                        if ( create_if_not_exist )
                        {
                            if ( !create_ini_file( ini_file_path.c_str(), env_file_path.c_str() ) )
                            {
                                ini_file_path.clear();
                            }
                        }
                        else
                        {
                            ini_file_path.clear();
                        }
                    }
                }
            }
        }
    }

    if ( !ini_file_path.size() )
    {
        // 3. Versuchen die DFCom.ini im App-Data oder Home-Verzeichnis zu finden.
        // Ggf. "create_if_not_exist" eine Standarddatei erzeugen.
        std::string env_file_path;
#if defined(_MSC_VER) || defined(__MINGW32__)
        char *dfcom_ini_path = std::getenv( "LOCALAPPDATA" );
        if ( nullptr != dfcom_ini_path )
        {
            env_file_path = dfcom_ini_path;
        }
        // Windows akzeptiert auch '/' als Pfadtrenner, somit Gemeinsamkeit mit GNU/Linux herstellen.
        boost::replace_all( env_file_path, "\\", "/" );
#else
        char *dfcom_ini_path = std::getenv( "HOME" );
        if ( nullptr != dfcom_ini_path )
        {
            env_file_path = dfcom_ini_path;
        }
#endif
        if ( env_file_path.size() )
        {
            if ( '/' == env_file_path[env_file_path.size() - 1] )
            {
                env_file_path[env_file_path.size() - 1] = 0;
            }

            // Prüfen ob das Verzeichnis existiert.
            struct stat lstat;
            if ( stat( env_file_path.c_str(), &lstat ) == 0 && (lstat.st_mode & S_IFMT) == S_IFDIR )
            {
                // Prüfen ob das Unterverzeichnis existiert.
                env_file_path += "/datafox";
                if ( stat( env_file_path.c_str(), &lstat ) == -1 || (lstat.st_mode & S_IFMT) != S_IFDIR )
                {
                    if ( create_if_not_exist )
                    {
#if defined(_MSC_VER) || defined(__MINGW32__)
                        if ( mkdir( env_file_path.c_str() ) != 0 )
#else
                        if ( mkdir( env_file_path.c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH ) != 0 )
#endif
                        {
                            env_file_path.clear();
                        }
                    }
                    else
                    {
                        env_file_path.clear();
                    }
                }

                if ( env_file_path.size() )
                {
                    ini_file_path = env_file_path;
                    ini_file_path += "/DFCom.ini";
                    if ( stat( ini_file_path.c_str(), &lstat ) == -1 || (lstat.st_mode & S_IFMT) != S_IFREG )
                    {
                        if ( create_if_not_exist )
                        {
                            if ( !create_ini_file( ini_file_path.c_str(), env_file_path.c_str() ) )
                            {
                                ini_file_path.clear();
                            }
                        }
                        else
                        {
                            ini_file_path.clear();
                        }
                    }
                }
            }
        }
    }

    return ini_file_path;
}

bool CDFLog::create_ini_file( const std::string &file_path, const std::string &log_base_path )
{
    std::ofstream out( file_path );
    // Wurde die Datei gültig erstellt.
    if ( out.is_open() )
    {
        // Sektion "Log" anlegen
        out << "[Log]" << std::endl;
#if !defined(_MSC_VER) && !defined(__MINGW32__)
        out << "Enable=0" << std::endl; // Unter Linux ist das Loggen standardmaessig aus.
        out << "; ACHTUNG: Das Loggen ist deaktiviert, da der Key -Erase- aktuell unter GNU/Linux nicht unterstuetzt wird." << std::endl;
        out << ";          Bei Aktivierung liegt das Loeschen der Logdateien in Ihren Haenden." << std::endl;
#endif
#if defined(_MSC_VER) || defined(__MINGW32__)
        out << "Enable=1" << std::endl;
#endif
        out << "Path=" << log_base_path << std::endl;
        out << "Folder=" << SUB_FOLDER << std::endl;
        out << "Trace=0" << std::endl;
        out << "Erase=1" << std::endl;
        out << "Keep=10" << std::endl;
        out << "Elapse=1" << std::endl;
        out << "List=1" << std::endl;

        out << std::endl << std::endl;
        out << "; Kurzbeschreibung der enthaltenen Parameter:" << std::endl;
        out << "; Enable: Aktivieren des Hintergrundloggens. Wertebereich: 0 = Nicht loggen, 1 = Loggen aktiviert." << std::endl;
        out << "; Path: Gibt das Basisverzeichnis an. Bei Verwendung von Backslash-Zeichen \\ muessen diese doppelt eingegeben werden." << std::endl;
        out << "; Folder: Das zu verwendende Unterverzeichnis, dieses wird wenn nicht bereits vorhanden angelegt." << std::endl;
        out << "; Trace: Gibt an ob das DLL eigene Detaillog aktiviert sein soll. Wertebereich: 0 = Deaktiviert, 1 = Aktiviert, 2 = Aktiviert mit Packetdaten.\n; \tBitte beachten Sie, dass hierbei groessere Datenmengen anfallen koennen." << std::endl;
        out << "; Erase: Gibt an ob die Logdateien automatisch geloescht werden, unter Linux werden die Logdateien nicht automatisch geloescht. Wertebreich: 1 = Automatisch loeschen, 0 = Manuell loeschen.\n; \tBitte beachten Sie dass die Dateiverwaltung nach Abschalten uebernommen werden muss." << std::endl;
        out << "; Keep: Gibt bei Erase=1 die Anzahl der zu haltenden Logtage an (nicht unter Linux gueltig). Wertebereich: 1 - 100 Logtage. Logtage bedeutet hier, Tage an denen geloggt wurde." << std::endl;
        out << "; Elapse: Gibt an ob die Ausfuehrungsdauer der jeweiligen Funktion in eckigen Klammern hinter jedem Logeintrag ausgegeben wird, dabei hat dieser folgende Aufteilung [Sekunden.Hundertstel]." << std::endl;
        out << "; List: Gibt an ob die uebertragenen Listendaten vor jedem uebertragen, in das Logverzeichnis exportiert werden.\n; \tWertebereich: 0 = Nein, 1 = In Binaerformat mitprotokollieren, 2 = In Binaerform und Textformat (tab-Getrennt)." << std::endl;

        // Sektion "Docking" anlegen
        out << std::endl << std::endl;
        out << "[Docking]" << std::endl;
        out << "Prefer=2" << std::endl;
        //out << "Autodetect=1" << std::endl;

        out << std::endl << std::endl;
        out << "; Kurzbeschreibung der enthaltenen Parameter:" << std::endl;
        out << "; Sektion [Docking] ist global gültig. Für eine Verbindungsspezifische Einstellung verwenden Sie bitte" << std::endl;
        out << "; einen Sektionsnamen [Docking_connectionId] z. B. [Docking_5] für die Verbindungs-Id 5." << std::endl;
        out << "; Prefer: Bevorzugte Version der Dockingstation. Wertebereich: 1 = Version 1, 2 = Version 2." << std::endl;
        //out << "; Autodetect: Automatisch die Version der Dockingstation ermitteln, angefangen wird die Ermittlung mit dem Wert von Prefer. Wertebereich: 0 = Aus, 1 = An." << std::endl;

        // Sektion "Docking" anlegen
        out << std::endl << std::endl;
        out << "[Channels]" << std::endl;
        out << "CheckOpenCloseBySameThread=1" << std::endl;
        //out << "Autodetect=1" << std::endl;

        out << std::endl << std::endl;
        out << "; Kurzbeschreibung der enthaltenen Parameter:" << std::endl;
        out << "; Sektion [Channels] ist global gueltig für das Verhalten der Verbindungsklassen." << std::endl;
        out << "; CheckOpenCloseBySameThread: Das Oeffnen und Schliessen der Verbindung, muss durch den selben Thread erfolgen. Wertebereich: 0 = Deaktiviert, 1 = Aktiviert (Default)." << std::endl;
    }

    return out.is_open();
}

#include <boost/algorithm/string.hpp>

void CDFLog::logIniRead()
{
    char section[20];
    std::string ini_file_path = find_ini_file();

    if ( !ini_file_path.size() )
    {
        // Fehler bei Ermittlung
        m_enabled = false;
        m_trace_mode = 0; // Dann auch Trace deaktivieren.
        m_erase_files = false;
        m_write_list = 0;
        return;
    }

    std::sprintf( section, "Docking_%i", m_connection_id );
    // Bevorzugt Dockingstation V1/2 ansprechen.
    m_docking_prefer_type = m_log_ini.GetUInt( section, "Prefer", 1234, ini_file_path.c_str() );
    if ( m_docking_prefer_type == 1234 ) // Ggf. globale Einstellung laden.
    {
        m_docking_prefer_type = m_log_ini.GetUInt( "Docking", "Prefer", 2, ini_file_path.c_str() );
    }

    if ( m_docking_prefer_type < 1 )
    {
        m_docking_prefer_type = 1;
    }
    else if ( m_docking_prefer_type > 2 )
    {
        m_docking_prefer_type = 2;
    }


    // Log aktiv?
    m_enabled = (m_log_ini.GetUInt( "Log", "Enable", 1, ini_file_path.c_str() )) ? true : false;
    if ( !m_enabled )
    {
        // Abbruch durch Deaktivierung.
        m_trace_mode = 0; // Dann auch Trace deaktivieren.
        m_erase_files = false;
        m_write_list = 0;
        return;
    }
    // Trace aktiv?
    m_trace_mode = m_log_ini.GetUInt( "Log", "Trace", 0, ini_file_path.c_str() );

    // Basispfad fuer Log
    std::memset( m_log_check.m_base_path, 0, sizeof( m_log_check.m_base_path ) );
    m_log_ini.GetString( "Log", "Path", m_log_check.m_base_path, ini_file_path.c_str() );
    if ( m_log_check.m_base_path[0] == 0 ) 
    {
        std::string log_base_path( ini_file_path );
        auto pos = log_base_path.find_last_of( '/' );
        if ( pos != std::string::npos )
        {
            log_base_path.resize( pos );
        }
        // Verzeichnis der Ini als Basis
        std::strcpy( m_log_check.m_base_path, log_base_path.c_str() );
    } 
    else 
    {
        std::string log_base_path( m_log_check.m_base_path );
#if defined(_MSC_VER) || defined(__MINGW32__)
        // Windows akzeptiert auch '/' als Pfadtrenner, somit Gemeinsamkeit mit GNU/Linux herstellen.
        boost::replace_all( log_base_path, "\\", "/" );
#endif
        if ( '/' == log_base_path[log_base_path.size() - 1] )
        {
            log_base_path[log_base_path.size() - 1] = 0;
        }

        // Aufbereitetes Verzeichnis als Basis
        std::strcpy( m_log_check.m_base_path, log_base_path.c_str() );
    }

    // Unterordner fuer Logdateien
    std::memset( m_log_check.m_sub_folder, 0, sizeof( m_log_check.m_sub_folder ) );
    m_log_ini.GetString( "Log", "Folder", m_log_check.m_sub_folder, ini_file_path.c_str() );
    if ( m_log_check.m_sub_folder[0] == 0 )
    {
        std::strcpy( m_log_check.m_sub_folder, SUB_FOLDER );
    }

    std::string log_path( m_log_check.m_base_path );
    log_path += '/';
    log_path += m_log_check.m_sub_folder;
    if ( '/' == log_path[log_path.size() - 1] )
    {
        log_path[log_path.size() - 1] = 0;
    }

    // pruefen und evnt. anlegen des Ordners
    struct stat lstat;
    if ( stat( log_path.c_str(), &lstat ) != 0 || (lstat.st_mode & S_IFMT) != S_IFDIR )
    {
#if defined(_MSC_VER) || defined(__MINGW32__)
        if ( mkdir( log_path.c_str() ) != 0 )
        {
#else
        if ( mkdir( log_path.c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH) != 0 )
        {
#endif
            // Fehler bei Ermittlung
            m_enabled = false;
            m_trace_mode = 0; // Dann auch Trace deaktivieren.
            m_erase_files = false;
            m_write_list = 0;
            return;
        }
    }

    // Schreibrecht prüfen.
    if ( access( log_path.c_str(), 2 ) == -1 )
    {
        // Fehler bei Ermittlung
        m_enabled = false;
        m_trace_mode = 0; // Dann auch Trace deaktivieren.
        m_erase_files = false;
        m_write_list = 0;
        return;
    }

    // Alte Dateien loeschen
    m_erase_files = (m_log_ini.GetUInt( "Log", "Erase", 1, ini_file_path.c_str() )) ? true : false;
    
    m_log_check.m_erase_file_days = m_log_ini.GetUInt( "Log", "Keep", 10, ini_file_path.c_str() );
    if ( m_log_check.m_erase_file_days < 1 )
    {
        m_log_check.m_erase_file_days = 1;
    }
    if ( m_log_check.m_erase_file_days > 100 )
    {
        m_log_check.m_erase_file_days = 100;
    }

    // Ausfuehrungszeit angeben
    m_elapse = (m_log_ini.GetUInt( "Log", "Elapse", 1, ini_file_path.c_str() )) ? true : false;
    // Listen wegschreiben
    m_write_list = m_log_ini.GetUInt( "Log", "List", 1, ini_file_path.c_str() );

    // Sektion [Channels]
    // ----------------------
    // Oeffnen und Schliessen muss durch den selben Thread erfolgen.
    m_check_open_and_close_by_same_thread = (m_log_ini.GetUInt( "Channels", "CheckOpenCloseBySameThread", 1, ini_file_path.c_str() )) ? true : false;

    // Dateinamen für Log und Trace setzen, alte Logdateien ggf. loeschen.
    check_current_file_paths();
}

void CDFLog::logCheck() 
{
    // Soll durchgefuehrt werden?
    if ( !m_enabled || !m_erase_files )
    {
        return;
    }

#if defined(_MSC_VER) || defined(__MINGW32__)
    /*
    // Suche nach Dateien
    sprintf(szFile, LOG_SEARCH_FILENAME, m_szBasePath, m_szSubFolder);
    intptr_t fp;
    struct finddata_t finddata;
    if ((fp = findfirst(szFile, &finddata)) != -1) {

    do {

    for (i=0; i < 5; i++) {
    // Stimmt mit einem der zu erhaltenden Dateien ueberein?
    if (strstr(szFileName[i], finddata.name) != 0) {
    break;
    }
    }
    if (i == 5) {
    // Sammeln der zu loeschenden Dateien,
    // da erst nach findclose geloescht werden kann!
    strcpy(szDeleteFileName[j++], finddata.name);
    } 

    } while ((j < 5) && (findnext(fp, &finddata) != -1));
    findclose(fp);

    if (j) {
    // Dateien loeschen
    for (i=0; i < j; i++) {

    sprintf(szFile, LOG_DELETE_FILENAME, m_szBasePath, m_szSubFolder, szDeleteFileName[i]);
    if (remove(szFile) == -1) {
    // Konnte nicht geloescht werden!
    }
    }
    }
    }
    */
    m_log_check.Check();

#else // 2011_02_03: VORSICHT: Unter Linux werden die Log-Dateien nicht gelöscht.
    /*
    int i, j;
    char szFile[260];
    char szFileName[5][260];
    char szDeleteFileName[5][260];

    // Namen der Logdateien fuer die letzten fuenf Tage erstellen.
    for (i=0; i < 5; i++) {
    // Dateiname der Logdatei
    time_t t = time(NULL);
    t -= 3600 * 24 * i;
    tm *ltm = localtime(&t);
    sprintf(szFileName[i], LOG_FILENAME, m_logCheck.m_szBasePath, m_logCheck.m_szSubFolder, 
    ltm->tm_year % 100, ltm->tm_mon + 1, ltm->tm_mday, m_channelID);

    //@LOG:
    printf("Halte Datei: %s\n", szFileName[i]);
    }
    // Alle anderen Logdateien loeschen 

    j = 0;

    // Suche nach Dateien
    sprintf(szFile, "%s%s", m_logCheck.m_szBasePath, m_logCheck.m_szSubFolder);

    DIR *dir;
    if ((dir = opendir(szFile)) != NULL) {

    struct dirent *ent;
    while((j < 5) && ((ent = readdir(dir)) != NULL)) {

    if (ent->d_type != DT_REG && ent->d_type != DT_UNKNOWN)
    {
    continue;
    }
    for (i=0; i < 5; i++) {
    // Stimmt mit einem der zu erhaltenden Dateien ueberein?
    if (strstr(szFileName[i], ent->d_name) != 0) {
    //@LOG:
    printf("Datei wird gehalten: %s\n", szFileName[i]);
    break;
    }
    }
    if (i == 5) {
    // Sammeln der zu loeschenden Dateien,
    // da erst nach closedir geloescht werden kann!
    strcpy(szDeleteFileName[j++], ent->d_name);
    } 
    }
    closedir(dir);

    if (j) {
    // Dateien loeschen
    for (i=0; i < j; i++) {

    sprintf(szFile, LOG_DELETE_FILENAME, m_logCheck.m_szBasePath, m_logCheck.m_szSubFolder, szDeleteFileName[i]);
    //@LOG:
    printf("Datei wird gelöscht: %s\n", szFile);
    if (remove(szFile) == -1) {
    // Konnte nicht geloescht werden!
    }
    }
    }
    }
    */
#endif

}

std::string CDFLog::get_current_log_file_path()
{
    std::array<char, 260> file_path;

    // Datum
    time_t t = std::time( nullptr );
    tm *ltm = localtime( &t );

    // Dateiname
    snprintf( file_path.data(), file_path.size() - 1, LOG_FILENAME, m_log_check.m_base_path, m_log_check.m_sub_folder,
        ltm->tm_year % 100, ltm->tm_mon + 1, ltm->tm_mday, m_connection_id );

    return file_path.data();
}

std::string CDFLog::get_current_trace_file_path()
{
    std::array<char, 260> file_path;

    // Datum
    time_t t = std::time( nullptr );
    tm *ltm = localtime( &t );

    // Dateiname
    snprintf( file_path.data(), file_path.size() - 1, LOG_FILENAME, m_log_check.m_base_path, m_log_check.m_sub_folder,
        ltm->tm_year % 100, ltm->tm_mon + 1, ltm->tm_mday, m_connection_id + 1000 );

    return file_path.data();
}

void CDFLog::check_current_file_paths()
{
    if ( get_current_log_file_path() != m_file_path )
    {
        // Dateiname hat sich geaendert.
        m_file_path = get_current_log_file_path();

        if ( cb_change_trace_file_path != nullptr )
        {
            // Setzen des Namens fuer Tracefile
            cb_change_trace_file_path( get_current_trace_file_path() );
        }

        logCheck(); // Eventuell die aelteste Logdatei loeschen.
    }
}

void CDFLog::logBegin( const char *Format, ... )
{
    if ( m_enabled == false )
    {
        return;
    }

    // Bereichssperre
    std::lock_guard<std::mutex> guard( m_mutex_file );

    char szTime[9];
    char szLine[0x400];
    va_list ap;
    FILE *Logfp;

    // Uhrzeit des Logeintrages
    time_t t = std::time( nullptr );
    m_clock_begin = std::clock();
    tm *ltm = std::localtime(&t);
    std::sprintf( szTime, "%02i:%02i:%02i", ltm->tm_hour, ltm->tm_min, ltm->tm_sec );

    // Prüfen der Dateinamen und ggf. anpassen.
    check_current_file_paths();

    std::sprintf( szLine, "%s\t", szTime );

    unsigned long pid = 0;
    std::stringstream ss_ids;
#if defined(_MSC_VER) || defined(__MINGW32__)
    pid = GetCurrentProcessId();
#else
    pid = getpid();
#endif
    ss_ids << pid << '\t' << std::this_thread::get_id() << '\t';
    // ProcessId und ThreadId einfuegen
    std::strcpy( &szLine[std::strlen( szLine )], ss_ids.str().c_str() );

    va_start( ap, Format );
    std::vsprintf( &szLine[std::strlen( szLine )], Format, ap );
    va_end( ap );

    if ( (Logfp = std::fopen( m_file_path.c_str(), "a" )) == (FILE *)0 )
    {
        return;
    }

    std::fwrite( szLine, 1, std::strlen( szLine ), Logfp );

    std::fclose( Logfp );
}

void CDFLog::logAppend( const char *Format, ... )
{
    if ( m_enabled == false )
    {
        return;
    }

    // Bereichssperre
    std::lock_guard<std::mutex> guard( m_mutex_file );

    va_list ap;
    FILE *Logfp;

    if ( (Logfp = std::fopen( m_file_path.c_str(), "a")) == (FILE *)0 )
    {
        return;
    }

    va_start( ap, Format );
    std::vfprintf( Logfp, Format, ap );
    va_end( ap );

    std::fclose( Logfp );
}

void CDFLog::logEnd(const char *Format, ...)
{
    if ( m_enabled == false )
    {
        return;
    }

    // Bereichssperre
    std::lock_guard<std::mutex> guard( m_mutex_file );

    va_list ap;
    FILE *Logfp;

    if ( (Logfp = std::fopen( m_file_path.c_str(), "a")) == (FILE *)0 )
    {
        return;
    }

    va_start( ap, Format );
    std::vfprintf( Logfp, Format, ap );
    va_end( ap );

    // Ausfuehrungszeit angeben?
    if ( m_elapse )
    {
        // Ausfuehrungszeit berechnen
        double clockDiff;
        clockDiff = (double)(std::clock() - m_clock_begin) / CLOCKS_PER_SEC;
#if defined(_MSC_VER) || defined(__MINGW32__)
        std::fprintf( Logfp, "\t[%2.1f]\n", clockDiff );
#else
        std::fprintf( Logfp, "\t[%2.1f]\r\n", clockDiff );
#endif
    }
    else
    {
#if defined(_MSC_VER) || defined(__MINGW32__)
        std::fprintf( Logfp, "\n" );
#else
        std::fprintf( Logfp, "\r\n" );
#endif
    }

    std::fclose( Logfp );
}

#if defined(_MSC_VER) || defined(__MINGW32__)
void CDFLog::MakeFilePath( char *path_buffer, char *szFileName, char *szFileExt )
{
    char drive[_MAX_DRIVE];
    char dir[_MAX_DIR];
    char fname[_MAX_FNAME];
    char ext[_MAX_EXT];

    // Speicher initialisieren
    std::memset( drive, 0, sizeof( drive ) );
    std::memset( dir, 0, sizeof( dir ) );
    std::memset( fname, 0, sizeof( fname ) );
    std::memset( ext, 0, sizeof( ext ) );

    _splitpath( path_buffer, drive, dir, fname, ext );
    // Wenn keine Extension, dann gehoert eventueller Dateiname noch zum Pfad
    if ( !std::strlen( ext ) )
    {
        if ( std::strlen( fname ) )
        {
            std::strcat( path_buffer, "\\" );
            // Neu auswerten
            _splitpath( path_buffer, drive, dir, fname, ext );
        }
    }

    // Speicher loeschen
    std::memset( path_buffer, 0, _MAX_PATH );

    _makepath( path_buffer, drive, dir, (szFileName) ? szFileName : fname, (szFileExt) ? szFileExt : ext );
}
#endif


